//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "waveformdecodersine.h"

#ifdef FIT_SINE

WaveformDecoderSine::WaveformDecoderSine ()
    : WaveformDecoder ()
{
    fitterSine = new FitterSine(MAX_FITSINE_POINTS);
}
WaveformDecoderSine::~WaveformDecoderSine ()
{
    delete fitterSine;
}

int WaveformDecoderSine::find_sine (const Cfg &cfg, struct adcm_ev_t * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(cfg);
    Q_ASSERT(ev);

    int fit_ok = 1;
    if (!cfg || !ev || !ws) {
        LOG ("adcm_ev_find_sine: Bad input param\n");
        return (-EINVAL);
    }

    for (int c = 0; c < ADC_NCH; c++)
    {
        adcm_chbuf_t *evc = ev->get_ev_chbuf(c);
        if (!evc) continue;
        const wbc_t *wb = &ws->wb[c];
        if (wb->len <=0)
            continue;

        {
            //    double chi2=0, niter=0;
            {
                int ret = 0;
                double par[4]={0,0,0,0};
                double par0[4]={0,0,0,0};
                fitstatus_t fitstatus;
                memset(&fitstatus, 0, sizeof(fitstatus_t));
                fitstatus.niter = get_dec_fititer(cfg);

                //        ret = fitterGaus->fit (get_dec_fitfixpar(cfg), fb->d, fb->len, par, &fitstatus);
                double x[MAX_FITSINE_POINTS];
                int len = wb->len;
                if (len > MAX_FITSINE_POINTS)
                    len = MAX_FITSINE_POINTS;
                for(int i=0;i<len; i++)
                    x[i] = wb->d[i];
                ret = fitterSine->fit (0, x, len, par0, par, &fitstatus);
                if (ret != 0) {
                    if (decst)
                        decst->dsp_fit_fail++;
                    print_decoder_dsp_err (decst, "fit failed, ch=%d, code=%d", c, ret);
                    evc->flags |= ADCM_CH_TFIT_ERR;
                    fit_ok = 0;
                } else {
                    evc->fasttime = par0[3];
                    evc->dc = par[0];
                    evc->pa = par[1];
                    evc->pt = par[2];
                    //  		  if (par[3]==0) par[3]=0.000001; // fix 1/0 problem
                    //  		  evc->pw = 2.*M_PI/par[3];
                    evc->pw = par[3];
                    //                  chi2 = fitstatus.fnorm;
                    //                  niter = fitstatus.niter;
                    // f(x) = p0 + p1 * sin(p2 + p3*x)
                    //      	  printf("fit result: f(x) = %e + %e * sin(%e + %e * x)\n", par[0], par[1], par[2], par[3]);
                }
            }

            //    log_debug (3, " * ch=%d, ip=%d, a=%g, t=%.2f, w=%.2f\n", c, ip, a, t, w);
            if (fit_ok) {
                if (decst)
                    decst->dsp_sine_ok++;
                //      log_debug(3, " + ch %d: t=%.2f", c, t);
            } else {
                //  	  evc->flags |= F_SINE_ERR; // FIXME
            }
            // debug data
            //printf("pp=%d, fasttime=%f, t=%f\n", ev->ch[c].pp, ev->ch[c].fasttime, evc->pt);
            //    evc->pchi2 = chi2;
            //    evc->pniter = niter;

            //    if (fit_ok) evc->flags |= ADCM_CH_EVAL_SINE; // FIXME

        }
    }

    return (0);
}

void WaveformDecoderSine::decode_work (const Cfg &cfg, struct adcm_ev_t * ev, struct adcm_decstat_t * decst,
                                       decst_t_t &pt, ftimer_t &t)
{
    WaveformDecoder::decode_work(cfg, ev, decst, pt, t);

    if (stages & ADCM_CH_EVAL_SINE)
        find_sine (cfg, ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_sine);
}

#endif /* FIT_SINE */
