/*
**    Copyright 2019 Timur Gandzhelashvili
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Adc64ConfigDb.h"

#include "Adc64Config.h"
#include "dominosettings.h"

#include "Adc64ConfigConverter.h"

Adc64ConfigDb::Adc64ConfigDb(const ConfigSelector &_cs, QObject *parent)
    : AbstractConfigDb(_cs, parent)
{
    AbstractConfigConverter<Adc64Config>::registerTypes();
}

void Adc64ConfigDb::setConfig(const Adc64Config &newConfig)
{
    if(!initialized)
        return;
    if (newConfig == config)
        return;
    config = newConfig;

    auto json = QVariant::fromValue(config).value<QJsonObject>();
    BaseConfig::write(cs, json);
}

void Adc64ConfigDb::update_config_from_db()
{
    auto json = read(cs);
    config = QVariant::fromValue(json).value<Adc64Config>();
    config.setRun(false);
    config.blockSparseSetup();
    emit configChanged(config);
}
