/*
**    Copyright 2019 Timur Gandzhelashvili
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Adc64SystemConfigDb.h"

#include <QDebug>
#include <QJsonDocument>
#include <QStandardPaths>

#include "Adc64ConfigConverter.h"
#include "Adc64SystemConfig.h"
#include "Adc64Config.h"
#include "dominosettings.h"

static const char *DEVICE = "device";

Adc64SystemConfigDb::Adc64SystemConfigDb(const ConfigSelector &cs, QObject *parent)
    : AbstractConfigDb(cs, parent),
      config(new Adc64SystemConfig)
{
    AbstractConfigConverter<Adc64Config>::registerTypes();
    AbstractConfigConverter<Adc64SystemConfig>::registerTypes();
}

Adc64SystemConfigDb::~Adc64SystemConfigDb()
{
    delete config;
}

void Adc64SystemConfigDb::setConfig(const Adc64SystemConfig &newConfig)
{
    if (newConfig == *config)
        return;
    *config = newConfig;

    auto json = QVariant::fromValue(*config).value<QJsonObject>();
    write_config(json);
}

void Adc64SystemConfigDb::update_config_from_db()
{
    const auto &json = BaseConfig::read(cs);
    *config = QVariant::fromValue(json).value<Adc64SystemConfig>();
    synchronize_with_adc64(config);
    config->setRun(false);
    emit configChanged(*config);
}


ConfigDbSyncResult Adc64SystemConfigDb::synchronize_with_adc64(Adc64SystemConfig *config)
{
    QMap<DeviceIndex, bool> to_return;
    for(const DeviceIndex &index : config->device_configs.keys()){
        to_return[index] = false;
    }

    const ConfigSelector adc64Cs(DaqModule(DaqModule::Type::Adc64), cs.getConfigIndexName());
    const auto &adc64_json = BaseConfig::read(adc64Cs);
    const auto &adc64_cfg = QVariant::fromValue(adc64_json).value<Adc64Config>();
    QMap<DeviceIndex, DominoSettings> device_configs_adc64;


    bool device_config_changed = false;
    if(!adc64_json.isEmpty()) {
        for(auto it=config->device_configs.begin(); it!=config->device_configs.end(); ++it) {
            auto dev_it = adc64_cfg.device_configs.find(it.key());
            bool found = dev_it != adc64_cfg.device_configs.end();
            device_config_changed |= found;

            if(found) {
                bool en_bak = it->enabled;
                it.value() = dev_it.value();
                it->enabled = en_bak;
            }
            to_return[it.key()] = found;
        }

    }

    Adc64SystemConfig::setMStreamSetup(config);
    emit sync_result(to_return);
    return to_return;
}
