/*
**    Copyright 2019 Timur Gandzhelashvili
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64SYSTEMCONFIGDB_H
#define ADC64SYSTEMCONFIGDB_H

#include <QMap>

#include "base/DeviceIndex.h"
#include "mongo/AbstractConfigDb.h"

class Adc64SystemConfig;

using ConfigDbSyncResult = QMap<DeviceIndex, bool>;

class Adc64SystemConfigDb : public AbstractConfigDb
{
    Q_OBJECT

public:
    Adc64SystemConfigDb(const ConfigSelector &cs, QObject *parent);
    ~Adc64SystemConfigDb() override;

    ConfigDbSyncResult synchronize_with_adc64(Adc64SystemConfig *config);

signals:
    void configChanged(const Adc64SystemConfig &) const;
    void sync_result(const ConfigDbSyncResult &) const;

public slots:
    void setConfig(const Adc64SystemConfig &newConfig);

protected:
    void update_config_from_db() override final;

private:
    Adc64SystemConfig * const config;
};

#endif // ADC64SYSTEMCONFIGDB_H
