/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AdcDeviceProxy.h"

#include <QHostAddress>

#include "dominodevice.h"

Q_DECLARE_METATYPE(QHostAddress)

AdcDeviceProxy::AdcDeviceProxy(const DeviceIndex &index, DominoDevice *device, QObject *parent)
    : QObject(parent),
      index(index),
      dev(device)
{
    qRegisterMetaType<QHostAddress>();
    connect(this, &AdcDeviceProxy::proxySetHostAddress,
            device, &DominoDevice::setHostAddress,
            Qt::QueuedConnection);
    connect(device, &DominoDevice::deviceConnected,
            this, &AdcDeviceProxy::deviceConnected,
            Qt::QueuedConnection);
    connect(device, &DominoDevice::deviceDisconnected,
            this, &AdcDeviceProxy::deviceDisconnected,
            Qt::QueuedConnection);
}

void AdcDeviceProxy::setHostAddress(const DeviceIndex &index, const QHostAddress &address)
{
    emit proxySetHostAddress(index, address);
}

void AdcDeviceProxy::setDeleteFlag()
{
    dev->setDeleteFlag();
}
