/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCDEVICEPROXY_H
#define ADCDEVICEPROXY_H

#include <QObject>

#include "base/DeviceIndex.h"

class DominoDevice;
class QHostAddress;

class AdcDeviceProxy : public QObject
{
    Q_OBJECT
public:
    AdcDeviceProxy(const DeviceIndex &index, DominoDevice *device, QObject *parent = nullptr);
    DeviceIndex getIndex() const { return index; }
    void setHostAddress(const DeviceIndex &index, const QHostAddress &address);
    bool isConnected() const { return connected; }
    void setDeleteFlag();

signals:
    void proxySetHostAddress(const DeviceIndex &index, const QHostAddress &address); // TODO: hide impl

private slots:
    void deviceConnected(const DeviceIndex &index) {connected = true; }
    void deviceDisconnected(const DeviceIndex &index) {connected = false; }

private:
    const DeviceIndex index;
    bool connected = false;
    DominoDevice *const dev;
};

#endif // ADCDEVICEPROXY_H
