/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AdcStatusTableModel.h"
#include "Adc64SystemConfig.h"
#include "base/DeviceIndexEnabled.h"
#include "device-discover/DeviceTableModel.h"

AdcStatusTableModel::AdcStatusTableModel(QObject *parent)
    : MergeColumnsTableModel(parent)
    , config(new Adc64SystemConfig)
    , modelA(new DeviceTableModel(this))
    , modelB(new DeviceTableModel(this))
{
    modelA->setObjectName("modelA");
    modelB->setObjectName("modelB");
    connect(this, &AdcStatusTableModel::lldpCacheUpdated,
            modelA, &DeviceTableModel::lldpCacheUpdated);
    connect(this, &AdcStatusTableModel::lldpCacheUpdated,
            modelB, &DeviceTableModel::lldpCacheUpdated);

    setSourceModelA(modelA);
    setSourceModelB(modelB);
}

AdcStatusTableModel::~AdcStatusTableModel()
{
    delete config;
}
