/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AdcStatusWidget.h"

#include "AdcStatusTableModel.h"
#include "Adc64SystemConfig.h"
#include "device-discover/DeviceIndexFilterProxyModel.h"

AdcStatusWidget::AdcStatusWidget(QWidget *parent)
    : QTableView(parent)
    , model(new AdcStatusTableModel(this))
    , indexFilter(new DeviceIndexFilterProxyModel(this))
{
    connect(this, &AdcStatusWidget::setDeviceList,
            model, &AdcStatusTableModel::setDeviceList);

    connect(this, &AdcStatusWidget::lldpCacheUpdated,
            model, &AdcStatusTableModel::lldpCacheUpdated);

    indexFilter->setSourceModel(model);

//    setModel(indexFilter);
    setModel(model);
}

void AdcStatusWidget::setConfig(const Adc64SystemConfig &config)
{
    const auto &indices = config.device_configs.keys();
    const DeviceIndexSet indexSet(makeDeviceIndexSet(indices));
    indexFilter->setFilter(indexSet);
}
