/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64CONFIG_H
#define ADC64CONFIG_H

#include "dominosettings.h"
#include "base/DeviceIndex.h"

#include <QMap>
#include <QMetaType>
#include <QString>

class Adc64Config
{
public:
    Adc64Config() = default;
    void blockSparseSetup();
    void setRun(bool val);
    bool isRunning() const { return run; }
    void setShowSparseMarkers(bool val) { showSparseMarkers = val; }
    bool isShowSparseMarkers() const { return showSparseMarkers; }
    void setShowPcbCh(bool val) { showPcbCh = val; }
    bool isShowPcbCh() const { return showPcbCh; }
    void setPeriodicalAdcDelayCheck(bool val);
    bool isPeriodicalAdcDelayCheck() const { return periodicalAdcDelayCheck; }

    bool operator==(const Adc64Config &r) const;
    bool operator!=(const Adc64Config &r) const { return !operator==(r); }

    bool preamplifier = false; // move to DominoSettings
    bool run = false;
    bool showSparseMarkers = true;
    bool showPcbCh = false;
    bool periodicalAdcDelayCheck = false;

    QString adcDelayResultDir;
    QMap<QString, FirPreset> fir_map;
    QMap<DeviceIndex, DominoSettings> device_configs;
};

Q_DECLARE_METATYPE(Adc64Config)

#endif // ADC64CONFIG_H
