/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Adc64ConfigDispatcher.h"

#include "Adc64Config.h"

Adc64ConfigDispatcher::Adc64ConfigDispatcher(QObject *parent)
    : BaseConfigDispatcher(parent),
      cache(new Adc64Config())
{
    qRegisterMetaType<Adc64Config>();
}

Adc64ConfigDispatcher::~Adc64ConfigDispatcher()
{
    delete cache;
}

void Adc64ConfigDispatcher::setConfig(const Adc64Config &config)
{
    if (config == *cache)
        return;
    *cache = config;
    qDebug() << __PRETTY_FUNCTION__ << "config changed for " << cache->device_configs.keys();
    emit configChanged(config);
    for(auto dc : cache->device_configs)
        emit devSerDesDelayChanged(dc.devIndex, dc.serDesDelayPreset);
}

void Adc64ConfigDispatcher::setDevSerDesDelay(const DeviceIndex &index, const DevSerDesDelay &delay)
{
    auto it = cache->device_configs.find(index);
    if(it == cache->device_configs.end())
        return;
    if(it.value().serDesDelayPreset == delay)
        return;
    it.value().serDesDelayPreset = delay;
    emit configChanged(*cache);
    emit devSerDesDelayChanged(index, delay);
}

void Adc64ConfigDispatcher::defaultSerDesDelay(const DeviceIndex &index, const DevSerDesDelay &delay) const
{
    auto it = cache->device_configs.find(index);
    if(it == cache->device_configs.end())
        return;
    auto delayCandidat = delay;
    delayCandidat.insert(it.value().serDesDelayPreset);
    if(it.value().serDesDelayPreset == delayCandidat)
        return;

    it.value().serDesDelayPreset = delayCandidat;
    emit configChanged(*cache);
    emit devSerDesDelayChanged(index, delayCandidat);
}
