/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64CONFIGDISPATCHER_H
#define ADC64CONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"
#include "device-modules/DeserializersControl/AdcSerDesDelay.h"

class Adc64Config;
class DeviceIndex;

class Adc64ConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT

public:
    Adc64ConfigDispatcher(QObject *parent = nullptr);
    virtual ~Adc64ConfigDispatcher() override;

    virtual void sendCache() const override {
        BaseConfigDispatcher::sendCache();
        emit configChanged(*cache);
    }

    template<class T>
    void subscribe(T *obj) {
        connect(obj, &T::configChanged,
                this, &Adc64ConfigDispatcher::setConfig);
        connect(this, &Adc64ConfigDispatcher::configChanged,
                obj, &T::setConfig);
        qDebug() << "Adc64Config:"
                 << metaObject()->className()
                 << "<->"
                 << obj->metaObject()->className();
    }

    template<class T>
    void subscribeSerDesDelay(T *obj) {
        connect(obj, &T::devSerDesDelayChanged,
                this, &Adc64ConfigDispatcher::setDevSerDesDelay);
        connect(this, &Adc64ConfigDispatcher::devSerDesDelayChanged,
                obj, &T::setDevSerDesDelay);
        qDebug() << "Adc64Config:"
                 << metaObject()->className()
                 << "<->"
                 << obj->metaObject()->className();
    }

    // Adc64Config
signals: void configChanged(const Adc64Config &) const;
public slots: void setConfig(const Adc64Config &config);

    // DevSerDesDelay
signals: void devSerDesDelayChanged(const DeviceIndex &index,
                                    const DevSerDesDelay &delay) const;
public slots: void setDevSerDesDelay(const DeviceIndex &index,
                                     const DevSerDesDelay &delay);
    void defaultSerDesDelay(const DeviceIndex &index,
                            const DevSerDesDelay &delay) const;

private:
    Adc64Config * const cache;
};

#endif // ADC64CONFIGDISPATCHER_H
