/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Adc64SystemConfig.h"

#include <QDebug>
#include <QJsonObject>
#include <QJsonDocument>

bool Adc64SystemConfig::operator==(const Adc64SystemConfig &r) const
{
    return true &&
            run == r.run &&
            allow_remote_control == r.allow_remote_control &&
            write_enable == r.write_enable &&
            storage_path == r.storage_path &&
            show_details == r.show_details &&
            event_number_limit == r.event_number_limit &&
            mtu_size == r.mtu_size &&
            trig_on_xoff_stop == r.trig_on_xoff_stop &&
            device_configs == r.device_configs;
}

void Adc64SystemConfig::setMStreamSetup(Adc64SystemConfig *config)
{
    for (DeviceIndex index : config->device_configs.keys()) {
        DominoSettings &ds = config->device_configs[index];
        ds.mstreamEnable = true;
        ds.run = false;
    }
}

void Adc64SystemConfig::setRun(bool val)
{
    run = val;

    for(auto it=device_configs.begin(); it!=device_configs.end(); ++it)
        it->run = it->enabled && run;
}

bool Adc64SystemConfig::hasEnabledDevice() const
{
    for(const auto &ds : device_configs)
        if(ds.enabled)
            return true;
    return false;
}

QDebug operator <<(QDebug stream, const Adc64SystemConfig &trig)
{
    stream.noquote().nospace()
            << "Adc64SystemConfig("
            << QString("run: %1, ").arg(trig.run)
            << QString("allow_remote_control: %1, ").arg(trig.allow_remote_control)
            << QString("write_enable: %1, ").arg(trig.write_enable)
            << QString("storage_path: %1, ").arg(trig.storage_path)
            << QString("show_details: %1, ").arg(trig.show_details)
            << QString("event_number_limit: %1, ").arg(trig.event_number_limit)
            << QString("mtu_size: %1, ").arg(trig.mtu_size)
            << QString("trig_on_xoff_stop: %1, ").arg(trig.trig_on_xoff_stop)
            << QString("device_configs: {");
    for(auto it=trig.device_configs.cbegin(); it!=trig.device_configs.cend(); ++it) {
        auto val = QVariant::fromValue(it.value()).value<QJsonObject>();

        stream.noquote().nospace()<<it.key()<<": DominoSettings("<<QJsonDocument(val).toJson(QJsonDocument::Compact)<<")";
    }
    stream.noquote().nospace()<< "})";
    return stream;
}
