/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64SYSTEMCONFIG_H
#define ADC64SYSTEMCONFIG_H

#include "dominosettings.h"
#include "base/DeviceIndex.h"

#include <QMap>
#include <QMetaType>
#include <QString>
class QDebug;

class Adc64SystemConfig
{
public:
    Adc64SystemConfig() = default;
    Adc64SystemConfig(const Adc64SystemConfig &r) = default;
    bool operator==(const Adc64SystemConfig &r) const;
    static void setMStreamSetup(Adc64SystemConfig *config);
    void setRun(bool val);
    bool hasEnabledDevice() const;

    bool run = false;
    bool allow_remote_control = true;
    bool write_enable = false;
    QString storage_path;
    bool show_details = false;
    quint64 event_number_limit = 0;
    quint16 mtu_size = 1500; // FIXME
    bool trig_on_xoff_stop = true;
    QMap<DeviceIndex, DominoSettings> device_configs;
};

Q_DECLARE_METATYPE(Adc64SystemConfig)

QDebug operator <<(QDebug stream, const Adc64SystemConfig &cfg);

#endif // ADC64SYSTEMCONFIG_H
