/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64SYSTEMCONFIGDISPATCHER_H
#define ADC64SYSTEMCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"

class Adc64SystemConfig;

class Adc64SystemConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT

public:
    Adc64SystemConfigDispatcher(QObject *parent = nullptr);
    virtual ~Adc64SystemConfigDispatcher() override;

    virtual void sendCache() const override {
        BaseConfigDispatcher::sendCache();
        emit configChanged(*cache);
    }

    template<class T>
    void subscribe(T *obj) {
        connect(obj, &T::configChanged,
                this, &Adc64SystemConfigDispatcher::setConfig);
        connect(this, &Adc64SystemConfigDispatcher::configChanged,
                obj, &T::setConfig);
        qDebug() << "Adc64SystemConfig:"
                 << metaObject()->className()
                 << "<->"
                 << obj->metaObject()->className();
    }

signals:
    void configChanged(const Adc64SystemConfig &) const;

public slots:
    void setConfig(const Adc64SystemConfig &config);

private:
    Adc64SystemConfig * const cache;
};

#endif // ADC64SYSTEMCONFIGDISPATCHER_H
