#include "Adc64SystemSignalRouter.h"

#include "Adc64SystemConfigDispatcher.h"
#include "MainWindow.h"
#include "remote-control-server/RemoteControlServer.h"
#include "remote-control-server/EvNumChecker.h"
#include "pnp-server/PNPServer.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void Adc64SystemSignalRouter::connectAll() const
{
    connect_ConfigurationDispatcher();
    connect_MainWindow();
    connect_PnpServer();
    connect_RemoteControlServer();
}

void Adc64SystemSignalRouter::connect_ConfigurationDispatcher() const
{
    if(!configDispatcher)
        return;
    if(mainWindow){
        configDispatcher->subscribe(mainWindow);
        configDispatcher->subscribeCS(mainWindow);
    }
    if(configurationManager)
        configDispatcher->subscribeCS(configurationManager);
    if(configDb) {
        configDispatcher->subscribe(configDb);
        configDispatcher->subscribeCS(configDb);
    }
    connect(configDb, &Adc64SystemConfigDb::sync_result,
            mainWindow, &MainWindow::configDbSyncResult);
}

void Adc64SystemSignalRouter::connect_MainWindow() const
{
    if(configurationManager && mainWindow)
        connect(configurationManager, &ConfigurationManager::switch_to_config,
                mainWindow, &MainWindow::configSelectorChanged);
}

void Adc64SystemSignalRouter::connect_PnpServer() const
{
    if(!mainWindow || !pnpServer)
        return;

    connect(mainWindow, &MainWindow::programDescriptionUpdated,
            pnpServer, &PNPServer::updateProgramDescription);
    connect(mainWindow, &MainWindow::closeProgram,
            pnpServer, &PNPServer::closeProgram);
    connect(mainWindow, &MainWindow::deleteAll,
            pnpServer, &PNPServer::deleteLater);
}

void Adc64SystemSignalRouter::connect_RemoteControlServer() const
{
    if(evNumChecker && mainWindow)
        connect(evNumChecker, &EvNumChecker::checkFail,
                mainWindow, &MainWindow::sendRunError);

    if(!remoteControl)
        return;
    if(mainWindow) {
        connect(remoteControl, &RemoteControlServer::startRequest,
                mainWindow, &MainWindow::onRemoteStartRequest);
        connect(remoteControl, &RemoteControlServer::stopRequest,
                mainWindow, &MainWindow::onRemoteStopRequest);
        connect(mainWindow, &MainWindow::sendRemoteState,
                remoteControl, &RemoteControlServer::sendState);
        connect(remoteControl, &RemoteControlServer::connectionStatusUpdated,
                mainWindow, &MainWindow::remoteStatusUpdated);
        connect(mainWindow, &MainWindow::deleteAll,
                remoteControl, &RemoteControlServer::deleteLater);
        connect(mainWindow, &MainWindow::runError,
                remoteControl, &RemoteControlServer::onRunError);
        connect(mainWindow, &MainWindow::runWarning,
                remoteControl, &RemoteControlServer::onRunWarning);
        connect(mainWindow, &MainWindow::allowRemoteControl,
                remoteControl, &RemoteControlServer::startListen);
        connect(mainWindow, &MainWindow::config_transfer_to_rc,
                remoteControl, &RemoteControlServer::config_transfer);
        connect(mainWindow, &MainWindow::programDescriptionUpdated,
                remoteControl, &RemoteControlServer::changeProgramDescription);
    }

    if(evNumChecker) {
        connect(evNumChecker, &EvNumChecker::completedEvNumCheck,
                remoteControl, &RemoteControlServer::evNumCheckResponse);
        connect(remoteControl, &RemoteControlServer::evNumCheckRequest,
                evNumChecker, &EvNumChecker::onRemoteEvNumCheckRequest);
    }
}
