/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICETABLEWIDGET_H
#define DEVICETABLEWIDGET_H

#include <QTableWidget>

#include "Adc64SystemConfigDb.h"
#include "base/DeviceIndex.h"

class AdcStatus;
class Adc64SystemConfig;
class LldpInfo;
class LldpInfoCache;

class DeviceTableWidget : public QTableWidget
{
    Q_OBJECT

public:
    enum Columns { COLUMN_DEV_ID_STR, COLUMN_SERIAL_STR, COLUMN_SLOT, COLUMN_IP, COLUMN_TEMP, COLUMN_EVENT,
           COLUMN_TRIG_ON_XOFF, COLUMN_SYNCH_STATUS, COLUMN_ADC_STATE_OK,
           COLUMN_WR_TIME, COLUMN_WR_TIME_OFFSET,
           COLUMN_WR_LINK_ERROR, COLUMN_WR_SYNC_LOST, COLUMN_WR_RX_CODE_ERROR,
           COLUMN_MS_EV_RATE,COLUMN_MS_DATA_RATE,
           COLUMN_DEV_SER, COLUMN_DEV_ID,
           COLUMN_UPTIME, COLUMN_LM_RESET,
           COLUMN_NUMBER};

    explicit DeviceTableWidget(QWidget *parent = nullptr);
    ~DeviceTableWidget();
    void clear();
    void insertRow(const DeviceIndex &index);
    void removeRow(const DeviceIndex &index);

signals:
    void configChanged(const Adc64SystemConfig &config);

public slots:
    void setConfig(const Adc64SystemConfig &newConfig);
    void adcStateChanged(const DeviceIndex &index, bool online);
    void updateAdcStatus(const DeviceIndex &index, const AdcStatus &status);
    void lldpCacheUpdated(const LldpInfoCache &cache);
    void configDbSyncResult(const ConfigDbSyncResult &synchResult);

private slots:

protected:

private:
    void setTableLabels();
    DeviceIndex getIndexByRow(int row) const;
    int getRowByIndex(const DeviceIndex &index) const;
    void updateDeviceDiscoverInfo(const DeviceIndex &index, const LldpInfo &lldpInfo);
    void adjustTableHeight();

    Adc64SystemConfig * const config;

};

#endif // DEVICETABLEWIDGET_H
