#include "WrDaqDialog.h"
#include "ui_WrDaqDialog.h"

#include "Adc64SystemConfig.h"
#include "base/gui_utils.h"

WrDaqDialog::WrDaqDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::WrDaqDialog)
{
    ui->setupUi(this);
    restore_gui_values();
}

WrDaqDialog::~WrDaqDialog()
{
    delete ui;
}

void WrDaqDialog::accept()
{
    apply();
    QDialog::accept();
}

void WrDaqDialog::apply()
{
    emit configChanged(config);
}

void WrDaqDialog::restore_gui_values()
{
    MultiSignalBlocker msb(children());

    cbMap.clear();
    ui->comboBoxDevice->clear();
    const auto &dcs = config.device_configs;
    for (auto it = dcs.begin(); it != dcs.end(); ++it) {
        const DeviceIndex &deviceIndex = it.key();
        const DominoSettings &ds = it.value();
        int i = ui->comboBoxDevice->count();
        cbMap[i] = deviceIndex;
        ui->comboBoxDevice->insertItem(i, ds.devIndex.getIdent());
        if (deviceIndex == curIndex)
            ui->comboBoxDevice->setCurrentIndex(i);
    }
    if (! dcs.contains(curIndex)) {
        curIndex = cbMap[0];
        ui->comboBoxDevice->setCurrentIndex(0);
    }
    if (dcs.isEmpty())
        return;
    updateCurrentDevice();
}

void WrDaqDialog::setConfig(const Adc64SystemConfig &r)
{
    auto it = r.device_configs.find(curIndex);
    if (it == r.device_configs.end())
        return;

    auto r_ = r.device_configs[curIndex];
    auto config_ = config.device_configs[curIndex];

    bool dsAreSame = (config_.ad5622op == r_.ad5622op) &&
                     (config_.amp_board_set == r_.amp_board_set);
    config = r;
    if (dsAreSame)
        return;
    restore_gui_values();
}

DeviceIndex WrDaqDialog::getCurrentIndex() const
{
    int index = ui->comboBoxDevice->currentIndex();
    auto it = cbMap.find(index);
    if (it == cbMap.end())
        return DeviceIndex();
    return it.value();
}

void WrDaqDialog::on_comboBoxDevice_currentIndexChanged(int index)
{
    Q_UNUSED(index)
    curIndex = getCurrentIndex();
    updateCurrentDevice();
}

void WrDaqDialog::updateCurrentDevice()
{
    MultiSignalBlocker msb(children());
    const DominoSettings &setup = config.device_configs[getCurrentIndex()];

    if (setup.amp_board_set == AMP_BOARD_AD5622v2) {
        ui->radioButtonVer2->toggle();
    } else {
        ui->radioButtonVer1->toggle();
    }
    ui->spinBoxBaseLine->setValue(setup.ad5622op.data_baseline);
    ui->spinBoxLevel->setValue(setup.ad5622op.data_level);
}

void WrDaqDialog::on_spinBoxBaseLine_valueChanged(int baseline)
{
    auto &dcs = config.device_configs;
    auto it = dcs.find(curIndex);
    if (it == dcs.end())
        return;
    (*it).ad5622op.data_baseline = baseline;
}

void WrDaqDialog::on_spinBoxLevel_valueChanged(int level)
{
    auto &dcs = config.device_configs;
    auto it = dcs.find(curIndex);
    if (it == dcs.end())
        return;
    (*it).ad5622op.data_level = level;
}

void WrDaqDialog::on_radioButtonVer1_toggled(bool checked)
{
    auto &dcs = config.device_configs;
    auto it = dcs.find(curIndex);
    if (it == dcs.end())
        return;
    (*it).amp_board_set = checked ? AMP_BOARD_AD5622 : AMP_BOARD_AD5622v2;

}

void WrDaqDialog::on_radioButtonVer2_toggled(bool checked)
{
    on_radioButtonVer1_toggled(!checked);
}
