/*
**    ADC64 System Application
**
**    Copyright 2013-2015 AFI Electronics
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <exception>
#include <QApplication>
#include <QDebug>
#include <QSettings>
#include <QString>

#include "MainWindow.h"
#include "adc64-common/Adc64StartupOptions.h"
#include "Adc64SystemApp.h"
#include "base/app/RunGuard.h"
#include "mongo/PrintConfigMap.h"

int main (int argc, char **argv)
{
    Adc64SystemApp daqApp(argc, argv);
    daqApp.init();

    if(daqApp.hasConfigListOpt())
        printConfigMap(daqApp.getType());

    int ret = -1;
    try{
        const auto cs = daqApp.getConfigSelector();
        RunGuard::runSingleOrExit(cs);

        MainWindow w(cs);
        w.show();

        ret = daqApp.exec();
    } catch (std::exception &e) {
        qCritical() << "Exception: " << e.what ();
    } catch (...){
        qCritical() << "Unknown exception";
    }
    qInfo() << "Shutdown complete";
    return ret;
}
