#include "Adc64SignalRouter.h"

#include <QObject>

#include "AdcSerDesCalDialog/AdcSerDesCalDialog.h"
#include "configuration-manager/ConfigurationManager.h"
#include "device-discover/DiscoverDialog.h"
#include "mldiscover/DeviceDispatcher.h"
#include "mldiscover/LldpDiscovery.h"

#include "adc64-common/dominodevice.h"
#include "Adc64ConfigDb.h"
#include "Adc64ConfigDispatcher.h"
#include "CopyConfigDialog.h"
#include "DominoDeviceManager.h"
#include "mainwindow.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void Adc64SignalRouter::connectAll()
{
    connect_ConfigurationDispatcher();
    connect_DominoDeviceManager();
    connect_DeviceDispatcher();
    connect_lldp();
    connect_CopyConfigDialog();
    if(adcSerDesCalDialog)
        connect(adcSerDesCalDialog, &AdcSerDesCalDialog::requestCalibration,
                mainWindow, &MainWindow::findAdcClockDelay);

}

void Adc64SignalRouter::connect_DominoDevice(const DominoDevice *dev)
{
    if(!dev)
        return;
    if(adcSerDesCalDialog) {
        connect(dev, &DominoDevice::adcDesCountUpdated,
                adcSerDesCalDialog, &AdcSerDesCalDialog::adcDesCountUpdated);
        connect(dev, &DominoDevice::serDesCalProgressUpdated,
                adcSerDesCalDialog, &AdcSerDesCalDialog::progressUpdated);
    }
    if(configDispatcher)
        connect(dev, &DominoDevice::defaultSerDesDelay,
                configDispatcher, &Adc64ConfigDispatcher::defaultSerDesDelay);
}

void Adc64SignalRouter::connect_ConfigurationDispatcher()
{
    if(!configDispatcher)
        return;
    if(configurationManager)
        configDispatcher->subscribeCS(configurationManager);
    if(mainWindow) {
        configDispatcher->subscribe(mainWindow);
        configDispatcher->subscribeCS(mainWindow);
    }
    if(configDb) {
        configDispatcher->subscribe(configDb);
        configDispatcher->subscribeCS(configDb);
    }
    if(copyConfigDialog)
        configDispatcher->subscribe(copyConfigDialog);
    if(adcSerDesCalDialog)
        configDispatcher->subscribeSerDesDelay(adcSerDesCalDialog);
    if(deviceMgr)
        configDispatcher->subscribeSerDesDelay(deviceMgr);
}

void Adc64SignalRouter::connect_DominoDeviceManager()
{
    if(!deviceMgr || !mainWindow)
        return;

    connect(mainWindow, &MainWindow::clearEventNumber,
            deviceMgr, &DominoDeviceManager::clearFullEventNumber);
    connect(mainWindow, &MainWindow::setupChangedSignal,
            deviceMgr, &DominoDeviceManager::setupChanged);
    connect(deviceMgr, &DominoDeviceManager::deviceConnected,
            mainWindow, &MainWindow::onDeviceConnect);
    connect(deviceMgr, &DominoDeviceManager::deviceDisconnected,
            mainWindow, &MainWindow::onDeviceDisconnect);
    connect(deviceMgr, &DominoDeviceManager::deviceInfoUpdated,
            mainWindow, &MainWindow::deviceInfoUpdated);
    connect(deviceMgr, &DominoDeviceManager::monitorEventChanged,
            mainWindow, &MainWindow::onMonitorEvent);

//    connect(mainWindow, &MainWindow::setDeviceIdFilter,
//            discoverDialog, &DiscoverDialog::setDeviceIdFilter);
}

void Adc64SignalRouter::connect_DeviceDispatcher()
{
    if(!deviceDispatcher)
        return;
    if(deviceMgr) {
        connect(deviceDispatcher, &DeviceDispatcher::deviceListChanged,
                deviceMgr, &DominoDeviceManager::setDevices);
        connect(deviceDispatcher, &DeviceDispatcher::setHostAddress,
                deviceMgr, &DominoDeviceManager::setHostAddress);
    }
    if(mainWindow) {
        // DeviceDispatcher <=> MainWindow
        deviceDispatcher->subscribeDevices(mainWindow);
    }
    if(discoverDialog) {
        // DeviceDispatcher <=> DiscoverDialog
        deviceDispatcher->subscribeDevices(discoverDialog);
    }
    if(adcSerDesCalDialog)
        deviceDispatcher->subscribeDevices(adcSerDesCalDialog);
}

void Adc64SignalRouter::connect_lldp()
{
    auto &lldpDicovery = LldpDiscovery::instance();
    if(deviceDispatcher)
        lldpDicovery.subscribe(deviceDispatcher);
    if(discoverDialog)
        lldpDicovery.subscribe(discoverDialog);
//    if(mainWindow)
//        lldpDicovery.subscribe(mainWindow);

}

void Adc64SignalRouter::connect_CopyConfigDialog()
{
    if(!mainWindow || !copyConfigDialog)
        return;
    connect(mainWindow, &MainWindow::selectedDeviceChanged,
            copyConfigDialog, &CopyConfigDialog::setSelectedDevice);
}
