/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICETABLEWIDGET_H
#define DEVICETABLEWIDGET_H

#include <QMap>
#include <QWidget>

#include "base/DeviceIndex.h"

namespace Ui {
class DeviceTableWidget;
}

class AdcStatus;
class DeviceIndexEnabledMap;

class DeviceTableWidget : public QWidget
{
    Q_OBJECT

public:
    enum {COL_SERIAL, COL_ID, COL_ID_STR, COL_SERIAL_STR, COL_TEMP, COL_WR_TIME,
          COL_FW, COL_CH,
          COL_STATUS, COL_ADC_LOCK, COL_ADC_TEST, COL_ADC_DELAY_CHECK,
          COL_TOTAL};

    explicit DeviceTableWidget(QWidget *parent = nullptr);
    ~DeviceTableWidget();
    void initTable() const;
    int getDeviceRow(const DeviceIndex &index) const;

signals:
    void selectedDeviceChanged(const DeviceIndex &);
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &devices_);
    void deviceStatusUpdated(const DeviceIndex &index, const QString &s) const;
    void adcLockStatusUpdated(const DeviceIndex &index, bool ok, quint16 locked) const;
    void adcRampStatusUpdated(const DeviceIndex &index, bool ok, quint8 rampError) const;
    void adcDelayCheckUpdated(const DeviceIndex &index,
                              QSet<int> failedAdcIdNull,
                              QSet<int> failedAdcId) const;
    void updateAdcStatus(const DeviceIndex &index, const AdcStatus &status);
    void onDeviceConnect(const DeviceIndex &i);
    void onDeviceDisconnect(const DeviceIndex &i);
    void updateChannelOffset(const QMap<DeviceIndex, QPair<int, int> > &map);

protected slots:
    void on_tableWidgetDevices_itemSelectionChanged();

private:
    DeviceIndex getDeviceIndex(int row) const;
    void updateTableSize() const;
    void addDeviceRow(const DeviceIndex &index);
    void removeDevices(const DeviceIndexSet &setToDel);
    void updateWrTimeCol() const;

    Ui::DeviceTableWidget * const ui;
    DeviceIndex selectedDevice;
    DeviceIndexSet curDeviceSet;
    QMap<DeviceIndex, QPair<int, int> > devChRange;
};

#endif // DEVICETABLEWIDGET_H
