//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DOMINODEVICEMANAGER_H
#define DOMINODEVICEMANAGER_H

#include <QList>
#include <QMap>
#include <QObject>
#include <QSet>
#include <QSharedPointer>
#include <QVariant>

#include "adc64-common/AdcStatus.h"
#include "base/DeviceIndex.h"
//#include "DeviceEntry.h"
#include "device-modules/DeserializersControl/AdcSerDesDelay.h"

class AdcDeviceProxy;
class DeviceIndexEnabledMap;
class DominoDevice;
class QHostAddress;
class DominoSettings;
class EventData;
class DominoData;
class QTimer;
class Adc64Config;

//class DeviceDatabase : public QMap<QString, DeviceEntry> {};

class DominoDeviceManager : public QObject
{
    Q_OBJECT
public:
    explicit DominoDeviceManager(QObject *parent = nullptr);
    ~DominoDeviceManager() override;
//    DeviceProperties getProperties() const;
    bool isAllConnected() const;
//    QtMregDevice::DeviceId getDeviceId() const;
    QString getDeviceIdStr() const;
    DeviceIdSet getSupportedDevices() const { return supportedDevices; }

signals:
    void setupChanged(const DominoSettings &);
    void deviceConnected(const DeviceIndex &);
    void deviceDisconnected(const DeviceIndex &i);
    void deviceTempChanged(const DeviceIndex &, double value);
//    void devicePropertiesChanged(const DeviceProperties &);
    void statusUpdated(const DeviceIndex &, const AdcStatus &);

    void tempChanged(const DeviceIndex &, double);
    void deviceInfoUpdated(DeviceIndex,int,int,bool);
    void dataAcquired(const EventData &);
    void monitorEventChanged(const EventData &);
    void clearEventNumber();
    void deleteDevice(const DeviceIndex &);
    void deviceStatusUpdated(const DeviceIndex &, const QString &);
    void adcLockStatusUpdated(DeviceIndex, bool ok, quint16 locked);
    void adcRampStatusUpdated(DeviceIndex, bool ok, quint8 rampError);
    void adcClkDelayCheckUpdated(DeviceIndex, QSet<int>, QSet<int>);
    void startAdcClockDelay(bool hardMode);
    void hardPatternTest(bool en) const;

public slots:
    void setHostAddress(const DeviceIndex &index, const QHostAddress &address);
    void setDevices(const DeviceIndexEnabledMap &deviceList);
//    void clearEventNumber();
    void clearFullEventNumber();
    void findAdcClockDelay(bool hardMode, const QString &dirPath);

    // DevSerDesDelay
signals: void devSerDesDelayChanged(const DeviceIndex &index,
                                    const DevSerDesDelay &delay) const;
public slots: void setDevSerDesDelay(const DeviceIndex &,
                                     const DevSerDesDelay &) {}

private slots:
//    void onDeviceConnect(const DeviceIndex &index);
//    void onDeviceDisconnect(const DeviceIndex &index);
//    void onDeviceTempChanged(const DeviceIndex &index, double);
    void onDeviceDataAcquired(const DeviceIndex &index, const DominoData &dominoData);
    void eventTimeout();
    void adcClkDelayUpdated(const DeviceIndex &index,const AdcClkStatus &s);

private:
    void removeDevices(const DeviceIndexSet &setToDel);
    void maybeSendMonitorEvent(const EventData &eventData);

    // DeviceDatabase devices;
    QMap<DeviceIndex, QPointer<AdcDeviceProxy> > devices; // access to objects in non-gui thread
    QMap<DeviceIndex, QThread *> devThreads;
    DeviceIdSet supportedDevices;
//    DeviceProperties * const properties;
    EventData * const eventData;
    EventData * const monitorEvent;
    QTimer * const eventTimeoutTimer;
    QMap<DeviceIndex, AdcClkStatus> delayStatusMap;
    QString findAdcClockDir;
};

#endif // DOMINODEVICEMANAGER_H
