#include "Config.h"

#include <QSettings>

void Config::loadSettings()
{
    QSettings settings;

    bool ok;
    const int cnt = settings.beginReadArray("devices");
    quint64 devSerial;
    DeviceId devId;
    for(int arrId=0; arrId<cnt; ++arrId) {
        settings.setArrayIndex(arrId);

        QString hexStr = settings.value("serial").toString();
        quint64 serId = hexStr.toULongLong(&ok, 16);
        if(ok && serId) {
            devSerial = serId;
        } else
            continue;
        hexStr = settings.value("id").toString();
        quint16 id = hexStr.toUShort(&ok, 16);
        if(ok && id){
            devId = static_cast<DeviceId>(id);
        } else
            continue;

        bool enabled = settings.value("enabled", true).toBool();
        auto index = DeviceIndex(devId, devSerial);
        devices[index] = DeviceIndexEnabled(index, enabled);
    }
    settings.endArray();


    filePath = settings.value("filePath").toString();
    verifyAfterWrite = settings.value("verifyAfterWrite").toBool();
}

void Config::saveSettings()
{
    QSettings settings;

    settings.setValue("filePath", filePath);
    settings.setValue("verifyAfterWrite", verifyAfterWrite);
    settings.remove("devices");
    int arrId = 0;
    settings.beginWriteArray("devices");
    for(const auto &die : devices) {
        settings.setArrayIndex(arrId);
        settings.setValue("id", QString("0x%1")
                          .arg(die.index.getDevId(), 2,16,QChar('0')));
        settings.setValue("serial", QString("0x%1")
                          .arg(die.index.getSerial(),8,16,QChar('0')) );
        settings.setValue("enabled", die.enabled);
        ++arrId;
    }
    settings.endArray();
}
