#ifndef ETHFLASHDEV_H
#define ETHFLASHDEV_H

#include <QObject>

#include "base/DeviceIndex.h"
#include "flash-prog/FlashDev.h"
#include "mregdev/fw_version.h"

class DeviceDescription;
class DeviceIndex;
class QtMregDevice;
class LldpInfoCache;

class QHostAddress;

class EthFlashDev : public QObject, public FlashDev
{
    Q_OBJECT
public:
    EthFlashDev(DeviceIndex index);

signals:
    void progressFormatChanged(const DeviceIndex &index, QString format) const;
    void devConnected(const DeviceIndex &, QString, const fw_version_t &) const;
    void taskFinished(const DeviceIndex & index,uint id, bool isWrite, bool ok, bool isGold) const;
    void progressUpdated(const DeviceIndex & index, int p) const;
    void liveMagicFailed(const DeviceIndex & index) const;
    void deviceDisconnected() const;
    void setStatus(const DeviceIndex &index, QString) const;
    void TempChanged(const DeviceIndex &, double);
    void setHostAddress(const DeviceIndex &, const QHostAddress &);

public slots:
    void cacheUpdated(const LldpInfoCache &cache);

private slots:
    void on_devConnected();
    void disconnectFromHost(const DeviceIndex &index);
    void setDeviceEnabled(const DeviceIndex &index, bool en);
    void startDevWrite(uint id, const QByteArray &fileData, int headerSize, bool isGoldenImage=false, bool verify=true);
    void startDevVerify(uint id, const QByteArray &fileData, int headerSize, bool isGoldenImage=false);
    void startDevErase(uint id, bool isGoldenImage=false);

protected:
    virtual bool sectorErase(bool isGoldenImage, uint32_t sizeToErase);
    virtual bool writeFlash(const char *fileData, bool isGoldenImage=false);
    virtual bool verifyFlash(bool isGoldenImage, uint sizeToRead, const char *data);

    virtual bool isConnected() const;
    virtual quint16 devReadReg(int regAddr);
    virtual void devWriteReg(int regAddr, uint16_t data);
    virtual void devWriteReg32(int regAddr, uint32_t data);
    virtual bool devBlkWriteReg(uint count, const uint *addr, uint16_t *data);
    virtual bool devBlkReadReg(uint count, const uint *addr, uint16_t *data);

    virtual void sendProgress(int p);
    virtual std::string getPrefix() const;
    virtual uint getRegBase() const;

    virtual bool canReadFlashId() const;

private:
    bool is_ADC64VE_module() const;

    const DeviceIndex index;
    QtMregDevice * const dev;
    quint16 hw_maj_ver;
    bool writeInProgress;
};

#endif // ETHFLASHDEV_H
