#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QThread>

#include "base/app/BaseMainWindow.h"
#include "Config.h"

class DeviceDispatcher;
class DeviceIndexEnabledMap;
class DiscoverDialog;
class QtMregDevice;

namespace Ui {
class MainWindow;
}


class MainWindow : public BaseMainWindow
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &devices);

private Q_SLOTS:
    void on_pushButtonChange_clicked();
    void on_comboBoxRegAddr_currentIndexChanged(int);
    void on_lineEditRegAddr_textEdited(const QString);
    void on_comboBoxRegType_currentIndexChanged(int);
    void on_pushButtonRead_clicked();
    void on_pushButtonWrite_clicked();

    void deviceConnected();
    void deviceDisconnected();
    void deviceTempChanged(const DeviceIndex &, double temp);
    void deviceStatusUpdated(const DeviceIndex &, QString s);

private:
    void addDevice(const DeviceIndex &index);
    void removeDevice(const DeviceIndex &index);
    QtMregDevice *createDevice(const DeviceIndex &index);
    bool saveConfig(bool checkWriteVal=true);
    void restoreGui();
    void updateLineEditWriteLen();

    Ui::MainWindow *ui;
    Config config;
    QtMregDevice  * dev = nullptr;
    bool connectedState = false;
    DeviceDispatcher * const deviceDispatcher;
    DiscoverDialog * const discoverDialog;
    double testValue = 0;
    quint64 curDevSerial;
};

#endif // MAINWINDOW_H
