#ifndef ANALIZER_H
#define ANALIZER_H

#include <QObject>
#include <QVector>

#include "base/ClientIndex.h"
#include "lib-common/MpdRawTypes.h"

#include "AnalyzerStat.h"
#include "EventMetaInfo.h"

template<class T> class RingBuffer;
class QSemaphore;
class QElapsedTimer;
class QTimer;

class Analizer : public QObject
{
    Q_OBJECT
public:
    Analizer(ClientIndex _index, RingBuffer<char> *dataRB, RingBuffer<EventMetaInfo> *evRB);
    virtual ~Analizer();
    ClientIndex getClientIndex() const { return index; }
    RingBuffer<EventMetaInfo> *getEventRingBuff() const { return eventRB; }

public slots:
    void startAnalize();
    void stopAnalize();
    void clientDescrChanged(const QString &descr);

signals:
    void updateRunState(bool isInRunMode) const;
    void statUpdated(const AnalyzerStat &s) const;

private slots:
    void run();
    void sentStat() const;

private:
    bool cleanUpFreedSpace(int timeoutMs=-1);
    inline qint64 timeLeftForRun() const { return qMax(100-runElapsedTimer->elapsed(),(qint64)0); }

    int analizedSize;
    MpdEventHdr mpdHdr;
    RingBuffer<char> *byteRB;
    RingBuffer<EventMetaInfo> *eventRB;
    EventMetaInfo *lastMetaInfo; // remember to trash corrupted data
    bool enabled;
    ClientIndex index;
    QElapsedTimer * const runElapsedTimer;
    QTimer * const runTimer;
    QTimer * const statTimer;
//    quint32 prevEvNum;
    QString clientDescr;
    AnalyzerStat stat;
};

#endif // ANALIZER_H
