#ifndef ANALYZERSTAT_H
#define ANALYZERSTAT_H

#include <QElapsedTimer>
#include <QMetaType>

#include "base/ClientIndex.h"

#include "params.h"

struct AnalyzerStat {
    AnalyzerStat() : index(ClientIndex()) {}

    AnalyzerStat(ClientIndex _id):index(_id) {
        clear();
    }

    void clear() {
        totalSize = 0;
        corruptedSize = 0;
        badSync = 0;
        evSize = 0;
        evCnt = 0;
        periodSize = 0;
        periodCnt = 0;
        timeMark.invalidate();
    }

    bool isNextPeriod(const QElapsedTimer &curT) const {
        return periodCnt>EV_SIZE_PERIOD_MIN_CNT &&
                ( !timeMark.isValid() || (timeMark.msecsTo(curT)>EV_SIZE_PERIOD_MS));
    }
    void newPeriod() {
        periodSize = 0;
        periodCnt = 0;
        timeMark.start();
    }

    // For every incoming record exept StartRun/StopRun
    void addEvent(quint32 size) {
        evSize += size;
        ++evCnt;
        periodSize += size;
        ++periodCnt;
    }

    ClientIndex index;
    quint64 totalSize; // total parsed size in bytes
    quint64 corruptedSize; // in bytes
    quint64 badSync; // increment on every badSync setting true (bad fragments)
    quint64 evSize; // of detected records
    quint64 evCnt; // of detected records
    quint64 periodSize;
    quint64 periodCnt;
    QElapsedTimer timeMark;
};
Q_DECLARE_METATYPE(AnalyzerStat)

#endif // ANALYZERSTAT_H
