#ifndef CLIENT_H
#define CLIENT_H

#include "EvBConfig.h"
#include "pnp-server/PNPMessage.h"
#include "ClientFsm.h"

#include <QObject>

template<class T> class RingBuffer;
class TcpClient;
class TcpStatistic;
class Analizer;
class EventMetaInfo;
class QHostAddress;
class QSemaphore;
class AnalyzerStat;
struct RcRunId;

class Client : public QObject
{
    Q_OBJECT
public:
    explicit Client(const EvBClientConfig &cc,int evNumIndex, QObject *parent);
    virtual ~Client() override;
    Analizer *getAnalizer() const { return analizer; }
    ClientIndex getIndex() const { return index; }
    QString getHostAddr() const;
    int getHostPort() const;
    bool isReadoutEnabled() const { return enabled; }
    void setProgDescription(const ProgramDescription &descr);
    ProgramDescription getProgDescr() const { return progDescription; }
    QString getName() const {
        return QString("Client #%1 \"%2\"")
               .arg(index.toString()).arg(progDescription.getName());
    }
    BaseFsm *getFsm() const { return fsm;}

signals:
    void startReadOut();
    void stopReadOut();
    void clientStateUpdated(ClientIndex index, FsmState state); // from combiner and clients
    void starting();
    void stoping();
    void destroyingClient();
    void sendStatus(ClientIndex,bool,QString);
    void sendStatistic(ClientIndex id, const TcpStatistic &statistic);
    void setTcpAddress(const QHostAddress &host, const int port);
    void setTcpAddress(const ProgramDescription &descr, const ProgramInterface &interface);
    void runError();
    void clientDescrChanged(const QString &descr) const;
    void statUpdated(const AnalyzerStat &) const;
    void inputBufferSizeUpdated(ClientIndex index, quint64 size) const;
    void printFsmInput() const;

public slots:
    void startRun();
    void stopRun();
    void destroyClient(ClientIndex i);
    void updateClientReadout(ClientIndex i, bool readout);

private slots:
    void onFsmStateChanged(const FsmState &cur);

private:
    ClientIndex index;
    bool enabled = true;
    ProgramDescription progDescription;
    ProgramInterface tcpInterface;
    int evNumIndex;
    RingBuffer<char> * const dataRingBuff;
    RingBuffer<EventMetaInfo> * const evRingBuff;
    QThread * const tcpClientThread;
    QThread * const analizerThread;
    TcpClient * const tcpClient;
    Analizer * analizer;
    ClientFsm * const fsm;
};

#endif // CLIENT_H
