#include "ClientFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.ClientFsm")
}

ClientFsm::ClientFsm(const ClientIndex &index, QObject *parent)
    :BaseFsm(parent)
{
    setObjectName(__func__);
    recreateInput(new ClientFsmInput(index));
}

void ClientFsm::setTcpState(FsmState tcpState)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << tcpState;
    getInput()->tcpState = tcpState;
    evaluate();
}

void ClientFsm::setAnalizerInRun(bool analizerInRun)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << analizerInRun;
    getInput()->analizerInRun = analizerInRun;
    evaluate();
}

bool ClientFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(getState() != FsmState::Error && getInput()->tcpState == FsmState::Error) {
        setState(FsmState::Error);
        return true;
    }

    switch (getState()) {
    case FsmState::Error:
        if (getInput()->tcpState == FsmState::Ready && !getInput()->analizerInRun) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    case FsmState::Ready:
        if (getRequestState() == FsmState::Starting) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Starting:
        if(getRequestState() == FsmState::Run) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Run:
        if (getRequestState() == FsmState::Ready) {
            acceptRequestedState();
            return true;
        }
        break;
    default:
    {}
    }

    return false;
}

qint64 ClientFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case FsmState::Starting: return 100;
    default:;
    }
    return BaseFsm::stateTimeoutMs(state);
}

void ClientFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

QDebug operator<<(QDebug dbg, const ClientFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "index:" << in.index << "; ";
    dbg << "tcpState:" << in.tcpState << "; ";
    dbg << "analizerInRun:" << in.analizerInRun << "; ";

    return dbg;
}
