#include "ClientManagerFsm.h"
#include  "base/ClientIndexEnabled.h"

ClientManagerFsm::ClientManagerFsm(QObject *parent) :
    BaseFsm(parent)
{
    setObjectName("ClientManagerFsm");
    recreateInput(new ClientManagerFsmInput());
}

void ClientManagerFsm::setClientList(const ClientIndexEnabledMap &newList)
{
    getInput()->clientsSet = newList.getEnabled();
    auto &clientsStates = getInput()->clientsStates;

    for(auto i : clientsStates.keys()) {
        if(!newList.contains(i))
            clientsStates.remove(i);
    }

    evaluate();
}

void ClientManagerFsm::setClientState(ClientIndex index, FsmState state)
{
    getInput()->clientsStates[index] = state;
    evaluate();
}

bool ClientManagerFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(hasErrorClient()){
        setState(FsmState::Error);
        return true;
    }

    switch(getState()){
    case FsmState::Error:
        if (checkAllStates(FsmState::Ready)) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    case FsmState::Ready:
        if(getRequestState() == FsmState::Starting) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Starting:
        if(checkAllStates(FsmState::Run)) {
            setState(FsmState::Run);
            return true;
        }
        break;
    case FsmState::Run:
        if(getRequestState() == FsmState::Ready &&
                checkAllStates(FsmState::Ready)){
            acceptRequestedState();
            return true;
        }
    default:;
    }

    return false;
}

void ClientManagerFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool ClientManagerFsm::checkAllStates(FsmState state) const
{
    for(ClientIndex index : getInput()->clientsSet){
        if(getInput()->clientsStates.value(index, FsmState::Unknown) != state)
            return false;
    }
    return true;
}

bool ClientManagerFsm::hasErrorClient() const
{
    for(ClientIndex index : getInput()->clientsSet){
        if(getInput()->clientsStates.value(index, FsmState::Unknown) == FsmState::Error)
            return true;
    }
    return false;
}

QDebug operator<<(QDebug dbg, const ClientManagerFsmInput &in)
{
    dbg << BaseFsmInput(in);
    dbg << "clientsSet:" << in.clientsSet << "; ";
    dbg << "clientsStates:" << in.clientsStates << "; ";
    return dbg;
}
