#ifndef CLIENTMANAGERFSM_H
#define CLIENTMANAGERFSM_H

#include "fsm/BaseFsm.h"
#include "base/ClientIndex.h"

class ClientIndexEnabledMap;

struct ClientManagerFsmInput : public BaseFsmInput
{
    ClientIndexSet clientsSet;
    QMap<ClientIndex, FsmState> clientsStates;
};

class ClientManagerFsm : public BaseFsm
{
    Q_OBJECT
public:
    ClientManagerFsm(QObject *parent = nullptr);

signals:
    void clientListChanged(const ClientIndexEnabledMap &);

public slots:
    void setClientList(const ClientIndexEnabledMap &newList);
    void setClientState(ClientIndex index, FsmState state);

protected slots:
    virtual bool evaluate() override;

protected:
    inline ClientManagerFsmInput *getInput() {
        return static_cast<ClientManagerFsmInput*>(BaseFsm::getInput());
    }
    inline const ClientManagerFsmInput *getInput() const {
        return static_cast<const ClientManagerFsmInput*>(BaseFsm::getInput());
    }
    virtual void printInputString(QDebug dbg) const override;

    bool checkAllStates(FsmState state) const;
    bool hasErrorClient() const;
};

QDebug operator<<(QDebug dbg, const ClientManagerFsmInput &in);

#endif // CLIENTMANAGERFSM_H
