#ifndef COMBINERFSM_H
#define COMBINERFSM_H

#include "base/ClientIndexEnabled.h"
#include "fsm/BaseFsm.h"

#include "EvBConfig.h"

#include <QMap>
#include <QSet>

struct CombinerFsmInput : public BaseFsmInput
{
    uint setupKey = 0;
    bool fileCreated = false;
    bool fileClosed = false;
    quint64 stopEvNum = 0;
    quint64 expectedEvNum = 0;
    COMBINER_OUT combinerOutType = COMBINER_FILE;
    QSet<int> outputList; // setOutputList
    QMap<int, bool> outputConnected; // setOutputTcpStatus
    QMap<int, bool> outputDone; // setOutputTcpStatus
};

class CombinerFsm : public BaseFsm
{
    Q_OBJECT
public:
    CombinerFsm(QObject *parent = nullptr);

public slots:
    void setOutputList(COMBINER_OUT combinerOut, int tcpNum);
    void setOutputTcpConnected(int id, bool connected);
    void setOutputTcpDone(int id, bool rbIsEmpty);
//    void setOutputState(const int &id, bool ok);
//    void setStateIndex(uint curStateIndex);
    void setStopEvNum(quint64 stopEvNum);
    void setExpectedEvNum(quint64 expectedEvNum);
    void setFileCreated(bool created);
    void setFileClosed(bool closed);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline CombinerFsmInput *getInput() {
        return static_cast<CombinerFsmInput*>(BaseFsm::getInput());
    }
    inline const CombinerFsmInput *getInput() const {
        return static_cast<const CombinerFsmInput*>(BaseFsm::getInput());
    }

    bool checkStart() const;
    bool checkStop() const;
    bool allOutputConnected() const;
    bool allOutputDone() const;
    bool checkStopEvNum() const;
};

QDebug operator<<(QDebug dbg, const CombinerFsmInput &in);

#endif // COMBINERFSM_H
