#ifndef COMBINERSTATS_H
#define COMBINERSTATS_H

#include <QMap>
#include <QMetaType>
#include <QSet>

#include <EventMetaInfo.h>
#include "EvBConfig.h"
#include "OutputTcpStatus.h"
#include "params.h"

struct CombinerStats {
    CombinerStats() { clear(); newPeriod(); }
    void clear() {
        eob = 0;
        regularEvents = 0;
        fullEvents = 0;
        partedEvents = 0;
        skipped = 0;
        monitorEvCount = 0;
        missedFileSize = 0;
        lastEventClients.clear();
//        lastEventNum.clear();
        for (ClientIndex key : lastEventNum.keys())
            lastEventNum[key] = 0;
        for (ClientIndex key : flushedSize.keys())
            flushedSize[key] = 0;
        for (ClientIndex key : dataEvents.keys())
            dataEvents[key] = 0;
        for (ClientIndex key : trashedEvents.keys())
            trashedEvents[key] = 0;
        clientNumDistr.clear();
        timeDistr.clear();
        timeDistrCnt = 0;
        fwDataPart = 0;
        fwFullSize = 0;
        hasPendingEvent = false;
//        tcpPorts.clear();
//        tcpIsFree.clear();
    }

    bool isNextPeriod(const QElapsedTimer &curT) const {
        return periodCnt>EV_SIZE_PERIOD_MIN_CNT &&
                ( !timeMark.isValid() || (timeMark.msecsTo(curT)>EV_SIZE_PERIOD_MS));
    }

    void newPeriod() {
        periodSize = 0;
        periodCnt = 0;
        timeMark.start();
    }

    void addEvent(quint32 size) {
        periodSize += size;
        ++periodCnt;
    }

    quint64 eob;
    quint64 regularEvents;
    quint64 fullEvents;
    quint64 partedEvents;
    quint64 skipped;
    quint64 expectedEvNum;
    quint64 monitorEvCount;
    quint64 missedFileSize;
    QSet<ClientIndex> lastEventClients; // set of clientIds in last flushed event
    QMap<int, quint64> clientNumDistr; //  [clientNumInEv] = count
                                       // (for 1st level EvB: clientNumInEv=devNumInEv)
    QMap<ClientIndex, quint64> lastEventNum; // last EvNum gained by Combiner
    QMap<ClientIndex, quint64> flushedSize; // summed size of all events gained by Combiner
    QMap<ClientIndex, quint64> dataEvents; // total events count gained by Combiner
    QMap<ClientIndex, quint64> trashedEvents; //  number of trashed evnets
    QMap<ClientIndex, qint64> timeDistr; // [index] time from erliest ev in ms
    QMap<ClientIndex, qint64> timeDistrFull; // [index] time from decoding till flush
    QMap<ClientIndex, int> freeEvPart; //  % of free events
    QMap<ClientIndex, int> preparedEvPart; //  % of prepared events
    QMap<ClientIndex, int> flushedEvPart; //  % of flushed events

    quint32 timeDistrCnt;
    QMap<int, OutputTcpStatus> tcpStatus;
    double fwDataPart;
    int fwFullSize;
    int fwPrefSize;
    bool hasPendingEvent;

    quint64 periodSize;
    quint64 periodCnt;
    QElapsedTimer timeMark;
    QElapsedTimer timeStamp; // updated on send stat
};
Q_DECLARE_METATYPE(CombinerStats)

#endif // COMBINERSTATS_H
