#include "EvBConfig.h"

#include <QDataStream>
#include <QDebug>

ClientIndexEnabledMap EvBConfig::getClientIndexEnabledMap() const
{
    ClientIndexEnabledMap map;
    for(auto cfg : clients.values()){
        auto &die = map[cfg.index];
        die.index = cfg.index;
        die.enabled = cfg.enabled;
    }
    return map;
}

QString EvBConfig::getOutputStr() const
{
    switch (combinerOut) {
    case COMBINER_FILE: return "file";
    case COMBINER_TCP: return "tcp";
    case COMBINER_IDLE: return "idle";
    }
    return "unknown";
}

QDebug operator<<(QDebug dbg, const EvBClientConfig &c)
{
    dbg << "EvBClientConfig{"
        << "client_index:" << c.index
        << "enabled:" << c.enabled
        << "inputBufSize:" << c.inputBufSize
        << "}";

    return dbg;
}

QDebug operator<<(QDebug dbg, const EvBConfig &c)
{
    dbg << "combinerOut:" << c.combinerOut
        << "isRemoteEnabled:" << c.isRemoteEnabled
        << "advancedCounters:" << c.advancedCounters
        << "combinerOutTcpNum:" << c.combinerOutTcpNum
        << "startEvNumber:" << c.startEvNumber
        << "evNumberStep:" << c.evNumberStep
        << "storageDir:" << c.storageDir
        << "clients:" << c.clients
        << "timeDistrEn:" << c.timeDistrEn
        << "outputBufSize:" << c.outputBufSize
        << "monitorPort:" << c.monitorPort;

    return dbg;
}

QDataStream &operator<<(QDataStream &out, const EvBClientConfig &r)
{
    out
            << r.index
            << r.enabled
            << r.inputBufSize;

    return out;
}

QDataStream &operator>>(QDataStream &in, EvBClientConfig &r)
{
    in
            >> r.index
            >> r.enabled
            >> r.inputBufSize;

    return in;
}

QDebug operator<<(QDebug dbg, const COMBINER_OUT &c)
{
    switch (c) {
    case COMBINER_FILE : dbg << "File"; break;
    case COMBINER_TCP: dbg << "Tcp"; break;
    case COMBINER_IDLE: dbg << "Idle"; break;
    }
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const EvBConfig &r)
{
    out
            << static_cast<int>(r.combinerOut)
            << r.isRemoteEnabled
            << r.advancedCounters
            << r.combinerOutTcpNum
            << r.startEvNumber
            << r.evNumberStep
            << r.storageDir
            << r.clients
            << r.timeDistrEn
            << r.outputBufSize
            << r.monitorPort;
    return out;
}

QDataStream &operator>>(QDataStream &in, EvBConfig &r)
{
    int combinerOut;
    in
            >> combinerOut
            >> r.isRemoteEnabled
            >> r.advancedCounters
            >> r.combinerOutTcpNum
            >> r.startEvNumber
            >> r.evNumberStep
            >> r.storageDir
            >> r.clients
            >> r.timeDistrEn
            >> r.outputBufSize
            >> r.monitorPort;
    r.combinerOut = static_cast<COMBINER_OUT>(combinerOut);
    return in;
}
