﻿#ifndef EVBCONFIG_H
#define EVBCONFIG_H

#include "base/RcRunId.h"
#include "base/ClientIndex.h"
#include "base/ClientIndexEnabled.h"

class QDataStream;

struct EvBClientConfig
{
    QString toString() const { return QString("Client #%1").arg(index.toString()); }

    ClientIndex index;
    bool enabled = true;
    quint64 inputBufSize = 0;
};
Q_DECLARE_METATYPE(EvBClientConfig)

inline bool operator==(const EvBClientConfig &l, const EvBClientConfig &r)
{
    return l.index == r.index &&
            l.enabled == r.enabled &&
            l.inputBufSize == r.inputBufSize;
}
QDebug operator<<(QDebug dbg, const EvBClientConfig &c);
QDataStream &operator<<(QDataStream &out, const EvBClientConfig &r);
QDataStream &operator>>(QDataStream &in, EvBClientConfig &r);

enum COMBINER_OUT{COMBINER_FILE=0, COMBINER_TCP=1, COMBINER_IDLE=2};
Q_DECLARE_METATYPE(COMBINER_OUT)
QDebug operator<<(QDebug dbg, const COMBINER_OUT &c);

using EvBClientMap = QMap<ClientIndex, EvBClientConfig>;

struct EvBConfig
{
    ClientIndexEnabledMap getClientIndexEnabledMap() const;
    int getEvNumIndex() const { return static_cast<int>(startEvNumber%evNumberStep); }
    bool isOutFile() const {return combinerOut==COMBINER_FILE;}
    bool isOutTcp() const {return combinerOut==COMBINER_TCP;}
    bool isOutIdle() const {return combinerOut==COMBINER_IDLE;}
    QString getOutputStr() const;

    COMBINER_OUT combinerOut = COMBINER_FILE;
    bool isRemoteEnabled = true;
    bool advancedCounters = false;
    int combinerOutTcpNum = 0;
    quint32 startEvNumber = 0; // EvB index in 2nd level
    quint32 evNumberStep = 1; // number of EvB in 2nd level
    QString storageDir;
    bool timeDistrEn = false;
    quint64 outputBufSize = 0;
    quint16 monitorPort = 0;
    QMap<ClientIndex, EvBClientConfig> clients;
};
Q_DECLARE_METATYPE(EvBConfig)

inline bool operator==(const EvBConfig &l, const EvBConfig &r)
{
    return l.combinerOut == r.combinerOut &&
            l.isRemoteEnabled == r.isRemoteEnabled &&
            l.advancedCounters == r.advancedCounters &&
            l.combinerOutTcpNum == r.combinerOutTcpNum &&
            l.startEvNumber == r.startEvNumber &&
            l.evNumberStep == r.evNumberStep &&
            l.storageDir == r.storageDir &&
            l.timeDistrEn == r.timeDistrEn &&
            l.outputBufSize == r.outputBufSize &&
            l.monitorPort == r.monitorPort &&
            l.clients == r.clients;
}
QDebug operator<<(QDebug dbg, const EvBConfig &c);
QDataStream &operator<<(QDataStream &out, const EvBConfig &r);
QDataStream &operator>>(QDataStream &in, EvBConfig &r);

#endif // EVBCONFIG_H
