#include "EvBConfigConverter.h"

#include "base/CoreTypeConverter.h"
#include "base/ConfigConverterUtil.h"
#include "EvBConfig.h"

namespace {
//struct ClientConfig;
constexpr const char *k_client_index = "client_index";
constexpr const char *k_enabled = "enabled";
constexpr const char *k_inputBufSize = "inputBufSize";

//class EvBConfig;
constexpr const char *k_combinerOut = "combinerOut";
constexpr const char *k_isRemoteEnabled = "isRemoteEnabled";
constexpr const char *k_advancedCounters = "advancedCounters";
constexpr const char *k_combinerOutTcpNum = "combinerOutTcpNum";
constexpr const char *k_startEvNumber = "startEvNumber";
constexpr const char *k_evNumberStep = "evNumberStep";
constexpr const char *k_storageDir = "storageDir";
constexpr const char *k_timeDistrEn = "timeDistrEn";
constexpr const char *k_outputBufSize = "outputBufSize";
constexpr const char *k_monitorPort = "monitorPort";
constexpr const char *k_clients = "clients";

template<typename T>
void test_get_data(const QJsonObject &m, const QString& key, T *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = QVariant::fromValue(it.value()).value<T>();
    }
}
template<>
void test_get_data(const QJsonObject &m, const QString& key, quint64 *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = jsonValueToUInt64(it.value());
    }
}
template<>
void test_get_data(const QJsonObject &m, const QString& key, EvBClientMap *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = ConfigConverterUtil::toMap<ClientIndex, EvBClientConfig>(it.value());
    }
}
}

// ===================== ClientConfig Start =====================

template<>
bool AbstractConfigConverter<EvBClientConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<EvBClientConfig>::toJsonObject(const EvBClientConfig &in)
{
    QVariantMap m;
    m[k_client_index] = QVariant::fromValue(in.index);
    m[k_enabled] = QVariant::fromValue(in.enabled);
    m[k_inputBufSize] = jsonValueFromUInt64(in.inputBufSize);
    return fromVariantMap(m);
}

template<>
EvBClientConfig AbstractConfigConverter<EvBClientConfig>::fromJsonObject(const QJsonObject &m)
{
    EvBClientConfig out;
    update_value(m, k_client_index, &out.index);
    update_value(m, k_enabled, &out.enabled);
    out.inputBufSize = jsonValueToUInt64(m[k_inputBufSize]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<EvBClientConfig>::toJsonValue(const EvBClientConfig &in)
{
    return toJsonObject(in);
}

template<>
EvBClientConfig AbstractConfigConverter<EvBClientConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<EvBClientConfig>::registerSubTypes()
{
    CoreTypeConverter::registerTypes();
    AbstractConfigConverter<ClientIndex>::registerTypes();
}

// ===================== ClientConfig End =====================

// ===================== EvBConfig Start =====================

template<>
bool AbstractConfigConverter<EvBConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<EvBConfig>::toJsonObject(const EvBConfig &in)
{
    QVariantMap m;
    m[k_combinerOut] = QVariant::fromValue(static_cast<int>(in.combinerOut));
    m[k_isRemoteEnabled] = QVariant::fromValue(in.isRemoteEnabled);
    m[k_advancedCounters] = QVariant::fromValue(in.advancedCounters);
    m[k_combinerOutTcpNum] = QVariant::fromValue(in.combinerOutTcpNum);
    m[k_startEvNumber] = QVariant::fromValue(in.startEvNumber);
    m[k_evNumberStep] = QVariant::fromValue(in.evNumberStep);
    m[k_storageDir] = QVariant::fromValue(in.storageDir);
    m[k_timeDistrEn] = QVariant::fromValue(in.timeDistrEn);
    m[k_outputBufSize] = jsonValueFromUInt64(in.outputBufSize);
    m[k_monitorPort] = QVariant::fromValue(in.monitorPort);
    m[k_clients] = ConfigConverterUtil::toJsonValue<ClientIndex, EvBClientConfig>(in.clients);
    return fromVariantMap(m);
}

template<>
EvBConfig AbstractConfigConverter<EvBConfig>::fromJsonObject(const QJsonObject &m)
{
    EvBConfig out;

    update_value(m, k_combinerOut, &out.combinerOut);
    update_value(m, k_isRemoteEnabled, &out.isRemoteEnabled);
    update_value(m, k_advancedCounters, &out.advancedCounters);
    update_value(m, k_combinerOutTcpNum, &out.combinerOutTcpNum);
    update_value(m, k_startEvNumber, &out.startEvNumber);
    update_value(m, k_evNumberStep, &out.evNumberStep);
    update_value(m, k_storageDir, &out.storageDir);
    update_value(m, k_timeDistrEn, &out.timeDistrEn);
    update_value(m, k_outputBufSize, &out.outputBufSize);
    out.outputBufSize = jsonValueToUInt64(m[k_outputBufSize]);
    update_value(m, k_monitorPort, &out.monitorPort);
    out.clients = ConfigConverterUtil::toMap<ClientIndex, EvBClientConfig>
        (m[k_clients]);
    if(out.clients.remove(ClientIndex())) {
        qWarning() << "Some error in clients config";
    }
    return out;
}

template<>
QJsonValue AbstractConfigConverter<EvBConfig>::toJsonValue(const EvBConfig &in)
{
    return toJsonObject(in);
}

template<>
EvBConfig AbstractConfigConverter<EvBConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<EvBConfig>::registerSubTypes()
{
    CoreTypeConverter::registerTypes();

    AbstractConfigConverter<EvBClientConfig>::registerTypes();
}
// ===================== EvBConfig End =====================
