#include "EvBConfigDispatcher.h"

#include "EvBConfig.h"

#include "base/util.h"

EvBConfigDispatcher::EvBConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    config(new EvBConfig())
{}

void EvBConfigDispatcher::setConfig(const EvBConfig &newConfig)
{
    if (newConfig == *config)
        return;

    const auto &newClientsSet = newConfig.getClientIndexEnabledMap();
    if(newClientsSet != config->getClientIndexEnabledMap()){
        emit clientListChanged(newClientsSet);
    }

    *config = newConfig;
    emit configChanged(*config);
}

void EvBConfigDispatcher::setClientList(const ClientIndexEnabledMap &new_clients)
{
    auto curClients = config->getClientIndexEnabledMap();
    ClientIndexEnabledMap filtered_clients(new_clients);
    filtered_clients.remove(ClientIndex()); // remove invalid index if present
    if (filtered_clients == curClients)
        return;

    const auto &oldSet = ClientIndexSet(curClients);
    const auto &newSet = ClientIndexSet(filtered_clients);
    curClients = filtered_clients;

    for(auto index : oldSet - newSet)
        config->clients.remove(index);

    for(auto index : newSet) {
        auto &cfg = config->clients[index];
        cfg.index = index;
        cfg.enabled = new_clients[index].enabled;
    }

    emit configChanged(*config);
    emit clientListChanged(curClients);
}
