#ifndef EVBCONFIGDISPATCHER_H
#define EVBCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"
#include "base/ClientIndexEnabled.h"

#include "EvBConfig.h"

class EvBConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit EvBConfigDispatcher(QObject *parent = nullptr);
    virtual void sendCache() const override {
        BaseConfigDispatcher::sendCache();
        emit configChanged(*config);
    }

    template<class T>
    void subscribeEvbConfig(T *obj) {
            connect(this, &EvBConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &EvBConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            qDebug() << "Config:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeClients(T *obj) {
        connect(obj, &T::clientListChanged,
                this, &EvBConfigDispatcher::setClientList,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::clientListChanged,
                obj, &T::setClientList,
                Qt::UniqueConnection);
    }

signals:
    void configChanged(const EvBConfig &) const;
    void clientListChanged(const ClientIndexEnabledMap &) const;

public slots:
    void setConfig(const EvBConfig &newConfig);
    void setClientList(const ClientIndexEnabledMap &new_clients);

private:

    std::unique_ptr<EvBConfig> config;
};

#endif // EVBCONFIGDISPATCHER_H
