#include "EvbSignalRouter.h"

#include "mldiscover/LldpDiscovery.h"
#include "pnp-server/PNPServer.h"
#include "remote-control-server/RemoteControlServer.h"

#include "EvBConfigDb.h"
#include "EventBuilderFsm.h"
#include "MainWindow.h"
#include "RunControl.h"
#include "ClientManagerFsm.h"
#include "Combiner.h"
#include "CombinerFsm.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void EvbSignalRouter::connectAll()
{
    Q_ASSERT(pnpServer);
    connect_PnpServer();
    connect_RemoteControlServer();
    connect_Combiner();
    connect_Fsm();
    connect_WidgetRunControl();
}

void EvbSignalRouter::connect_PnpServer()
{
    if(!pnpServer || !mainWindow)
        return;

    connect(mainWindow, &MainWindow::closeProgram, pnpServer,
            &PNPServer::closeProgram);
    connect(mainWindow, &MainWindow::changeProgramDescription,
            pnpServer, &PNPServer::changeProgramDescription);
    connect(mainWindow, &MainWindow::destroyAll,
            pnpServer, &PNPServer::deleteLater);

    connect(pnpServer, &PNPServer::programDiscovered,
            mainWindow, &MainWindow::gotProgramDescription);
    connect(pnpServer, &PNPServer::programClosed,
            mainWindow, &MainWindow::gotClosedProgram);
}

void EvbSignalRouter::connect_RemoteControlServer()
{
    if(!remoteControlServer || !mainWindow)
        return;

    connect(remoteControlServer, &RemoteControlServer::startRequest,
            mainWindow, &MainWindow::remoteStartRequest);
    connect(remoteControlServer, &RemoteControlServer::stopRequest,
            mainWindow, &MainWindow::remoteStopRequest);
    connect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated,
            mainWindow, &MainWindow::onRemoteStatusUpdated);

    connect(mainWindow, &MainWindow::destroyAll,
            remoteControlServer, &RemoteControlServer::deleteLater);
    connect(mainWindow, &MainWindow::sendRemoteState,
            remoteControlServer, &RemoteControlServer::sendState);
    connect(mainWindow, &MainWindow::sendRunError,
            remoteControlServer, &RemoteControlServer::onRunError);

    connect(mainWindow, &MainWindow::sendRunWarning,
            remoteControlServer, &RemoteControlServer::onRunWarning);
    connect(mainWindow, &MainWindow::sendBackPreasure,
            remoteControlServer, &RemoteControlServer::onBackPressure);

    connect(mainWindow, &MainWindow::allowRemoteControl,
            remoteControlServer, &RemoteControlServer::startListen);
    connect(mainWindow, &MainWindow::config_transfer_to_rc,
            remoteControlServer, &RemoteControlServer::config_transfer);
    connect(mainWindow, &MainWindow::changeProgramDescription,
            remoteControlServer, &RemoteControlServer::changeProgramDescription);
}

void EvbSignalRouter::connect_Combiner()
{
    if(!combiner)
        return;

    connect(mainWindow, &MainWindow::destroyAll,
            combiner, &PNPServer::deleteLater);
}

void EvbSignalRouter::connect_Fsm()
{
    if(!evbFsm || !client_manager_fsm)
        return;

    if(combinerFsm) {
        if(evbFsm)
            connect(combinerFsm, &BaseFsm::stateChanged,
                    evbFsm, &EventBuilderFsm::setCombinerState);
        if(mainWindow)
            connect(combinerFsm, &BaseFsm::stateChanged,
                    mainWindow, &MainWindow::onCombinerFsmStateChanged);
    }

    if(mainWindow) {
        connect(evbFsm, &EventBuilderFsm::stateChanged,
                mainWindow, &MainWindow::onEvbFsmStateChanged);
        connect(client_manager_fsm, &ClientManagerFsm::stateChanged,
                mainWindow, &MainWindow::onClientManagerFsmStateChanged);
        connect(mainWindow, &MainWindow::printFsmInput,
                evbFsm, &EventBuilderFsm::printFsmInput);
        connect(mainWindow, &MainWindow::printFsmInput,
                client_manager_fsm, &BaseFsm::printFsmInput);
        connect(mainWindow, &MainWindow::printFsmInput,
                combinerFsm, &BaseFsm::printFsmInput);
        connect(client_manager_fsm, &ClientManagerFsm::stateChanged,
                evbFsm, &EventBuilderFsm::setClientManagerFsmState);
    }
    if(widgetRunControl)
        connect(evbFsm, &EventBuilderFsm::stateChanged,
                widgetRunControl, &RunControl::onStateChanged);
    if(remoteControlServer)
        connect(evbFsm, &EventBuilderFsm::stateChanged,
                remoteControlServer, &RemoteControlServer::onFsmStateChange);
}

void EvbSignalRouter::connect_WidgetRunControl()
{
    if(!widgetRunControl)
        return;

    if(mainWindow) {
        connect(widgetRunControl, &RunControl::startLocalRun,
                mainWindow, &MainWindow::startLocalRun);
        connect(widgetRunControl, &RunControl::stopLocalRun,
                mainWindow, &MainWindow::stopLocalRun);
    }
}

