#ifndef EVBSIGNALROUTER_H
#define EVBSIGNALROUTER_H

class Client;
class Combiner;
class ConfigurationManager;
class EvBConfigDb;
class EvBConfigDispatcher;
class EventBuilderFsm;
class MainWindow;
class PNPServer;
class RemoteControlServer;
class RunControl;
class CombinerFsm;
class ClientManagerFsm;

class EvbSignalRouter
{
    EvbSignalRouter() = default;
public:
    // Singleton
    static EvbSignalRouter & instance()
    {
        // thread-safe since C++11
        static EvbSignalRouter obj;
        return obj;
    }

    // delete copy and move constructors and assign operators
    EvbSignalRouter(EvbSignalRouter const &) = delete;
    EvbSignalRouter(EvbSignalRouter &&) = delete;
    EvbSignalRouter &operator=(EvbSignalRouter const &) = delete;
    EvbSignalRouter &operator=(EvbSignalRouter &&) = delete;

    void setConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }

    void setMainWindow(MainWindow *p)
    { mainWindow = p; }

    void setPnpServer(PNPServer *p)
    { pnpServer = p; }

    void setEvbFsm(EventBuilderFsm *p)
    { evbFsm = p; }

    void setClientManagerFsm(ClientManagerFsm *p)
    { client_manager_fsm = p; }

    void setCombinerFsm(CombinerFsm *p)
    { combinerFsm = p; }

    void setRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

    void setCombiner(Combiner *p)
    { combiner = p; }

    void setEvBConfigDb(EvBConfigDb * p)
    { configDb = p; }

    void setWidgetRunControl(RunControl * p)
    { widgetRunControl = p; }

    void connectAll();

private:
    void connect_PnpServer();
    void connect_RemoteControlServer();
    void connect_Combiner();
    void connect_Fsm();
    void connect_WidgetRunControl();

    ConfigurationManager *configurationManager = nullptr;
    MainWindow *mainWindow = nullptr;
    PNPServer *pnpServer = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    Combiner * combiner = nullptr;
    EvBConfigDb * configDb = nullptr;
    RunControl *widgetRunControl = nullptr;
    EventBuilderFsm * evbFsm = nullptr;
    ClientManagerFsm * client_manager_fsm = nullptr;
    CombinerFsm * combinerFsm = nullptr;
};

#endif // EVBSIGNALROUTER_H
