#include "EventBuilderFsm.h"

#include <QDebug>
#include <QLoggingCategory>

#include "EvBConfig.h"

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.EvB")
}

EventBuilderFsm::EventBuilderFsm(QObject *parent):
    BaseFsm(parent)
{
    setObjectName("EventBuilderFsm");
    recreateInput(new EventBuilderFsmInput());
}

void EventBuilderFsm::setCombinerState(FsmState cur_state, FsmState old_state)
{
    Q_UNUSED(old_state);
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << cur_state;

    getInput()->combinerState = cur_state;
    evaluate();
}

void EventBuilderFsm::setClientManagerFsmState(FsmState cur_state, FsmState old_state)
{
    Q_UNUSED(old_state);

    getInput()->clientManagerFsmState = cur_state;
    evaluate();
}

bool EventBuilderFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(getState()!=FsmState::Error){
        if(getInput()->combinerState == FsmState::Error) {
            auto dbg = qDebug(fsm) << objectName() << "Combiner in error st. Input:";
            printInputString(dbg);
            setState(FsmState::Error);
            return true;
        }

        if(getInput()->clientManagerFsmState == FsmState::Error) {
            auto dbg = qDebug(fsm) << objectName() << "ClientManagerFsm in error st. Input:";
            printInputString(dbg);
            setState(FsmState::Error);
            return true;
        }
    }

    switch (getState()) {
    case FsmState::Error:
        if (allInState(FsmState::Ready)) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    case FsmState::Ready:
        if (allInState(FsmState::Ready)) {
            if (getRequestState() == FsmState::Starting) {
                acceptRequestedState();
                return true;
            }
        } else {
            setState(FsmState::Error);
            return true;
        }
        break;
    case FsmState::Starting:
        if(allInState(FsmState::Run)) {
            setState(FsmState::Run);
            return true;
        }
        break;
    case FsmState::Run:
        if (!allInState(FsmState::Run)) {
            qDebug(fsm) << objectName() << "Go to Error state (some dev cfg is invalid)";
            setState(FsmState::Error); // stop on error
            return true;
        }
        if (getRequestState() == FsmState::PrepareStopping ||
                getRequestState() == FsmState::Stopping) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::PrepareStopping:
        if(getRequestState() == FsmState::Stopping &&
           allInState(FsmState::Stopping)) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Stopping:
        if(allInState(FsmState::Ready)) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    default:;
    }

    return false;
}

void EventBuilderFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

qint64 EventBuilderFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case FsmState::Starting:
//        return 1000;
    case FsmState::PrepareStopping:
//        return 10000;
    case FsmState::Stopping:
//        return 100;
        return 1e4;
    default:
        return BaseFsm::stateTimeoutMs(state);
    }
}

bool EventBuilderFsm::allInState(FsmState state) const
{
    if(getInput()->combinerState != state)
        return false;
    if(getInput()->clientManagerFsmState != state)
        return false;

    return true;
}

QDebug operator<<(QDebug dbg, const EventBuilderFsmInput &in)
{
    dbg << BaseFsmInput(in);
    dbg << "combiner:" << in.combinerState << ";";
    dbg << "clientManager:" << in.clientManagerFsmState << ";";
    return dbg;
}
