#include "StatisticDialog.h"
#include "ui_StatisticDialog.h"
#include <QTimer>

namespace {
enum COL_STAT_T {
//    COL_STAT_T_PROGRAM_TYPE,
    COL_STAT_T_PROGRAM_INDEX,
    COL_STAT_T_FREE_PART,
    COL_STAT_T_DATA_PART,
    COL_STAT_T_TRASH_PART,
    COL_STAT_T_RCV_BUFF_SIZE,
    COL_STAT_T_READ_BUFFER_SIZE,
    COL_STAT_T_AVAIL_SIZE,
    COL_STAT_T_TOTAL
};
enum COL_STAT_A {
//    COL_STAT_A_PROGRAM_TYPE,
    COL_STAT_A_PROGRAM_INDEX,
    COL_STAT_A_TOT_SIZE,
    COL_STAT_A_CORR_SIZE,
    COL_STAT_A_BAD_SYNC,
    COL_STAT_A_EV_SIZE,
    COL_STAT_A_EV_CNT,
    COL_STAT_A_PERIOD_EV_SIZE,
    COL_STAT_A_TOTAL
};
enum COL_STAT_C {
    COL_STAT_C_ID,
    COL_STAT_C_DATAPART,
    COL_STAT_C_FULL_SIZE,
    COL_STAT_C_PREF_SIZE,
    COL_STAT_C_SPEED,
    COL_STAT_C_TOTAL
};

const QMap<int, QString> TableHeadersTcp = {
//    {COL_STAT_T_PROGRAM_TYPE, "program_type"},
    {COL_STAT_T_PROGRAM_INDEX, "Client"}, // prev was program_index
    {COL_STAT_T_DATA_PART, "Data %"},
    {COL_STAT_T_FREE_PART, "Free %"},
    {COL_STAT_T_TRASH_PART, "Trash %"},
    {COL_STAT_T_RCV_BUFF_SIZE, "Rcv buff size"},
    {COL_STAT_T_READ_BUFFER_SIZE, "Read buffer size"},
    {COL_STAT_T_AVAIL_SIZE, "Avail"},
};
const QMap<int, QString> TableHeadersAnal = {
//    {COL_STAT_A_PROGRAM_TYPE, "program_type"},
    {COL_STAT_A_PROGRAM_INDEX, "Client"}, // prev was program_index
    {COL_STAT_A_TOT_SIZE, "Tot. size"},
    {COL_STAT_A_CORR_SIZE, "Corrupted"},
    {COL_STAT_A_BAD_SYNC, "Bad sync"},
    {COL_STAT_A_EV_SIZE, "Ev size"},
    {COL_STAT_A_EV_CNT, "Ev cnt"},
    {COL_STAT_A_PERIOD_EV_SIZE, "Period ev. size"},
};
const QMap<int, QString> TableHeadersWriter = {
    {COL_STAT_C_ID, "id"},
    {COL_STAT_C_DATAPART, "FW datapart"},
    {COL_STAT_C_FULL_SIZE, "FW full size, MB"},
    {COL_STAT_C_PREF_SIZE, "FW pref size, MB"},
    {COL_STAT_C_SPEED, "FW speed, MB/s"},
};
}

StatisticDialog::StatisticDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::StatisticDialog),
    refreshTimer(new QTimer(this))
{
    ui->setupUi(this);
    init();
    refreshTimer->setInterval(100);
    refreshTimer->setSingleShot(true);
    connect(refreshTimer, &QTimer::timeout, this, &StatisticDialog::updateStat);
}

StatisticDialog::~StatisticDialog()
{
    delete ui;
}

void StatisticDialog::setVisible(bool visible)
{
    QDialog::setVisible(visible);
    if(visible) {
        updateStat();
        updateCombinerStats(combStat);
    }
}

void StatisticDialog::analizerListUpdated(const ClientIndexSet &clientIds)
{
    const auto &toAdd = clientIds - curClientIds;
    const auto &toDel = curClientIds - clientIds;
    curClientIds = clientIds;

    for(auto i : toDel) {
        for(auto t : {Table::Tcp, Table::Anal}) {
            auto table = getTableWidget(t);
            int r = getRowByClientIndex(t, i);
            if(r == -1) continue;
            table->removeRow(r);
        }
        tcpStats.remove(i);
        analyzerStats.remove(i);
    }

    for(auto t : {Table::Tcp, Table::Anal}) {
        auto table = getTableWidget(t);
        for(auto i : toAdd) {
            int r = table->rowCount();
            table->setRowCount(r+1);
            for(int c=0; c<table->columnCount(); ++c) {
                table->setItem(r, c, new QTableWidgetItem());
            }
            auto item = table->item(r, getTableIdCol(t));
            item->setData(Qt::UserRole, QVariant::fromValue(i));
            item->setText(i.toString());
        }
    }
}

void StatisticDialog::analyzerStatUpdated(const AnalyzerStat &s)
{
    analyzerStats[s.index] = s;
    if(!refreshTimer->isActive())
        refreshTimer->start();
}

void StatisticDialog::updateTcpStatistic(const ClientIndex &index, const TcpStatistic &statistic)
{
    tcpStats[index] = statistic;
    if(!refreshTimer->isActive())
        refreshTimer->start();
}

void StatisticDialog::updateCombinerStats(const CombinerStats &_combStat)
{
    if(isHidden()) {
        combStat = _combStat;
        return;
    }

    const qint64 dtMs = (combStat.timeStamp.isValid() && _combStat.timeStamp.isValid()) ? combStat.timeStamp.msecsTo(_combStat.timeStamp) : 0;
    qint64 fullSize = 0;
    for (ClientIndex id : _combStat.flushedSize.keys()) {
        if(combStat.flushedSize.contains(id))
            fullSize = _combStat.flushedSize[id] - combStat.flushedSize[id];
    }
    const double speed = dtMs ? (1. * fullSize / dtMs *1e3) : 0;
    combStat = _combStat;
    QTableWidget *table = ui->tableWidgetCombiner;
    table->item(0, COL_STAT_C_DATAPART)->setText(QString::number(1e2*_combStat.fwDataPart,'f',1));
    table->item(0, COL_STAT_C_FULL_SIZE)->setText(QString::number(1e-6*_combStat.fwFullSize,'f',3));
    table->item(0, COL_STAT_C_PREF_SIZE)->setText(QString::number(1e-6*_combStat.fwPrefSize,'f',3));
    table->item(0, COL_STAT_C_SPEED)->setText(dtMs ? QString::number(1e-6*speed,'f',3) : QString("-"));
}

void StatisticDialog::updateStat()
{
    if(isHidden())
        return;

    updateAnalStat();
    updateTcpStat();
}

void StatisticDialog::init()
{
    for(auto t : {Table::Tcp, Table::Anal, Table::Writer}) {
        auto table = getTableWidget(t);
        int colCnt = -1;
        const QMap<int, QString> *headers = nullptr;
        switch (t) {
        case Table::Tcp:
            colCnt = COL_STAT_T_TOTAL;
            headers = &TableHeadersTcp;
            break;
        case Table::Anal:
            colCnt = COL_STAT_A_TOTAL;
            headers = &TableHeadersAnal;
            break;
        case Table::Writer:
            colCnt = COL_STAT_C_TOTAL;
            headers = &TableHeadersWriter;
            break;
        }
        table->setColumnCount(colCnt);
        for(int c=0; c<colCnt; ++c)
            table->setHorizontalHeaderItem(c, new QTableWidgetItem(headers->value(c)));

#if QT_VERSION >= 0x050000
        table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
        table->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif
        if(t==Table::Writer) {
            table->setRowCount(1);
            for(int col=0;col<colCnt;++col)
                table->setItem(0, col, new QTableWidgetItem());
            table->item(0, COL_STAT_C_ID)->setText("File writer");
            updateTableSize(table);
        }
    }
}

void StatisticDialog::updateTableSize(QTableWidget *table)
{
    if(table == ui->tableWidgetCombiner) {
        int height=table->horizontalHeader()->height();
        for(int i=0;i<table->rowCount();++i){
            height += table->rowHeight(i);
        }
        height += 5;
        table->setFixedHeight(height);
    }

    int width=table->verticalHeader()->width();
    for(int i=0;i<table->columnCount();++i){
        width += table->columnWidth(i);
    }
    width += 5;
    table->setMinimumWidth(width);
}

QTableWidget *StatisticDialog::getTableWidget(StatisticDialog::Table t) const
{
    switch (t) {
    case Table::Tcp: return ui->tableWidgetClient;
    case Table::Anal: return ui->tableWidgetAnalyzer;
    case Table::Writer: return ui->tableWidgetCombiner;
    }
}

int StatisticDialog::getTableIdCol(StatisticDialog::Table t) const
{
    switch (t) {
    case Table::Tcp:  return COL_STAT_T_PROGRAM_INDEX;
    case Table::Anal: return COL_STAT_A_PROGRAM_INDEX;
    case Table::Writer: return -1;
    }
}

int StatisticDialog::getRowByClientIndex(Table t, const ClientIndex &i) const
{
    auto table = getTableWidget(t);

    for(int r=0; r<table->rowCount(); ++r)
        if(getClientIndexByRow(t, r) == i)
            return r;
    return -1;
}

ClientIndex StatisticDialog::getClientIndexByRow(Table t, int r) const
{
    auto table = getTableWidget(t);
    int col = getTableIdCol(t);

    auto item = table->item(r, col);
    return item ? item->data(Qt::UserRole).value<ClientIndex>() : ClientIndex();
}

void StatisticDialog::updateAnalStat()
{
    QTableWidget *table = ui->tableWidgetAnalyzer;
    table->setUpdatesEnabled(false);
    for(const auto &s : analyzerStats) {
        int row = getRowByClientIndex(Table::Anal, s.index);
        if(row==-1)
            continue;

        table->item(row, COL_STAT_A_TOT_SIZE)->setText(QString::number(s.totalSize));
        table->item(row, COL_STAT_A_CORR_SIZE)->setText(QString::number(s.corruptedSize));
        table->item(row, COL_STAT_A_BAD_SYNC)->setText(QString::number(s.badSync));
        table->item(row, COL_STAT_A_EV_SIZE)->setText(QString::number(s.evSize));
        table->item(row, COL_STAT_A_EV_CNT)->setText(QString::number(s.evCnt));
        table->item(row, COL_STAT_A_PERIOD_EV_SIZE)->
            setText(s.periodCnt ? QString::number(s.periodSize/s.periodCnt) : "-");
    }
    analyzerStats.clear();

    updateTableSize(table);
    table->setUpdatesEnabled(true);
}

void StatisticDialog::updateTcpStat()
{
    QTableWidget *table = ui->tableWidgetClient;
    table->setUpdatesEnabled(false);
    for (auto s : tcpStats) {
        int row = getRowByClientIndex(Table::Tcp, s.index);
        if(row==-1)
            continue;

        table->item(row, COL_STAT_T_RCV_BUFF_SIZE)->setText(QString("%L1 B").arg(s.rcvBufSize));
        table->item(row, COL_STAT_T_READ_BUFFER_SIZE)->setText(QString("%L1 B").arg(s.readBufferSize));
        table->item(row, COL_STAT_T_AVAIL_SIZE)->setText(QString("%L1 B").arg(s.bytesAvailable));

        table->item(row, COL_STAT_T_FREE_PART)->setText(QString::number(s.freePart, 'f', 1));
        table->item(row, COL_STAT_T_DATA_PART)->setText(QString::number(s.dataPart, 'f', 1));
        table->item(row, COL_STAT_T_TRASH_PART)->setText(QString::number(s.trashPart, 'f', 1));
    }

    updateTableSize(table);
    table->setUpdatesEnabled(true);
}
