#ifndef STATISTICDIALOG_H
#define STATISTICDIALOG_H

#include <QDialog>
#include <QMap>

#include "Analizer.h"
#include "Combiner.h"
#include "TcpClient.h"
class QTableWidget;

namespace Ui {
class StatisticDialog;
}

class StatisticDialog : public QDialog
{
    Q_OBJECT
    enum class Table {
        Tcp,
        Anal,
        Writer
    };

public:
    explicit StatisticDialog(QWidget *parent = nullptr);
    ~StatisticDialog();

public slots:
    virtual void setVisible(bool visible);
    void analizerListUpdated(const ClientIndexSet &clientIds);
    void analyzerStatUpdated(const AnalyzerStat &s);
    void updateTcpStatistic(const ClientIndex &index, const TcpStatistic &statistic);
    void updateCombinerStats(const CombinerStats &_combStat);

private slots:
    void updateStat();

private:
    void init();
    void updateTableSize(QTableWidget *table);
    QTableWidget * getTableWidget(Table t) const;
    int getTableIdCol(Table t) const;
    int getRowByClientIndex(Table t, const ClientIndex &i) const;
    ClientIndex getClientIndexByRow(Table t, int r) const;
    void updateAnalStat();
    void updateTcpStat();

    Ui::StatisticDialog *ui;
    ClientIndexSet curClientIds;
    QMap<ClientIndex, AnalyzerStat> analyzerStats;
    QMap<ClientIndex, TcpStatistic> tcpStats;
    CombinerStats combStat;
    QTimer * const refreshTimer;
};

#endif // STATISTICDIALOG_H
