#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include "base/ClientIndex.h"
#include "TcpClientFsm.h"
#include <QHostAddress>
template<class T> class RingBuffer;
class QTcpSocket;

class Analizer;
class AnalyzerStat;
class QHostInfo;
class QSemaphore;
class QTimer;
class ProgramDescription;
class ProgramInterface;
class DnsLookup;

struct TcpStatistic {
    ClientIndex index;
    QString name;
    float freePart;
    float dataPart;
    float trashPart;
    qint64 rcvBufSize;
    qint64 readBufferSize;
    qint64 bytesAvailable;
};

Q_DECLARE_METATYPE(QHostAddress)
Q_DECLARE_METATYPE(TcpStatistic)

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(ClientIndex i, RingBuffer<char> *byteRB);
    virtual ~TcpClient();

    QString getHostAddr() const { return hostAddr.toString(); }
    int getHostPort() const { return hostPort; }
    BaseFsm *getFsm() const { return fsm;}

public slots:
    void stopReadOut();
    void startReadOut();
    void startRun();
    void stopRun();
    void setAddress(const ProgramDescription &descr, const ProgramInterface &interface);
    void clientDescrChanged(const QString &descr);
    void analyzerStatUpdated(const AnalyzerStat &aStat);

private slots:
    void connectToHost();
    void readData();
    void displayError(QAbstractSocket::SocketError socketError);
    void connected();
    void disconnected();
    void doSendMetrics();
    void updateStatus();
    void sendStatus_(ClientIndex index, bool isConnected, QString state);
    void gotHostInfo(const QHostInfo &hostInfo);
    void resizeBuffer();
    void low_space_timer_slot();
    void onFsmStateChanged(const FsmState &cur);

signals:
    void dataAcquired(ClientIndex index, QByteArray);
    void sendStatus(ClientIndex index, bool isConnected, QString state);
    void sendStatistic(ClientIndex index, const TcpStatistic &statistic);
    void updateRunState(bool isAnalizer, bool isInRunMode);
    void gotError(QString errStr="");
    void inputBufferSizeUpdated(ClientIndex index, quint64 size) const;

private:
    void disconnectFromHost();
    bool isConnectionAlive() const;
    QString getName() const;
    void resetReadOut();

    ClientIndex index;
    QString clientDescr;
    bool readoutEnabled;
    QTcpSocket *tcpSocket;
    DnsLookup * const dnsLookup;
    QHostAddress hostAddr;
    QString hostName;
    quint32 hostPort;
    QTimer * const reconnectTimer;
    QTimer * const delayReadTimer;
    QTimer * const statusTimer;
    QTimer * const sendMetricsTimer;
    QTimer * const resizeTimer;
    QTimer * low_space_timer;
    int connectingAttempts;
    quint64 packageCounter;

    RingBuffer<char> *byteRB;
    bool noFreeSpace;
    TcpStatistic statistic;
    int loopRecivedSize;
    bool srcBusy;
    bool waitingResize;
    int preferedBufSize;
    TcpClientFsm * const fsm;
};

#endif // TCPCLIENT_H
