#include "TcpClientFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.tcpClient")
}

TcpClientFsm::TcpClientFsm(QObject *parent)
    :BaseFsm(parent)
{
    setObjectName(__func__);
    recreateInput(new TcpClientFsmInput());
}

void TcpClientFsm::setConnected(bool connected)
{
    getInput()->connected = connected;
    evaluate();
}

void TcpClientFsm::setClientEn(bool clientEn)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << clientEn;
    getInput()->clientEn = clientEn;
    evaluate();
}

bool TcpClientFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(getState() != FsmState::Error) {
        if (getInput()->clientEn && !getInput()->connected) {
            setState(FsmState::Error);
            return true;
        }
    }

    switch (getState()) {
    case FsmState::Error:
        if (!getInput()->clientEn || getInput()->connected) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    case FsmState::Ready:
        if (getRequestState() == FsmState::Starting) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Starting:
        if(getRequestState() == FsmState::Run) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Run:
        if (getRequestState() == FsmState::Ready) {
            acceptRequestedState();
            return true;
        }
        break;
    default:
    {}
    }

    return false;
}

void TcpClientFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

QDebug operator<<(QDebug dbg, const TcpClientFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "connected:" << in.connected << "; ";
    dbg << "clientEn:" << in.clientEn << "; ";

    return dbg;
}
