#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "base/app/BaseMainWindow.h"

#include "MscConfig.h"
#include "pnp-server/PNPMessage.h"

#include <QMap>

class ConfigIndexName;
class ConfigSelector;
class ConfigurationManager;
struct DeviceConfig;
class DeviceDispatcher;
class DeviceIndex;
class DeviceIndexEnabledMap;
class DeviceStatus;
class DeviceWidget;
class DiscoverDialog;
class LldpInfoCache;
class MscConfigDb;
class MscConfigDispatcher;
class PNPServer;
class ProgramDescription;
class QTableWidget;

namespace Ui {
class MainWindow;
}

class MainWindow : public BaseMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(const ConfigSelector &cs, QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void change_program_description(const ProgramDescription &) const;
    void delete_all();
    void deviceListChanged(const DeviceIndexEnabledMap &);
    void configChanged(const MscConfig &);
    void switch_to_config(const ConfigIndexName &conf) const;

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &map);
    void setConfig(const MscConfig &newConfig);
    void configSelectorChanged(const ConfigIndexName &conf);

private slots:
    void add_log(QtMsgType severity, const QString &log, const QString &thrName);
    void update_device_config(const DeviceConfig &device_config);
    void update_status(const DeviceStatus &device_status);

    void on_actionConfiguration_manager_triggered();
    void on_actionFind_device_triggered();
    void on_actionSet_hit_width_extension_triggered();
    void on_actionLut_c_triggered();
    void on_pushButtonLog_clicked(bool checked);

private:
    void init();
    void init_gui();
    void populate_devices_widgets();
    void create_device_widget(DeviceConfig device_config);
    void populate_devices_table();
    void update_program_description();
    void set_window_title();
    int get_device_row(const DeviceIndex &index);
    DeviceIndex get_index_by_row(int row);
    static void update_table_size(QTableWidget *t);

    Ui::MainWindow * const ui;
    MscConfig config;
    MscConfigDb * const configDb;
    MscConfigDispatcher * const configDispatcher;
    ConfigurationManager * const configuration_manager;
    DiscoverDialog * const discoverDialog;
    DeviceDispatcher * const deviceDispatcher;
    ProgramDescription program_description;
    PNPServer * const pnp_server;
    QMap<DeviceIndex, DeviceWidget*> device_widgets;
};

#endif // MAINWINDOW_H
