/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MscConfigConverter.h"

#include "base/ConfigConverterUtil.h"
#include "base/DeviceIndexConverter.h"

namespace {
const char* CONFIG_PREFIX = "device";

const char* CHANNELS_THR = "channels_thr";
const char* CHANNELS_BASELINE = "channels_baseline";

constexpr const char *k_enabled = "enabled";
constexpr const char *k_device_configs = "device_configs";
constexpr const char *k_device_index = "device_index";
constexpr const char *k_lut_path = "lut_path";
constexpr const char *k_hit_extension = "hit_extension";

constexpr const char *k_show_log = "show_log";
}

template<>
bool AbstractConfigConverter<DeviceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DeviceConfig>::toJsonObject(const DeviceConfig &in)
{
    QVariantMap m;

    m[k_device_index] = QVariant::fromValue(in.device_index);
    m[k_enabled] = QVariant::fromValue(in.enabled);

    m[CHANNELS_THR] = ConfigConverterUtil::toJsonValue(in.channel_thr_map);
    m[CHANNELS_BASELINE] = ConfigConverterUtil::toJsonValue(in.baseLine);

    m[k_lut_path] = QVariant::fromValue(in.lut_path);
    m[k_hit_extension] = QVariant::fromValue(in.hit_extension);

    return fromVariantMap(m);
}

template<>
DeviceConfig AbstractConfigConverter<DeviceConfig>::fromJsonObject(const QJsonObject &in)
{
    DeviceConfig out;
    update_value(in, k_device_index, &out.device_index);
    update_value(in, k_enabled, &out.enabled);

    out.channel_thr_map = ConfigConverterUtil::toMap<int, double>(in[CHANNELS_THR]);
    out.baseLine = ConfigConverterUtil::toMap<int, double>(in[CHANNELS_BASELINE]);

    update_value(in, k_lut_path, &out.lut_path);
    update_value(in, k_hit_extension, &out.hit_extension);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<DeviceConfig>::toJsonValue(const DeviceConfig &in)
{
    return toJsonObject(in);
}

template<>
DeviceConfig AbstractConfigConverter<DeviceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DeviceConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
}


template<>
bool AbstractConfigConverter<MscConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<MscConfig>::toJsonObject(const MscConfig &in)
{
    QVariantMap m;

    m[k_hit_extension] = QVariant::fromValue(in.hit_extension);
    m[k_show_log] = QVariant::fromValue(in.show_log);
    m[k_device_configs] = ConfigConverterUtil::toJsonValue(in.device_configs);

    return fromVariantMap(m);
}

template<>
MscConfig AbstractConfigConverter<MscConfig>::fromJsonObject(const QJsonObject &in)
{
    MscConfig out;

    update_value(in, k_hit_extension, &out.hit_extension);
    update_value(in, k_show_log, &out.show_log);

    out.device_configs = ConfigConverterUtil::toMap<DeviceIndex, DeviceConfig>(
                in[k_device_configs]);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<MscConfig>::toJsonValue(const MscConfig &in)
{
    return toJsonObject(in);
}

template<>
MscConfig AbstractConfigConverter<MscConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<MscConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<DeviceConfig>::registerTypes();
}
