/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MSCCONFIGDISPATCHER_H
#define MSCCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"

#include "MscConfig.h"

class MscConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit MscConfigDispatcher(QObject *parent = nullptr);

    virtual void sendCache() const override {
        BaseConfigDispatcher::sendCache();
        emit configChanged(*cache);
    }

    template<class T>
    void subscribe(T *obj) {
            connect(this, &MscConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &MscConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            qDebug() << "Config:"
                     << (objectName().isEmpty() ? metaObject()->className() : objectName())
                     << "<->"
                     << (obj->objectName().isEmpty() ? obj->metaObject()->className() : obj->objectName());
    }

signals:
    void configChanged(const MscConfig &) const;

public slots:
    void setConfig(const MscConfig &newConfig);

private:
    std::unique_ptr<MscConfig> cache;
};

#endif // MSCCONFIGDISPATCHER_H
