/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RcConfigConverter.h"

#include "base/ClientConfigConverter.h"
#include "base/ConfigConverterUtil.h"

//using RcProgramConfigConverter = AbstractConfigConverter<RcProgramConfig>;
//using TrigConfigConverter = AbstractConfigConverter<TrigConfig>;
//using RcTrigConfigConverter = AbstractConfigConverter<RcTrigConfig>;
//using ClientConfigConverter = AbstractConfigConverter<ClientConfig>;
//using RcClientMapConverter = AbstractConfigConverter<RcClientMap>;
//using RcXoffChannelDescrConverter = AbstractConfigConverter<RcXoffChannelDescr>;
//using RcVmeTrigsConverter = AbstractConfigConverter<RcVmeTrigs>;


template<>
bool AbstractConfigConverter<RcProgramConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcProgramConfig>::toJsonObject(const RcProgramConfig &in)
{
    QVariantMap m;
    m["runEnable"] = QVariant::fromValue(in.runEnable);
    m["runId"] = QVariant::fromValue(in.runId);
    m["localRunNumber"] = QVariant::fromValue(in.localRunNumber);
    m["allowRemoteControl"] = QVariant::fromValue(in.allowRemoteControl);
    m["enableStatusDump"] = QVariant::fromValue(in.enableStatusDump);
    m["statusDumpDirName"] = QVariant::fromValue(in.statusDumpDirName);
    return fromVariantMap(m);
}

template<>
RcProgramConfig AbstractConfigConverter<RcProgramConfig>::fromJsonObject(const QJsonObject &m)
{
    RcProgramConfig out;
    out.runEnable = QVariant::fromValue(m["runEnable"]).value<bool>();
    out.runId = QVariant::fromValue(m["runId"]).value<RcRunId>();
    out.localRunNumber = QVariant::fromValue(m["localRunNumber"]).value<int>();
    out.allowRemoteControl = QVariant::fromValue(m["allowRemoteControl"]).value<bool>();
    out.enableStatusDump = QVariant::fromValue(m["enableStatusDump"]).value<bool>();
    out.statusDumpDirName = QVariant::fromValue(m["statusDumpDirName"]).value<QString>();

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcProgramConfig>::toJsonValue(const RcProgramConfig &in)
{
    return toJsonObject(in);
}

template<>
RcProgramConfig AbstractConfigConverter<RcProgramConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcProgramConfig>::registerSubTypes()
{
    AbstractConfigConverter<RcRunId>::registerTypes();
}

// ---

template<>
bool AbstractConfigConverter<TrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TrigConfig>::toJsonObject(const TrigConfig &in)
{
    QVariantMap m;
    m["xOffMask"] = QVariant::fromValue(in.xOffMask);
    m["trigOpen"] = QVariant::fromValue(in.trigOpen);
    m["trigSrc"] = QVariant::fromValue(in.trigSrc);
    m["trigTimerFreq"] = QVariant::fromValue(in.trigTimerFreq);
    m["trigRandomFreq"] = QVariant::fromValue(in.trigRandomFreq);
    m["trigRandomDeadTimeUs"] = QVariant::fromValue(in.trigRandomDeadTimeUs);
    return fromVariantMap(m);
}

template<>
TrigConfig AbstractConfigConverter<TrigConfig>::fromJsonObject(const QJsonObject &m)
{
    TrigConfig data;
    data.xOffMask = QVariant::fromValue(m["xOffMask"]).value<quint16>();
    data.trigOpen = QVariant::fromValue(m["trigOpen"]).value<bool>();
    data.trigSrc = QVariant::fromValue(m["trigSrc"]).value<quint16>();
    data.trigTimerFreq = QVariant::fromValue(m["trigTimerFreq"]).value<double>();
    data.trigRandomFreq = QVariant::fromValue(m["trigRandomFreq"]).value<double>();
    data.trigRandomDeadTimeUs = QVariant::fromValue(m["trigRandomDeadTimeUs"]).value<double>();

    return data;
}

template<>
QJsonValue AbstractConfigConverter<TrigConfig>::toJsonValue(const TrigConfig &in)
{
    return toJsonObject(in);
}

template<>
TrigConfig AbstractConfigConverter<TrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TrigConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<SpillConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<SpillConfig>::toJsonObject(const SpillConfig &in)
{
    QVariantMap m;
    m["durPhase1"] = QVariant::fromValue(in.durPhase1);
    m["durPhase2"] = QVariant::fromValue(in.durPhase2);
    m["durPhase3"] = QVariant::fromValue(in.durPhase3);
    m["durPhase4"] = QVariant::fromValue(in.durPhase4);
    m["durPhase5"] = QVariant::fromValue(in.durPhase5);
    m["source"] = QVariant::fromValue(static_cast<int>(in.source));
    m["softSpill"] = QVariant::fromValue(in.softSpill);
    m["timerPeriod"] = QVariant::fromValue(in.timerPeriod);
    m["spillPlotStep"] = QVariant::fromValue(in.spillPlotStep);
    m["trigTimeStep"] = QVariant::fromValue(in.trigTimeStep);
    return fromVariantMap(m);
}

template<>
SpillConfig AbstractConfigConverter<SpillConfig>::fromJsonObject(const QJsonObject &m)
{
    SpillConfig out;
    out.durPhase1 = QVariant::fromValue(m["durPhase1"]).value<quint32>();
    out.durPhase2 = QVariant::fromValue(m["durPhase2"]).value<quint32>();
    out.durPhase3 = QVariant::fromValue(m["durPhase3"]).value<quint32>();
    out.durPhase4 = QVariant::fromValue(m["durPhase4"]).value<quint32>();
    out.durPhase5 = QVariant::fromValue(m["durPhase5"]).value<quint32>();
    out.source = static_cast<SpillConfig::spill_sel_t>(QVariant::fromValue(m["source"]).value<int>());
    out.softSpill = QVariant::fromValue(m["softSpill"]).value<bool>();
    out.timerPeriod = QVariant::fromValue(m["timerPeriod"]).value<quint32>();
    out.spillPlotStep = QVariant::fromValue(m["spillPlotStep"]).value<quint32>();
    out.trigTimeStep = QVariant::fromValue(m["trigTimeStep"]).value<quint32>();
    return out;
}

template<>
QJsonValue AbstractConfigConverter<SpillConfig>::toJsonValue(const SpillConfig &in)
{
    return toJsonObject(in);
}

template<>
SpillConfig AbstractConfigConverter<SpillConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<SpillConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<RcTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcTrigConfig>::toJsonObject(const RcTrigConfig &in)
{
    QVariantMap m;
    m["deviceIndex"] = QVariant::fromValue(in.deviceIndex);
    m["checkLvdsParity"] = QVariant::fromValue(in.checkLvdsParity);
    m["checkBeforeProtection"] = QVariant::fromValue(in.checkBeforeProtection);
    m["checkAfterProtection"] = QVariant::fromValue(in.checkAfterProtection);
    m["bapBlock"] = QVariant::fromValue(in.bapBlock);
    m["trigRejectDelay"] = QVariant::fromValue(in.trigRejectDelay);
    m["beforeProtWidth"] = QVariant::fromValue(in.beforeProtWidth);
    m["afterProtWidth"] = QVariant::fromValue(in.afterProtWidth);
    m["lvdsMask"] = QVariant::fromValue(in.lvdsMask);
    m["trigWordMask"] = QVariant::fromValue(in.trigWordMask);
    m["trig"] = QVariant::fromValue(in.trig);
    m["spill"] = QVariant::fromValue(in.spill);
    m["plotSrc"] = QVariant::fromValue(static_cast<int>(in.plotSrc));
    return fromVariantMap(m);
}

template<>
RcTrigConfig AbstractConfigConverter<RcTrigConfig>::fromJsonObject(const QJsonObject &m)
{
    RcTrigConfig out;
    out.deviceIndex = QVariant::fromValue(m["deviceIndex"]).value<DeviceIndex>();
    out.checkLvdsParity = QVariant::fromValue(m["checkLvdsParity"]).value<bool>();
    out.checkBeforeProtection = QVariant::fromValue(m["checkBeforeProtection"]).value<bool>();
    out.checkAfterProtection = QVariant::fromValue(m["checkAfterProtection"]).value<bool>();
    out.bapBlock = QVariant::fromValue(m["bapBlock"]).value<bool>();
    out.trigRejectDelay = QVariant::fromValue(m["trigRejectDelay"]).value<quint16>();
    out.beforeProtWidth = QVariant::fromValue(m["beforeProtWidth"]).value<quint16>();
    out.afterProtWidth = QVariant::fromValue(m["afterProtWidth"]).value<quint16>();
    out.lvdsMask = QVariant::fromValue(m["lvdsMask"]).value<quint16>();
    out.trigWordMask = QVariant::fromValue(m["trigWordMask"]).value<quint16>();
    out.trig = QVariant::fromValue(m["trig"]).value<TrigConfig>();
    out.spill = QVariant::fromValue(m["spill"]).value<SpillConfig>();
    out.plotSrc = static_cast<quint8>(QVariant::fromValue(m["plotSrc"]).value<int>());

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcTrigConfig>::toJsonValue(const RcTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
RcTrigConfig AbstractConfigConverter<RcTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcTrigConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<TrigConfig>::registerTypes();
    AbstractConfigConverter<SpillConfig>::registerTypes();
}

// ---

template<>
bool AbstractConfigConverter<VmeTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<VmeTrigConfig>::toJsonObject(const VmeTrigConfig &in)
{
    QVariantMap m;
    m["enabled"] = QVariant::fromValue(in.enabled);
    m["evNumReadReset"] = QVariant::fromValue(in.evNumReadReset);
    m["checkWr"] = QVariant::fromValue(in.checkWr);
    return fromVariantMap(m);
}

template<>
VmeTrigConfig AbstractConfigConverter<VmeTrigConfig>::fromJsonObject(const QJsonObject &m)
{
    VmeTrigConfig out;
    out.enabled = QVariant::fromValue(m["enabled"]).value<bool>();
    out.evNumReadReset = QVariant::fromValue(m["evNumReadReset"]).value<bool>();
    out.checkWr = QVariant::fromValue(m["checkWr"]).value<bool>();
    return out;
}

template<>
QJsonValue AbstractConfigConverter<VmeTrigConfig>::toJsonValue(const VmeTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
VmeTrigConfig AbstractConfigConverter<VmeTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<VmeTrigConfig>::registerSubTypes()
{
}

// ---

template<>
bool AbstractConfigConverter<RcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcConfig>::toJsonObject(const RcConfig &in)
{
    QVariantMap m;
    m["pc"] = QVariant::fromValue(in.pc);
    m["rcTrigConfig"] = QVariant::fromValue(in.rcTrigConfig);
    m["clients"] = ConfigConverterUtil::toJsonValue(in.clients);
    m["channelDescr"] = ConfigConverterUtil::toJsonValue(in.channelDescr);
    m["vmeTrigs"] = ConfigConverterUtil::toJsonValue(in.vmeTrigs);
    return fromVariantMap(m);
}

template<>
RcConfig AbstractConfigConverter<RcConfig>::fromJsonObject(const QJsonObject &m)
{
    RcConfig out;
    out.pc = QVariant::fromValue(m["pc"]).value<RcProgramConfig>();
    out.rcTrigConfig = QVariant::fromValue(m["rcTrigConfig"]).value<RcTrigConfig>();
    out.clients = ConfigConverterUtil::toMap<int, ClientConfig>(m["clients"]);
    out.channelDescr = ConfigConverterUtil::toMap<int, QString>(m["channelDescr"]);
    out.vmeTrigs = ConfigConverterUtil::toMap<DeviceIndex, VmeTrigConfig>(m["vmeTrigs"]);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<RcConfig>::toJsonValue(const RcConfig &in)
{
    return toJsonObject(in);
}

template<>
RcConfig AbstractConfigConverter<RcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcConfig>::registerSubTypes()
{
    AbstractConfigConverter<RcProgramConfig>::registerTypes();
    AbstractConfigConverter<TrigConfig>::registerTypes();
    AbstractConfigConverter<RcTrigConfig>::registerTypes();
    AbstractConfigConverter<ClientConfig>::registerTypes();
    AbstractConfigConverter<VmeTrigConfig>::registerTypes();
}

