/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RcConfigDispatcher.h"

#include <QLoggingCategory>

#include "RcConfig.h"
#include "base/util.h"

namespace {

Q_DECLARE_LOGGING_CATEGORY(config)
Q_LOGGING_CATEGORY(config, "afi.rc.config")

} // namespace

RcConfigDispatcher::RcConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new RcConfig())
{
}

RcConfigDispatcher::~RcConfigDispatcher()
{
    delete cache;
}

void RcConfigDispatcher::setConfig(const RcConfig &newConfig)
{
    if (newConfig == *cache)
        return;
    if (!cache->rcTrigConfig.deviceIndex.isNull() && newConfig.rcTrigConfig.deviceIndex.isNull()) {
        qDebug(config) << "device removed:" << cache->rcTrigConfig.deviceIndex.getIdent();
        qDebug(config) << "configChanged by" << qObjectAncestryStr(sender());
    }
    *cache = newConfig;
//    qDebug(config).space() << newConfig;
    emit configChanged(newConfig);
}
