/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCSIGNALROUTER_H
#define RCSIGNALROUTER_H

#include <QObject>

class ClientManagerWidget;
class ConfigurationManager;
class DeviceDescription;
class DeviceDispatcher;
class DeviceIndex;
class DiscoverDialog;
class EvNumChecker;
class LldpDiscovery;
class MainWindow;
class PNPServer;
class RcConfigDb;
class RcConfigDispatcher;
class RcCoreApp;
class RemoteControlServer;
class SlowControlServer;
class TrigDevice;
class TrigDeviceController;
class TrigMainForm;
class VmeTrigsController;
class RcFsm;

class RcSignalRouter : public QObject
{
    Q_OBJECT
public:
    // Singleton
    static RcSignalRouter & instance()
    {
        // thread-safe since C++11
        static RcSignalRouter obj;
        obj.setObjectName("RcSignalRouter");
        return obj;
    }

    // delete copy and move constructors and assign operators
    RcSignalRouter(RcSignalRouter const &) = delete;
    RcSignalRouter(RcSignalRouter &&) = delete;
    RcSignalRouter &operator=(RcSignalRouter const &) = delete;
    RcSignalRouter &operator=(RcSignalRouter &&) = delete;

    void addRcCoreApp(RcCoreApp *p)
    { rcCoreApp = p; }

    void addClientManagerWidget(ClientManagerWidget *p)
    { clientManagerWidget = p; }

    void addEvNumChecker(EvNumChecker *p)
    { evNumChecker = p; }

    void addMainWindow(MainWindow *p)
    { mainWindow = p; }

    void addConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }

    void addPnpServer(PNPServer *p)
    { pnpServer = p; }

    void addRcConfigDb(RcConfigDb *p)
    { configDb = p; }

    void addRcConfigDispatcher(RcConfigDispatcher *p)
    { rcConfigDispatcher = p; }

    void addRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }

    void addSlowControlServer(SlowControlServer *p)
    { slowControlServer = p; }

    void addTrigMainForm(TrigMainForm *p)
    { trigMainForm = p; }

    void addTrigDeviceController(TrigDeviceController *p)
    { trigDeviceController = p; }

    void addVmeTrigsController(VmeTrigsController *p)
    { vmeTrigsController = p; }

    void addRcFsm(RcFsm *p)
    { rcFsm = p; }

    void removeRemoteControlServer();

    void connectAll();
    void connect_TrigDevice(TrigDevice *trigDevice);

signals:
    void deviceDiscovered(const DeviceDescription &dd);
    void deviceDiscoveredAll(const DeviceDescription &dd);
    void deviceExpired(const DeviceIndex &i);

private:
    RcSignalRouter();

    void connect_ClientManagerWidget();
    void connect_Configuration();
    void connect_EvNumChecker();
    void connect_EvNumChecker(const TrigDevice *trigDevice);
    void connect_MainWindow();
    void connect_PnpServer();
    void connect_RemoteControlServer();
    void connect_SlowControlServer();
    void connect_TrigMainForm();
    void connect_TrigDeviceController();
    void connect_VmeTrigsController();

    RcCoreApp *rcCoreApp = nullptr;
    RcConfigDb * configDb = nullptr;
    ClientManagerWidget *clientManagerWidget = nullptr;
    RcConfigDispatcher * rcConfigDispatcher = nullptr;
    TrigDeviceController * trigDeviceController = nullptr;
    EvNumChecker *evNumChecker = nullptr;
    MainWindow *mainWindow = nullptr;
    ConfigurationManager *configurationManager = nullptr;
    PNPServer *pnpServer = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    SlowControlServer *slowControlServer = nullptr;
    TrigMainForm *trigMainForm = nullptr;
    VmeTrigsController *vmeTrigsController = nullptr;
    RcFsm *rcFsm = nullptr;
};

#endif // RCSIGNALROUTER_H
