/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGDEVICECONTROLLERPRIVATE_H
#define TRIGDEVICECONTROLLERPRIVATE_H

#include <QObject>

#include "RcConfig.h"
#include "base/ThreadController.h"
#include "TrigDevice.h"
#include "fsm/FsmState.h"

class TrigDeviceMgrFsm;

class TrigDeviceProxy
{
public:
    TrigDeviceProxy(TrigDevice *dev)
        : tc(new ThreadController(dev, nullptr)) { }
private:
    QSharedPointer<ThreadController> tc;
};

class TrigDeviceController;

class TrigDeviceControllerPrivate : public QObject
{
    Q_OBJECT
public:
    explicit TrigDeviceControllerPrivate(TrigDeviceController *p = nullptr, QObject *parent = nullptr);

    RcConfig config;
    QMap<DeviceIndex, TrigDeviceProxy> trigThreads;
    QMap<DeviceIndex, TrigStatus> trigStatus;

signals:
    void writeSetup(const RcTrigConfig &cfg, int setupKey) const;
    void printFsmInput();
    void preRunReset();

private slots:
    void onFsmStateChange(const FsmState &state);
    void setTrigStatus(const TrigStatus &status);
    void setupWritten(const DeviceIndex &index, int setupSeqNum);

private:
    void incrementSetupKey();
    void onTrigDevConfigUpdate();
    void updateDeviceSet();
    void onOpenGlobTrig();
    void setTrigState(bool open);

    Q_DECLARE_PUBLIC(TrigDeviceController);
    TrigDeviceController * const q_ptr;
    TrigDeviceMgrFsm * const fsm;
    int setupKey = 100;
    bool megamaster_flg = false;
    bool lookingForStopEvNum = false;
    quint64 stopEvNumber = 0;
};

#endif // TRIGDEVICECONTROLLERPRIVATE_H
