//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef VMETRIGDEVICE_H
#define VMETRIGDEVICE_H

#include "mregdev/qtmregdevice.h"

class VmeTrigStatus
{
public:
    bool isWrTimeValid() const { return deviceStatus & 0x1; }
    void clear() {
        temp  = 0;
        evNum = 0;
        wrSyncFail  = 0;
        wrLinkError = 0;
        deviceStatus= 0;
        busyTimer= 0;
        idleTimer= 0;
    }
    QString ident;
    QString statusText;
    bool liveMagikOk = false;
    bool connected = false;
    double temp = 0;
    quint64 evNum = 0;
    quint32 wrSyncFail = 0;
    quint32 wrLinkError = 0;
    quint16 deviceStatus = 0;
    quint64 busyTimer = 0;
    quint64 idleTimer = 0;
};

Q_DECLARE_METATYPE(VmeTrigStatus)

inline bool operator==(const VmeTrigStatus &l, const VmeTrigStatus &r)
{
    return
        l.ident == r.ident &&
        l.statusText == r.statusText &&
        l.liveMagikOk == r.liveMagikOk &&
        l.connected == r.connected &&
        l.temp == r.temp &&
        l.evNum == r.evNum &&
        l.wrSyncFail == r.wrSyncFail &&
        l.wrLinkError == r.wrLinkError &&
        l.deviceStatus == r.deviceStatus &&
        l.busyTimer == r.busyTimer &&
        l.idleTimer == r.idleTimer;
}

class VmeTrigDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit VmeTrigDevice(const DeviceIndex &serId, QObject *parent = nullptr);
    ~VmeTrigDevice();

signals:
    void updateStatus(const DeviceIndex &deviceIndex, const VmeTrigStatus &status);
    void evNumResponse(int checkIndex, quint64 deviceIndex, quint64 evNum) const;
    void runStateUpdated(const DeviceIndex &deviceIndex, int setupKey);
    void deviceConnected(const DeviceIndex &deviceIndex);
public slots:
//    void deleteTriger(const DeviceIndex &deviceIndex); // serial==0 - for all
    virtual void statusPoll();
    void resetEvNum(int setupKey);
    void vmeReadoutUpdated(const DeviceIndex &deviceIndex, bool _evNumReadReset);
    void evNumRequest(int checkIndex);

protected Q_SLOTS:
    virtual void on_deviceConnected() override;
    virtual void on_deviceDisconnected() override;


private:
    VmeTrigStatus status;
    DeviceIndex deviceIndex;
    bool evNumReadReset;
};

#endif // VMETRIGDEVICE_H
