#include "VmeTrigFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
    Q_DECLARE_LOGGING_CATEGORY(fsm)
    Q_LOGGING_CATEGORY(fsm, "afi.fsm.vme_device")
}

VmeTrigFsm::VmeTrigFsm(QObject *p):
    DeviceMgrFsm(p)
{
    setObjectName("VmeTrigFSM");
    recreateInput(new VmeTrigFsmInput());
}

void VmeTrigFsm::setWrState(const DeviceIndex &index, bool ok)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << ok;
    auto it = getInput()->wrOk.find(index);
    if(it == getInput()->wrOk.end())
        getInput()->wrOk[index] = ok;
    else if(*it != ok)
        *it = ok;
    else
        return;
    evaluate();
}

void VmeTrigFsm::setWrCheckSet(DeviceIndexSet wrCheckSet)
{
    getInput()->wrCheckSet = wrCheckSet;
    evaluate();
}

bool VmeTrigFsm::evaluate()
{
    bool wrOk = allWrOk();
    if(!wrOk) {
        setState(FsmState::Error);
        return true;
    }

    if(DeviceMgrFsm::evaluate())
        return true;

    switch (getState()) {
    case FsmState::Error:
        if (wrOk) {
            setState(FsmState::Ready);
            return true;
        }
        break;
    case FsmState::Starting:
        if(allWrOk() && getRequestState() == FsmState::Run){
            acceptRequestedState();
            return true;
        }
    default:;
    }

    return false;
}

void VmeTrigFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool VmeTrigFsm::allWrOk()
{
    for(auto devIndex : getInput()->wrCheckSet) {
        if(!getInput()->wrOk.value(devIndex, false))
            return false;
    }
    return true;
}

bool VmeTrigFsm::transitionAllowed(FsmState st1, FsmState st2)
{
//    bool ret = DeviceMgrFsm::transitionAllowed(st1, st2);
//    if(st2==FsmState::Error)
//        return ret || !allWrOk();

//    if(ret) {
//        switch (st1)  {
//        case FsmState::Error:
//        {
//            switch (st2)  {
//            case FsmState::Ready:
//                return allWrOk();
//            default:;
//            }
//            break;
//        }
//        case FsmState::Ready:

//            switch (st2)  {
//            case FsmState::Starting:
//                return allWrOk();
//            default:;
//            }
//            break;
//        case FsmState::Starting:
//            if(st2 == FsmState::Run)
//                return allWrOk();
//            break;
//        case FsmState::Run:
//            if(st2 == FsmState::Error)
//                return !allDevicesWritten() && setupKeyGuargExpired();
//            if(st2 == FsmState::Starting)
//                return getRequestState() == FsmState::Stopping;
//            break;
//        case FsmState::Stopping:
//            if(getRequestState() == FsmState::Ready
//                    && allDevicesWritten()) {
//                acceptRequestedState();
//                return true;
//            }
//            break;
//        default:;
//        }
//    }

//    return ret;
    return true;
}

QDebug operator<<(QDebug dbg, const VmeTrigFsmInput &in)
{
    dbg << DeviceMgrFsmInput(in);

    dbg << "wrCheckSet:" << in.wrCheckSet << "; ";
    dbg << "wrOk:" << in.wrOk << "; ";

    return dbg;
}
