/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VMETRIGSCONTROLLERPRIVATE_H
#define VMETRIGSCONTROLLERPRIVATE_H

#include <memory>

#include "base/DeviceIndexEnabled.h"
#include "base/RcRunId.h"
#include "fsm/FsmState.h"
#include "pnp-server/PNPMessage.h"
#include "run-control-client-manager/RunStates.h"

class DeviceDispatcher;
class RcConfigDispatcher;
class RcConfig;
class VmeTrigConfig;
class VmeTrigStatus;
class VmeTrigFsm;
class VmeTrigDeviceProxy;
class VmeTrigsController;

class VmeTrigsControllerPrivate : public QObject
{
    Q_OBJECT
public:
    VmeTrigsControllerPrivate(VmeTrigsController *p, QObject *parent = nullptr);
    ~VmeTrigsControllerPrivate();

signals:
    void printFsmInput();

private slots:
    void setStatus(const DeviceIndex &index, const QString &status);
    void trigStatusUpdated(const DeviceIndex &index, const VmeTrigStatus &status);
    void trigRunStateUpdated(const DeviceIndex &serial, int setupKey);
    void onFsmStateChange(const FsmState &state, const FsmState &);
    void deviceConnected(const DeviceIndex &index);

public:
    DeviceIndexEnabledMap myDevices() const;
    static DeviceIndexEnabledMap deviceListFromConfig(const RcConfig &newConfig);
    void addTrigDevice(const DeviceIndexEnabled &die);
    void removeTrigDevice(const DeviceIndex &index);
    void addTrigThread(const DeviceIndex &index, const VmeTrigConfig &vc);
    void removeTrigThread(const DeviceIndex &index);
    void sendVmeMap() const;
private:
    void incrementSetupKey();

    Q_DECLARE_PUBLIC(VmeTrigsController)
    VmeTrigsController * const q_ptr;
    std::unique_ptr<RcConfig> config;
    VmeTrigFsm * const fsm;
    QMap<DeviceIndex, VmeTrigDeviceProxy> trigThreads;
    QMap<DeviceIndex, VmeTrigStatus> trigStatus;
    QMap<DeviceIndex, int> trigSetupKey;

    RcRunId last_got_run_id;

    QMap<QUuid, ProgramDescription> programMap;
    bool controlEnabled = true;
    int setupKey = 100;
};  

#endif // VMETRIGSCONTROLLERPRIVATE_H
