/*
**    Copyright 2014-2020 Ivan Filippov
**    Copyright 2014-2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <iostream>

#include <QDebug>
#include <QString>

#include "MainWindow.h"
#include "base/app/RunGuard.h"
#include "metric-sender/MetricSender.h"
#include "mongo/PrintConfigMap.h"
#include "base/Globals.h"
#include "RunControlApp.h"

int main(int argc, char *argv[])
{
    RunControlApp daqApp(argc, argv);
    daqApp.init();

    if(daqApp.hasConfigListOpt())
        printConfigMap(daqApp.getType());

    int ret = 1;
    try {
        const auto cs = daqApp.getConfigSelector();
        RunGuard::runSingleOrExit(cs);
        MetricSender::instance().install();

        MainWindow w(cs);
        w.show();
        ret = daqApp.exec();
    } catch (std::exception &e) {
        std::cerr << "Std exception: " << e.what () << std::endl;
    } catch (...){
        qCritical() << "Unknown exception";
    }
    return ret;
}
