#include "OrderDialog.h"
#include "ui_OrderDialog.h"
#include <stdio.h>
#include <stdlib.h>
#include <QPushButton>
#include <QCursor>
#include <QDebug>

#include "mldiscover/DeviceDescription.h"

#include <QShortcut>
#include <QKeySequence>

namespace {
enum ORDER_COLS {ORDER_COL_ID, ORDER_COL_EN, ORDER_COL_SLOT, ORDER_COL_OLD_ID, ORDER_COL_CUR_ID, ORDER_COL_UP, ORDER_COL_DOWN, ORDER_COL_NUM};
}

OrderDialog::OrderDialog(TdcConfig *config, QMap<DeviceIndex, DeviceDescription> _knownDevDescr, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::OrderDialog),
    config(config),
    knownDevDescr(_knownDevDescr)
{
    ui->setupUi(this);
    QTableWidget *table = ui->tableWidget;
    table->setColumnCount(ORDER_COL_NUM);
    table->setHorizontalHeaderItem(ORDER_COL_ID, new QTableWidgetItem("Id"));
    table->setHorizontalHeaderItem(ORDER_COL_EN, new QTableWidgetItem("En"));
    table->setHorizontalHeaderItem(ORDER_COL_SLOT, new QTableWidgetItem("Slot"));
    table->setHorizontalHeaderItem(ORDER_COL_OLD_ID, new QTableWidgetItem("Old id"));
    table->setHorizontalHeaderItem(ORDER_COL_CUR_ID, new QTableWidgetItem("Cur id"));
    table->setHorizontalHeaderItem(ORDER_COL_UP, new QTableWidgetItem("Up"));
    table->setHorizontalHeaderItem(ORDER_COL_DOWN, new QTableWidgetItem("Down"));
    table->hideColumn(ORDER_COL_UP);
    table->hideColumn(ORDER_COL_DOWN);
    table->verticalHeader()->setSectionsMovable(true);
    table->setDragEnabled(true);
    table->setDragDropMode(QAbstractItemView::InternalMove);
    table->verticalHeader()->setSectionsMovable(true);
    table->setAcceptDrops(true);

    QList<TdcDeviceConfig> devices = config->known_configs.values();
    std::sort(devices.begin(),devices.end());

    QShortcut *sc = new QShortcut(QKeySequence(Qt::ALT+Qt::Key_Up),
                                  this, SLOT(move_click()));
    sc->setProperty("up", true);
    sc = new QShortcut(QKeySequence(Qt::ALT+Qt::Key_Down),
                       this, SLOT(move_click()));
    sc->setProperty("up", false);
    this->setShortcutEnabled(sc->id(),true);

    table->setRowCount(devices.size());

    table->resizeColumnsToContents();
    const int width = qMax(table->columnWidth(ORDER_COL_UP), table->columnWidth(ORDER_COL_DOWN));
    for(int row=0; row<devices.size(); ++row) {
        TdcDeviceConfig &s = config->known_configs[devices[row].index];
        table->setItem(row, ORDER_COL_ID, new QTableWidgetItem(s.index.getSerialStr()));
        table->item(row, ORDER_COL_ID)->setData(Qt::UserRole, QVariant::fromValue(s.index));
        table->setItem(row, ORDER_COL_EN, new QTableWidgetItem(s.enabled ? "on" : "off"));
        const auto &slot_str = QString::number(knownDevDescr.value(s.index).getLldpInfo().chassis_slot);
        table->setItem(row, ORDER_COL_SLOT, new QTableWidgetItem(slot_str));
        table->setItem(row, ORDER_COL_OLD_ID, new QTableWidgetItem(QString::number(s.order_index)));
        table->setItem(row, ORDER_COL_CUR_ID, new QTableWidgetItem(QString::number(row)));
        QPushButton *up  = new QPushButton(QChar(0x2191), table);
        QPushButton *down  = new QPushButton(QChar(0x2193), table);
        up->setFixedWidth(width);
        down->setFixedWidth(width);
        connect(up, &QPushButton::clicked, this, &OrderDialog::move_click);
        connect(down, &QPushButton::clicked, this, &OrderDialog::move_click);
        table->setCellWidget(row, ORDER_COL_UP, up);
        table->setCellWidget(row, ORDER_COL_DOWN, down);
        if(!s.enabled) {
            for(int col=0; col<table->columnCount(); ++col) {
                auto i = table->item(row, col);
                if(i) {
                    auto brush = i->background();
                    brush.setColor(Qt::gray);
                    brush.setStyle(Qt::SolidPattern);
                    i->setBackground(brush);
                }
            }
        }

        s.order_index = row;
    }

#ifndef QT_DEBUG
    table->hideColumn(ORDER_COL_OLD_ID);
    table->hideColumn(ORDER_COL_CUR_ID);
#endif
    update_button_states();
    table->resizeColumnsToContents();
}

OrderDialog::~OrderDialog()
{
    delete ui;
}

void OrderDialog::move_click()
{
    QTableWidget *table = ui->tableWidget;
    const int tot_cnt = table->rowCount();
    int row=0;
    bool up = true;
    auto key_sender = qobject_cast<QShortcut*>(sender());
    if(key_sender) {
        qInfo() << row;
        row = table->currentRow();
        up = key_sender->property("up").toBool();
        qInfo() << up;

    } else {
        auto pb_sender = qobject_cast<QPushButton*>(sender());
        if(pb_sender) {
            for(; row<tot_cnt; ++row) {
                if(sender() == table->cellWidget(row, ORDER_COL_UP)) {
                    break;
                }
                if(sender() == table->cellWidget(row, ORDER_COL_DOWN)) {
                    up=false;
                    break;
                }
            }
        } else {
            return;
        }
    }
    if(row>=tot_cnt) {

        return;
    }
    if((up && row==0) || (!up && row==(tot_cnt-1)))
        return;
    int upper_row = up ? row-1 : row;
    int lower_row = upper_row+1;
    DeviceIndex upper_index = getIndexByRow(upper_row);
    DeviceIndex lower_index = getIndexByRow(lower_row);
    auto &upper = config->known_configs[upper_index];
    auto &lower = config->known_configs[lower_index];
    std::swap(upper.order_index, lower.order_index);

    // take whole rows
    QList<QTableWidgetItem*> upperItems = takeRow(table,upper_row);
    QList<QTableWidgetItem*> lowerItems = takeRow(table,lower_row);

    // set back in reverse order
    setRow(table, lower_row, upperItems);
    setRow(table, upper_row, lowerItems);
    table->item(lower_row, ORDER_COL_CUR_ID)->setText(QString::number(upper.order_index));
    table->item(upper_row, ORDER_COL_CUR_ID)->setText(QString::number(lower.order_index));
    update_button_states();
//    QCursor c = cursor();
//    c.setPos(mapToGlobal(QPoint(width() / 2, height() / 2)));
//    c.setShape(Qt::BlankCursor);
    //    setCursor(c);
}

DeviceIndex OrderDialog::getIndexByRow(int row) const
{
    const auto &item = ui->tableWidget->item(row, ORDER_COL_ID);
    return item->data(Qt::UserRole).value<DeviceIndex>();
}

void OrderDialog::update_button_states()
{
    QTableWidget *table = ui->tableWidget;
    int tot_rows = table->rowCount();
    if(tot_rows==0) return;
    table->cellWidget(0, ORDER_COL_UP)->setEnabled(false);
    table->cellWidget(tot_rows-1, ORDER_COL_DOWN)->setEnabled(false);
    if(tot_rows==1) return;
    table->cellWidget(0, ORDER_COL_DOWN)->setEnabled(true);
    table->cellWidget(tot_rows-1, ORDER_COL_UP)->setEnabled(true);
}

QList<QTableWidgetItem *> OrderDialog::takeRow(QTableWidget *t, int row)
{
    QList<QTableWidgetItem*> rowItems;
    for (int col = 0; col < t->columnCount(); ++col)
    {
        rowItems << t->takeItem(row, col);
    }
    return rowItems;
}

void OrderDialog::setRow(QTableWidget *t, int row, const QList<QTableWidgetItem *> &rowItems)
{
    for (int col = 0; col < t->columnCount(); ++col)
    {
        t->setItem(row, col, rowItems.at(col));
    }
}
