//
//    Copyright 2014 AFI Electronics
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TDCREGS_H
#define TDCREGS_H

#include "mregdev/mregdevice.h"

// Control registers
enum {
    REG_DEVICE_CTRL = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,//0x40, //64
    REG_CLOCK_STATUS = 0x56, // RO
    REG_SYS_CLK_SOURCE = 0x57, // RO
};

// Trigger control & statistic
enum {
    REG_TRIG_CSR = 0x100,
    REG_TRIG_SRC = 0x101,
    REG32_TRIG_TIMER_PERIOD = 0x102, // 32bit RW
    REG64_TRIG_EVENT_NUM_LOAD = 0x104, // 64bit RW (since 28451)
    REG32_TRIG_EVENT_NUM_LOAD = 0x104, // 32bit RW
    REG_TRIG_QUEUE = 0x106, // RO (moved to 10C on 28451)
    REG64_TRIG_CUR_EV_NUM = 0x108, // 64bit RO (since 28451)
    REG_TRIG_BUSY_STATUS = 0x109, // RO (moved to 10D on 28451)
    REG_TRIG_QUEUE_NEW = 0x10C, // 64bit RO (since 28451)
    REG_TRIG_BUSY_STATUS_NEW = 0x10D, // RO (since 28451)
//    REG64_TRIG_CNT_ALL = 0x110, // 64bit RO (deprecated on 28451)
    REG64_TRIG_CNT_CANDID = 0x114, // 64bit RO (moved to 0x700 since 28451)
    REG64_TRIG_CNT_ACCEPTED = 0x118, // 64bit RO (moved to 0x700 since 28451)
};

enum {
    REG_TRIG_CSR_BIT_CNT_RESET = 1,
//    REG_TRIG_CSR_BIT_RUN_CNT_RESET = 2,
    REG_TRIG_CSR_BIT_CNT_LOCK = 4,
};

enum {
    REG_TRIG_SRC_BIT_TIMER = 1,
    REG_TRIG_SRC_BIT_LEMO = 2,
    REG_TRIG_SRC_BIT_TTC = 4,
};

// TDC Core Registers
enum {
    REG32_TDC_CH_EN_1 = 0x200,
    REG32_TDC_CH_EN_2 = 0x202,
    REG32_TDC_GEN_CTRL = 0x220,
    REG32_TDC_READ_CTRL = 0x222,
    REG32_TDC_MEASURE_CTRL = 0x224,
    REG32_TDC_TRIG_WIN_SETUP = 0x226,
    REG32_TDC_GENERAL_STATUS = 0x22C
//    REG32_TDC_RC_ADJUST_SETUP = 0x228 //unused
};

// HPTDCs Status Registers
enum {
    REG_HPTDC1_STATUS_1 = 0x280,
    REG_HPTDC1_STATUS_2 = 0x281,
    REG_HPTDC1_STATUS_3 = 0x282,
    REG_HPTDC1_STATUS_4 = 0x283,

    REG_HPTDC2_STATUS_1 = 0x284,
    REG_HPTDC2_STATUS_2 = 0x285,
    REG_HPTDC2_STATUS_3 = 0x286,
    REG_HPTDC2_STATUS_4 = 0x287
};

enum {
    REG32_TDC_GEN_CTRL_BIT_STANDBY = 1
};

// WR Time Emulator Registers
enum {
    REG_TAI_0 = 0x400, // tai sec 15:0
    REG_TAI_1 = 0x401, // tai sec 31:16
    REG_TAI_2 = 0x402, // tai sec 39:32
    REG_TAI_UPDATE = 0x403, // any write for update
};

// Run Statistic Counters
enum {
    REG_RUN_STAT_BASE                     = 0x700,
    REG64_RUN_STAT_RUN_TIME               = 0x0,
    REG64_RUN_STAT_LEMO_Input_Pulse_Count = 0x4,
    REG64_RUN_STAT_TTC_Input_Pulse_Count  = 0x8,
    REG64_RUN_STAT_VXS_Input_Pulse_Count  = 0xC,
    REG64_RUN_STAT_RESERV                 = 0x10,
    REG64_RUN_STAT_TRIG_CANDID            = 0x14,
    REG64_RUN_STAT_ACCEPT                 = 0x18,
    REG64_RUN_STAT_BLOCKED                = 0x1C,

    REG64_RUN_STAT_BUSY_TOTAL             = 0x40,
    REG64_RUN_STAT_BUSY_TDC               = 0x44,
    REG64_RUN_STAT_BUSY_READOUT           = 0x48,

    REG_RUN_STAT_CNT                      = REG64_RUN_STAT_BUSY_READOUT/4+1
};
//Statistics Readout CSR
enum {
    REG_STAT_BASE = 0x300,
    REG_STAT_BASE_NEW = 0x900,
    REG_STAT_CTRL = 0x00,
    REG_STAT_PERIOD = 0x02,
    REG_STAT_BLOCK_SIZE = 0x04,
    REG_STAT_RAM_ADDR = 0x05,
    REG_STAT_RAM_DATA = 0x06
};

//  == Board Level Registers ==


// On-Board Clock CSR
enum {
    REG_ON_BOARD_CLK_BASE   = 0x4300,
    REG_ON_BOARD_CLK_CSR    = REG_ON_BOARD_CLK_BASE + 0x0,
    REG_ON_BOARD_CLK_STATUS = REG_ON_BOARD_CLK_BASE + 0x1,
    REG_ON_BOARD_CLK_SOURCE = REG_ON_BOARD_CLK_BASE + 0x2,
};
enum {
    REG_DEVICE_CTRL_BIT_SOFT_RESET = 0x0001,
    REG_DEVICE_CTRL_BIT_CNT_LOCK = 0x0002,
    REG_DEVICE_CTRL_BIT_RUN = 0x8000
};

#endif // TDCREGS_H
