#include "TdcSignalRouter.h"

#include <QObject>

#include "configuration-manager/ConfigurationManager.h"
#include "mldiscover/LldpDiscovery.h"
#include "pnp-server/PNPServer.h"
#include "fsm/DeviceMgrFsm.h"

#include "MainWindow.h"
#include "TdcConfigDb.h"
#include "TdcConfigDispatcher.h"

#include "remote-control-server/EvNumChecker.h"
#include "remote-control-server/RemoteControlServer.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void TdcSignalRouter::connectAll() const
{
    connect_ConfigurationDispatcher();
    connect_PnpServer();
    connect_RemoteControlServer();
}

void TdcSignalRouter::connect_ConfigurationDispatcher() const
{
    if(!tdcConfigDispatcher)
        return;
    if(mainWindow) {
        tdcConfigDispatcher->subscribe(mainWindow);
        tdcConfigDispatcher->subscribeCS(mainWindow);
    }
    if(tdcConfigDb) {
        tdcConfigDispatcher->subscribe(tdcConfigDb);
        tdcConfigDispatcher->subscribeCS(tdcConfigDb);
    }
    if(configurationManager)
        tdcConfigDispatcher->subscribeCS(configurationManager);
}

void TdcSignalRouter::connect_PnpServer() const
{
    if(!pnpServer || !mainWindow)
        return;

    connect(mainWindow, &MainWindow::changeProgramDescription,
            pnpServer, &PNPServer::changeProgramDescription);
    connect(mainWindow, &MainWindow::closeProgram,
            pnpServer, &PNPServer::closeProgram);
    connect(mainWindow, &MainWindow::closeAll, pnpServer, &QObject::deleteLater);
}

void TdcSignalRouter::connect_RemoteControlServer() const
{
    if(!remoteControlServer)
        return;

    if(evNumChecker) {
        connect(evNumChecker, &EvNumChecker::completedEvNumCheck,
                remoteControlServer, &RemoteControlServer::evNumCheckResponse);
        connect(evNumChecker, &EvNumChecker::checkFail,
                mainWindow, &MainWindow::sendRunWarning);
        connect(remoteControlServer, &RemoteControlServer::evNumCheckRequest,
                evNumChecker, &EvNumChecker::onRemoteEvNumCheckRequest);
    }

    if(mainWindow) {
        connect(remoteControlServer, &RemoteControlServer::startRequest,
                mainWindow, &MainWindow::onRemoteStartRequest);
        connect(remoteControlServer, &RemoteControlServer::stopRequest,
                mainWindow, &MainWindow::onRemoteStopRequest);
        connect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated,
                mainWindow, &MainWindow::remoteStatusUpdated);
        connect(mainWindow, &MainWindow::sendRemoteState,
                remoteControlServer, &RemoteControlServer::sendState);
        connect(mainWindow, &MainWindow::closeAll,
                remoteControlServer, &RemoteControlServer::deleteLater);
        connect(mainWindow, &MainWindow::runError, remoteControlServer,
                QOverload<const RcRunId &, QString>::of(&RemoteControlServer::onRunError));
        connect(mainWindow, &MainWindow::runWarning,
                remoteControlServer, &RemoteControlServer::onRunWarning);
        connect(mainWindow, &MainWindow::allowRemoteControl,
                remoteControlServer, &RemoteControlServer::startListen);
        connect(mainWindow, &MainWindow::config_transfer_to_rc,
                remoteControlServer, &RemoteControlServer::config_transfer);
        connect(mainWindow, &MainWindow::changeProgramDescription,
                remoteControlServer, &RemoteControlServer::changeProgramDescription);
        connect(fsm, &BaseFsm::stateChanged,
                mainWindow, &MainWindow::onFsmStateChange);
        connect(fsm, &BaseFsm::stateChanged,
                remoteControlServer, &RemoteControlServer::onFsmStateChange);
    }
}
