#include "TdcConfig.h"

#include "mregdev/device_id.h"

#include <QSettings>

namespace {
const QString DEV_CONF_PREFIX = "DevConf_";
const QString DEV_CONF_DEFAULT = "DevConfDefault";
}

bool TdcConfig::operator==(const TdcConfig &r) const
{
    return true &&
           run == r.run &&
           remote_start_request == r.remote_start_request &&
           mstreamShowTable == r.mstreamShowTable &&
           mstreamShowLog == r.mstreamShowLog &&
           global_mode == r.global_mode &&
           trace_device_info == r.trace_device_info &&
           show_power_current == r.show_power_current &&
           configFileName == r.configFileName &&
           allow_remote_control == r.allow_remote_control &&
           sort_column == r.sort_column &&
           sort_asc == r.sort_asc &&
           show_cfg_widget == r.show_cfg_widget &&
           runId == r.runId &&
           curDev == r.curDev &&
           known_configs == r.known_configs &&
           default_device_config == r.default_device_config;
}

TdcDeviceConfig &TdcConfig::getConfig(const DeviceIndex &index)
{
    Q_ASSERT(!index.isNull());
    if(!known_configs.contains(index)) {
        TdcDeviceConfig &ds = known_configs[index];
        ds.index = index;
        ds.global_mode_copy(default_device_config);
    }
    return known_configs[index];
}

TdcDeviceConfig TdcConfig::getCurrentConfig()
{
    return getConfig(curDev);
}

bool TdcConfig::isRun() const
{
    for(const auto &s : known_configs.values())
        if(s.enabled && s.run) return true;
    return false;
}

bool TdcConfig::has_enabled_device() const
{
    for(const auto &s : known_configs.values())
        if(s.enabled) return true;
    return false;
}

DeviceIndex TdcConfig::get_device_index_by_key(QString key) const
{
    if(key.startsWith(DEV_CONF_PREFIX)) {
        const auto var = key.split("_");
        if(var.size() >= 3) {
            bool ok1,ok2;
            const auto id = var[1].toUShort(&ok1, 16);
            const auto ser = var[2].toULongLong(&ok2, 16);
            if(ok1 && ok2)
                return DeviceIndex(static_cast<DeviceId>(id), ser);
        }
    }
    return DeviceIndex();
}

