#include "TdcConfigConvertor.h"

#include "base/CoreTypeConverter.h"
#include "base/ConfigConverterUtil.h"

#include "TdcDeviceConfigConvertor.h"
#include "TdcConfig.h"

namespace {
constexpr const char *k_run = "run";
constexpr const char *k_remote_start_request = "remote_start_request";
constexpr const char *k_mstreamShowTable = "mstreamShowTable";
constexpr const char *k_mstreamShowLog = "mstreamShowLog";
constexpr const char *k_global_mode = "global_mode";
constexpr const char *k_trace_device_info = "trace_device_info";
constexpr const char *k_show_power_current = "show_power_current";
constexpr const char *k_configFileName = "configFileName";
constexpr const char *k_allow_remote_control = "allow_remote_control";
constexpr const char *k_sort_column = "sort_column";
constexpr const char *k_sort_asc = "sort_asc";
constexpr const char *k_show_cfg_widget = "show_cfg_widget";
constexpr const char *k_runId = "runId";
constexpr const char *k_curDev = "curDev";
constexpr const char *k_known_configs = "known_configs";
constexpr const char *k_default_device_config = "default_device_config";
}

template<>
bool AbstractConfigConverter<TdcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TdcConfig>::toJsonObject(const TdcConfig &in)
{
    QVariantMap m;

    m[k_run] = QVariant::fromValue(in.run);
    m[k_remote_start_request] = QVariant::fromValue(in.remote_start_request);
    m[k_mstreamShowTable] = QVariant::fromValue(in.mstreamShowTable);
    m[k_mstreamShowLog] = QVariant::fromValue(in.mstreamShowLog);
    m[k_global_mode] = QVariant::fromValue(in.global_mode);
    m[k_trace_device_info] = QVariant::fromValue(in.trace_device_info);
    m[k_show_power_current] = QVariant::fromValue(in.show_power_current);
    m[k_configFileName] = QVariant::fromValue(in.configFileName);
    m[k_allow_remote_control] = QVariant::fromValue(in.allow_remote_control);
    m[k_sort_column] = QVariant::fromValue(in.sort_column);
    m[k_sort_asc] = QVariant::fromValue(in.sort_asc);
    m[k_show_cfg_widget] = QVariant::fromValue(in.show_cfg_widget);
    m[k_runId] = QVariant::fromValue(in.runId);
    m[k_curDev] = QVariant::fromValue(in.curDev);
//    m[k_known_configs] = QVariant::fromValue(in.known_configs);
    m[k_known_configs] = ConfigConverterUtil::toJsonValue(in.known_configs);
    m[k_default_device_config] = QVariant::fromValue(in.default_device_config);

    return fromVariantMap(m);
}

template<>
TdcConfig AbstractConfigConverter<TdcConfig>::fromJsonObject(const QJsonObject &in)
{
    TdcConfig out;
//    out.value = QVariant::fromValue(in.value).value<ValueType>();
//    update_value(in, k_zzzz, &out.zzzz);

    update_value(in, k_run, &out.run);
    update_value(in, k_remote_start_request, &out.remote_start_request);
    update_value(in, k_mstreamShowTable, &out.mstreamShowTable);
    update_value(in, k_mstreamShowLog, &out.mstreamShowLog);
    update_value(in, k_global_mode, &out.global_mode);
    update_value(in, k_trace_device_info, &out.trace_device_info);
    update_value(in, k_show_power_current, &out.show_power_current);
    update_value(in, k_configFileName, &out.configFileName);
    update_value(in, k_allow_remote_control, &out.allow_remote_control);
    update_value(in, k_sort_column, &out.sort_column);
    update_value(in, k_sort_asc, &out.sort_asc);
    update_value(in, k_show_cfg_widget, &out.show_cfg_widget);
    update_value(in, k_runId, &out.runId);
    update_value(in, k_curDev, &out.curDev);
//    update_value(in, k_known_configs, &out.known_configs);
    out.known_configs = ConfigConverterUtil::toMap<DeviceIndex, TdcDeviceConfig>(in[k_known_configs]);
    update_value(in, k_default_device_config, &out.default_device_config);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TdcConfig>::toJsonValue(const TdcConfig &in)
{
    return toJsonObject(in);
}

template<>
TdcConfig AbstractConfigConverter<TdcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TdcConfig>::registerSubTypes()
{
    CoreTypeConverter::registerTypes();
    AbstractConfigConverter<TdcDeviceConfig>::registerTypes();
}
