#ifndef TDCDEVICECONFIG_H
#define TDCDEVICECONFIG_H

#include <QMap>
#include <QMetaType>
#include <QVector>

#include <base/DeviceIndex.h>

class QSettings;

typedef union {
    quint16 all;
    struct {
        unsigned int timer_en: 1;
        unsigned int ttl_en: 1;
        unsigned int ttc_en: 1;
        unsigned int vxs_en: 1;
        unsigned int unused2: 12;
    } bit;
} reg_trig_ctrl;
Q_DECLARE_METATYPE(reg_trig_ctrl)

bool operator==(const reg_trig_ctrl &l, const reg_trig_ctrl &r);

struct StatPreset {
    bool operator==(const StatPreset &r) const {
        return true &&
               type == r.type &&
               label == r.label &&
               regs == r.regs;
    }
    enum STAT_TYPE {STAT_DEFAULT,
                    STAT_DEBUG,
                    STAT_Test,
                    STAT_PRESET_CNT};
    StatPreset() {
        type = STAT_DEFAULT;
        label = "default";
        regs    << 0x004B // Board Temperature, RO
                << 0x004C // F/W Version, RO
                << 0X004D // F/W Revision, RO
                << 0x4001 // MCU
                << 0x4002
                << 0x4003
                << 0x4004
                << 0x4005
                << 0x4006
                << 0x4007
                << 0x4008
                << 0x4009
                << 0x400A
                << 0x400B
                << 0x400C;
    }

    STAT_TYPE type;
    QString label;
    QVector<quint16> regs;
};
Q_DECLARE_METATYPE(StatPreset);
//inline bool operator==(const StatPreset &l, const StatPreset &r)
//{
//    return true &&
//            l.type == r.type &&
//            l.label == r.label &&
//            l.regs == r.regs;
//}

class StatPresetFabric {
public:
    static StatPreset getPreset(int type);
    static QList<StatPreset> getAllPresets();
};

struct AdjustConfig {
    inline bool operator!=(const AdjustConfig &s) const {
        return
        readout_fifo_size != s.readout_fifo_size ||
        mode_rc_compression != s.mode_rc_compression ||
        leading_resolution != s.leading_resolution ||
        enable_pair != s.enable_pair ||
        dll_mode != s.dll_mode ||
        dll_clock != s.dll_clock ||
        core_clock_source != s.core_clock_source;
    }
    bool operator==(const AdjustConfig &r) const { return !operator!=(r); }

    quint8 readout_fifo_size = 7;
    bool mode_rc_compression = true;
    quint16 leading_resolution = 0;
    bool enable_pair = false;
    quint16 dll_mode = 2;
    quint16 dll_clock = 3;
    quint16 core_clock_source = 1;
};
Q_DECLARE_METATYPE(AdjustConfig)
//inline bool operator==(const AdjustConfig &l, const AdjustConfig &r)
//{
//    return true &&
//            l.readout_fifo_size == r.readout_fifo_size &&
//            l.mode_rc_compression == r.mode_rc_compression &&
//            l.leading_resolution == r.leading_resolution &&
//            l.enable_pair == r.enable_pair &&
//            l.dll_mode == r.dll_mode &&
//            l.dll_clock == r.dll_clock &&
//            l.core_clock_source == r.core_clock_source;
//}

struct TdcDeviceConfig {
public:
    TdcDeviceConfig();
    bool operator<(const TdcDeviceConfig &r) const;
    bool operator==(const TdcDeviceConfig &r) const;
    int getNCh() const { return nch; }
    void setNCh(int n);
    bool test(const TdcDeviceConfig &s)  {
        return trigsetup == s.trigsetup;
    }
    void global_mode_copy(const TdcDeviceConfig &s) {
        auto bak = s;
        bak.enabled = enabled;
        bak.index = index;
        bak.order_index = order_index;
        bak.setup_index = setup_index;
        bak.chTdcEn = chTdcEn;
        *this = bak;
    }

    bool enabled = true;
    DeviceIndex index;
    int order_index = -1;
    quint32 setup_index = 0;

    bool run = false;
    QMap<int, bool> chTdcEn;
    quint32 latency = 8; // in 24ns steps
    quint32 matchWin = 10; // in 24ns steps

    bool hptdcHdr = false;
    bool hptdcTrl = false;

    bool leadEdge = true;
    bool trailEdge = true;

    bool stat_readout= true;
    quint16 stat_period = 1000; // in ms
    quint32 trigPeriod = 125000000; // in 24ns step

    bool pll_bypass = true;
    reg_trig_ctrl trigsetup;
    AdjustConfig adjust_set;

    StatPreset stat_preset;

private:
    int nch = 0;
};
Q_DECLARE_METATYPE(TdcDeviceConfig)
//inline bool operator==(const TdcDeviceConfig &l, const TdcDeviceConfig &r)
//{
//    return true &&
//            l.enabled == r.enabled &&
//            l.index == r.index &&
//            l.order_index == r.order_index &&
//            l.setup_index == r.setup_index &&
//            l.run == r.run &&
//            l.chTdcEn == r.chTdcEn &&
//            l.latency == r.latency &&
//            l.matchWin == r.matchWin &&

//            l.hptdcHdr == r.hptdcHdr &&
//            l.hptdcTrl == r.hptdcTrl &&
//            l.leadEdge == r.leadEdge &&
//            l.trailEdge == r.trailEdge &&
//            l.stat_readout == r.stat_readout &&
//            l.stat_period == r.stat_period &&
//            l.trigPeriod == r.trigPeriod &&
//            l.pll_bypass == r.pll_bypass &&
//            l.trigsetup == r.trigsetup &&
//            l.adjust_set == r.adjust_set &&
//            l.stat_preset == r.stat_preset;
//}

#endif // TDCDEVICECONFIG_H

