#include "TdcDeviceConfigConvertor.h"
#include "base/ConfigConverterUtil.h"

#include "TdcDeviceConfig.h"

namespace {
// --------- StatPreset
constexpr const char *k_type = "type";
constexpr const char *k_label = "label";
constexpr const char *k_regs = "regs";

// --------- AdjustConfig
constexpr const char *k_readout_fifo_size = "readout_fifo_size";
constexpr const char *k_mode_rc_compression = "mode_rc_compression";
constexpr const char *k_leading_resolution = "leading_resolution";
constexpr const char *k_enable_pair = "enable_pair";
constexpr const char *k_dll_mode = "dll_mode";
constexpr const char *k_dll_clock = "dll_clock";
constexpr const char *k_core_clock_source = "core_clock_source";

// --------- TdcDeviceConfig
constexpr const char *k_enabled = "enabled";
constexpr const char *k_index = "index";
constexpr const char *k_order_index = "order_index";
constexpr const char *k_setup_index = "setup_index";
constexpr const char *k_run = "run";
constexpr const char *k_chTdcEn = "chTdcEn";
constexpr const char *k_latency = "latency";
constexpr const char *k_matchWin = "matchWin";

constexpr const char *k_hptdcHdr = "hptdcHdr";
constexpr const char *k_hptdcTrl = "hptdcTrl";
constexpr const char *k_leadEdge = "leadEdge";
constexpr const char *k_trailEdge = "trailEdge";
constexpr const char *k_stat_readout = "stat_readout";
constexpr const char *k_stat_period = "stat_period";
constexpr const char *k_trigPeriod = "trigPeriod";
constexpr const char *k_pll_bypass = "pll_bypass";
constexpr const char *k_trigsetup = "trigsetup_all";
constexpr const char *k_adjust_set = "adjust_set";
constexpr const char *k_stat_preset = "stat_preset";
}

// --------- StatPreset

template<>
bool AbstractConfigConverter<StatPreset>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<StatPreset>::toJsonObject(const StatPreset &in)
{
    QVariantMap m;

    m[k_type] = QVariant::fromValue(static_cast<int>(in.type));
    m[k_label] = QVariant::fromValue(in.label);
    m[k_regs] = ConfigConverterUtil::toJsonValue(in.regs.toList());

    return fromVariantMap(m);
}

template<>
StatPreset AbstractConfigConverter<StatPreset>::fromJsonObject(const QJsonObject &in)
{
    StatPreset out;

    out.type = static_cast<StatPreset::STAT_TYPE>(in[k_type].toInt());
    update_value(in, k_label, &out.label );
    out.regs = ConfigConverterUtil::toList<quint16>(in[k_regs]).toVector();

    return out;
}

template<>
QJsonValue AbstractConfigConverter<StatPreset>::toJsonValue(const StatPreset &in)
{
    return toJsonObject(in);
}

template<>
StatPreset AbstractConfigConverter<StatPreset>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<StatPreset>::registerSubTypes()
{
}

// --------- AdjustConfig

template<>
bool AbstractConfigConverter<AdjustConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AdjustConfig>::toJsonObject(const AdjustConfig &in)
{
    QVariantMap m;

    m[k_readout_fifo_size] = QVariant::fromValue(in.readout_fifo_size);
    m[k_mode_rc_compression] = QVariant::fromValue(in.mode_rc_compression);
    m[k_leading_resolution] = QVariant::fromValue(in.leading_resolution);
    m[k_enable_pair] = QVariant::fromValue(in.enable_pair);
    m[k_dll_mode] = QVariant::fromValue(in.dll_mode);
    m[k_dll_clock] = QVariant::fromValue(in.dll_clock);
    m[k_core_clock_source] = QVariant::fromValue(in.core_clock_source);

    return fromVariantMap(m);
}

template<>
AdjustConfig AbstractConfigConverter<AdjustConfig>::fromJsonObject(const QJsonObject &in)
{
    AdjustConfig out;

    update_value(in, k_readout_fifo_size, &out.readout_fifo_size );
    update_value(in, k_mode_rc_compression, &out.mode_rc_compression );
    update_value(in, k_leading_resolution, &out.leading_resolution );
    update_value(in, k_enable_pair, &out.enable_pair );
    update_value(in, k_dll_mode, &out.dll_mode );
    update_value(in, k_dll_clock, &out.dll_clock );
    update_value(in, k_core_clock_source, &out.core_clock_source );

    return out;
}

template<>
QJsonValue AbstractConfigConverter<AdjustConfig>::toJsonValue(const AdjustConfig &in)
{
    return toJsonObject(in);
}

template<>
AdjustConfig AbstractConfigConverter<AdjustConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<AdjustConfig>::registerSubTypes()
{
}

// --------- TdcDeviceConfig

template<>
bool AbstractConfigConverter<TdcDeviceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TdcDeviceConfig>::toJsonObject(const TdcDeviceConfig &in)
{
    QVariantMap m;

    m[k_enabled] = QVariant::fromValue(in.enabled);
    m[k_index] = QVariant::fromValue(in.index);
    m[k_order_index] = QVariant::fromValue(in.order_index);
    m[k_setup_index] = QVariant::fromValue(in.setup_index);
    m[k_run] = QVariant::fromValue(in.run);
//    m[k_chTdcEn] = QVariant::fromValue(in.chTdcEn);
    m[k_chTdcEn] = ConfigConverterUtil::toJsonValue<int, bool>(in.chTdcEn);
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_matchWin] = QVariant::fromValue(in.matchWin);

    m[k_hptdcHdr] = QVariant::fromValue(in.hptdcHdr);
    m[k_hptdcTrl] = QVariant::fromValue(in.hptdcTrl);
    m[k_leadEdge] = QVariant::fromValue(in.leadEdge);
    m[k_trailEdge] = QVariant::fromValue(in.trailEdge);
    m[k_stat_readout] = QVariant::fromValue(in.stat_readout);
    m[k_stat_period] = QVariant::fromValue(in.stat_period);
    m[k_trigPeriod] = QVariant::fromValue(in.trigPeriod);
    m[k_pll_bypass] = QVariant::fromValue(in.pll_bypass);
    m[k_trigsetup] = QVariant::fromValue(in.trigsetup.all);
    m[k_adjust_set] = QVariant::fromValue(in.adjust_set);
    m[k_stat_preset] = QVariant::fromValue(in.stat_preset);

    return fromVariantMap(m);
}

template<>
TdcDeviceConfig AbstractConfigConverter<TdcDeviceConfig>::fromJsonObject(const QJsonObject &in)
{
    TdcDeviceConfig out;

    update_value(in, k_enabled, &out.enabled );
    update_value(in, k_index, &out.index );
    update_value(in, k_order_index, &out.order_index );
    update_value(in, k_setup_index, &out.setup_index );
    update_value(in, k_run, &out.run );
//    update_value(in, k_chTdcEn, &out.chTdcEn );
    out.chTdcEn = ConfigConverterUtil::toMap<int,bool>(in[k_chTdcEn]);
    update_value(in, k_latency, &out.latency );
    update_value(in, k_matchWin, &out.matchWin );

    update_value(in, k_hptdcHdr, &out.hptdcHdr );
    update_value(in, k_hptdcTrl, &out.hptdcTrl );
    update_value(in, k_leadEdge, &out.leadEdge );
    update_value(in, k_trailEdge, &out.trailEdge );
    update_value(in, k_stat_readout, &out.stat_readout );
    update_value(in, k_stat_period, &out.stat_period );
    update_value(in, k_trigPeriod, &out.trigPeriod );
    update_value(in, k_pll_bypass, &out.pll_bypass );
    update_value(in, k_trigsetup, &out.trigsetup.all );
    update_value(in, k_adjust_set, &out.adjust_set );
    update_value(in, k_stat_preset, &out.stat_preset );

    return out;
}

template<>
QJsonValue AbstractConfigConverter<TdcDeviceConfig>::toJsonValue(const TdcDeviceConfig &in)
{
    return toJsonObject(in);
}

template<>
TdcDeviceConfig AbstractConfigConverter<TdcDeviceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TdcDeviceConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<AdjustConfig>::registerTypes();
    AbstractConfigConverter<StatPreset>::registerTypes();
}
