#ifndef DEVICECHMODEL_H
#define DEVICECHMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>

#include "base/DeviceIndex.h"

#include "TqdcConfig.h"
#include "DeviceChModelItem.h"
#include "TqdcStatus.h"
#include "TreeColumns.h"

class QTimer;

class DeviceChModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    DeviceChModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(TreeColumns::COL_TOTAL);
    }

    void setEnabled(bool enabled);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const TqdcConfig &config) const;

public slots:
    void setConfig(int config_key, const TqdcConfig &newConfig);
    void deviceStatusUpdated(const TqdcStatus &status);

private slots:
    void postCheckStateChange();

private:
    void sendDataChanged(QVector<int> roles = QVector<int>());

    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant fontData(const QModelIndex &index) const;

    inline TreeColumns getColumnByIndex(const QModelIndex &index) const;
    inline DeviceChModelItem *getItemByIndex(const QModelIndex &index) const;
    QVariant getAllState(QMap<int, bool> DeviceConfig::* ptr) const;
    QVariant getDevState(const DeviceIndex &index, QMap<int, bool> DeviceConfig::* ptr) const;

    TqdcConfig config;
    QMap<DeviceIndex, TqdcStatus> dev_status;
    DeviceChModelItem rootItem;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
};

#endif // DEVICECHMODEL_H
