#include "SpinBoxDelegate.h"

#include <QDebug>
#include <QTimer>
#include <QSpinBox>

#include "TreeColumns.h"
#include "TqdcThresholdModule.h"

SpinBoxDelegate::SpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
    , commitDataTimer(new QTimer(this))
{
    commitDataTimer->setInterval(1);
    commitDataTimer->setSingleShot(true);
    connect(commitDataTimer, &QTimer::timeout,
            this, &SpinBoxDelegate::flushCommitData);
}

QWidget *SpinBoxDelegate::createEditor(QWidget *parent,
                                       const QStyleOptionViewItem &/* option */,
                                       const QModelIndex &index) const
{
     QSpinBox *spin = new QSpinBox(parent);
     initSpinBox(spin);
     int value_int = index.model()->data(index, Qt::EditRole).toInt();
     spin->setValue(value_int);
     connect(spin, SIGNAL(valueChanged(int)), this, SLOT(dataChanged()));

     return spin;
}

void SpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    int value = index.model()->data(index, Qt::EditRole).toInt();
    QSpinBox *spinBox = static_cast<QSpinBox*>(editor);
    spinBox->blockSignals(true);
    spinBox->setValue(value);
    spinBox->blockSignals(false);
}

void SpinBoxDelegate::setModelData(QWidget *editor,
                                   QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    QSpinBox *spin_box = static_cast<QSpinBox*>(editor);
    spin_box->interpretText();
    int value = spin_box->value();
    if(model->data(index, Qt::EditRole).toInt() != value)
        model->setData(index, value, Qt::EditRole);
}

void SpinBoxDelegate::updateEditorGeometry(QWidget *editor,
                                           const QStyleOptionViewItem &option,
                                           const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}

void SpinBoxDelegate::initSpinBox(QSpinBox *s)
{
    s->setSuffix(" mV");
    s->setMaximum(DacConverter<TqdcThresholdModule>::getOutMax());
    s->setMinimum(DacConverter<TqdcThresholdModule>::getOutMin());
}

void SpinBoxDelegate::dataChanged()
{
    if(commitDataWidget) {
        if(commitDataWidget == sender())
            return;
        flushCommitData();
    }
    commitDataWidget = sender();
    commitDataTimer->start();
}

void SpinBoxDelegate::flushCommitData()
{
    if(!commitDataWidget)
        return;
    commitData(qobject_cast<QWidget*>(commitDataWidget));
    commitDataWidget = nullptr;
}

DeviceChModelItem *SpinBoxDelegate::getItemByIndex(const QModelIndex &index) const
{
    return static_cast<DeviceChModelItem*>(index.internalPointer());
}
