#ifndef TQDCDEVICE_H
#define TQDCDEVICE_H

#include "DeviceConfig.h"
#include "mregdev/qtmregdevice.h"

#include "CalResult.h"
#include "TqdcStatus.h"
class CalConfig;
class TtlIoModule;
struct DeviceConfig;
class AdcSerDesCalibration;
class AdcCoreModule;
class AdcSpiLtm9011Module;
class DacCalibration;
class DeserializersControlModule;
class HitCoreModule;
class TqdcThresholdModule;
class WrStatusModule;

class TQDCDevice : public QtMregDevice
{
    Q_OBJECT
public:
    typedef enum {TRIG_NEGATIVE = 0, TRIG_POSITIVE = 1} TrigPolarity;
    explicit TQDCDevice(const DeviceIndex &index);
    virtual ~TQDCDevice() override;
    void readStatistics();
    void clearStatistics();
    virtual bool checkLiveMagic() override;
    static quint16 dacValue(int mV);
    static quint16 dacValue(int mV, int dacShift);
    static double dac_mV(quint16 dacData);

signals:
    void statusUpdated(const TqdcStatus &);
    void deviceInfoUpdated(DeviceIndex index, int chNum, int maxSampleCnt);
    void calibrateFinished(const DeviceIndex &index, int cal_key);
    void configWritten(const DeviceIndex &config, int config_key);
    void calResultChanged(const CalResult &newCalResult);
    void ttlIoStartupCfg(const TtlIoConfig &cfg);
    void devSerDesDelayChanged(const DeviceIndex &index, const DevSerDesDelay &delay);
    void adcSerDesProgressUpdated(const DeviceIndex &index, int percenage);
    void adcDesCountUpdated(const DeviceIndex &devIndex, int serDesCnt) const;

public slots:
    void setDeviceConfig(int new_config_key, const DeviceConfig &newConfig);
    void setCalResult(int new_config_key, const CalResult &newCalResult);
    void writeSetup();
    void writeSetupForce();
    void softReset();
    void startCalibration(int cal_key);
    void startAdcSrdDesCalibration(int calibrationKey, bool hardMode, QString resDir);

protected slots:
    virtual void statusPoll() override;
    virtual void on_deviceConnected() override;
    virtual void on_deviceDisconnected() override;

private:
    mlink::RegOpVector WriteTrigSetup(reg_trig_ctrl trigsetup);
    mlink::RegOpVector setAdcLatency(int latency);
    void StartRun();
    void StopRun();
    bool isDesStatusOk(const quint16 &adc_status);
    int getDeserializeNum() const { return 4; }
    int getNumberOfChannels() const;
    void detectChannelMaxSampleCount();
    void setupAdc();
    void resetDac();
    void writeTime();
    void writeClockDataDelay();
    void ResetAdc();
//    QString adcStatusStr(int status);
    void readoutHptdcSetup();
    bool waitEndOfRun();
    void resetEvNumAndStats();
    void calibrateBaseline(QMap<int, double> *meanPtr, QMap<int, double> *rmsPtr);
    QMap<int, int> calibrateDac();
    QMap<int, double> readTemp();

protected:
    void sendLog(const QString &);

    DeviceConfig config;
    DeviceConfig configCache;
    QMap<int, int> calDacOffset;
    QMap<int, int> calDacOffsetCache;
    bool setupCacheValid;
    TqdcStatus status;
    int maxSampleCount;
    QMap<int, quint32> hitCnt;
    quint64 hitCntTime;
    bool isBlock;
    int config_key = -1;

    AdcCoreModule * const adcCore;
    AdcSpiLtm9011Module * const adcSpi;
    DeserializersControlModule * const desCtrl;
    HitCoreModule * const hitCore;
    TqdcThresholdModule * const thrMod;
    TtlIoModule * const ttlModule;
    WrStatusModule * const wrStatusMod;

    AdcSerDesCalibration * const adcSerDesCal;
    DacCalibration * const dacCal;
};

#endif // TQDCDEVICE_H
