#ifndef TQDCCORE_H
#define TQDCCORE_H

#include <QObject>

#include "base/DeviceIndex.h"
#include "mldiscover/LldpDiscovery.h"
#include "base/RcRunId.h"
#include "pnp-server/PNPMessage.h"
#include "fsm/FsmState.h"
#include "remote-control-server/RemoteControlServerStatus.h"

#include "CalConfig.h"
#include "TqdcConfig.h"
#include "TqdcDeviceGlobal.h"

class ConfigSelector;
class TqdcConfigDispatcher;
class RemoteControlServer;
class TqdcConfigDb;
class CalConfigDb;
class PNPServer;
class ProgramDescription;
class ConfigIndexName;
class DeviceIndexEnabledMap;
class DeviceDispatcher;
class MainWindow;
class AdcDeviceMgrFsm;

class TqdcCore : public QObject
{
    Q_OBJECT
public:
    TqdcCore(const ConfigSelector &cs, MainWindow *mw, QObject *parent = nullptr);
    virtual ~TqdcCore() override;

signals:
    void configChanged(const TqdcConfig &config) const;
    void switch_to_config(const ConfigIndexName &conf) const;
    void calConfigChanged(const CalConfig &) const;
    void allowRemoteControl(bool allow);
    void deleteDevice(const DeviceIndex &);
    void closeAll();
    void blockDeviceRun(bool isBlock);
    void prepareForStop(quint32 stopEvNum);
    void changeProgramDescription(const ProgramDescription &) const;
    void closeProgram(const ProgramDescription &) const;
    void updateProgramDescription(const ProgramDescription &) const;
    void updateControlState();
    void deviceInfoUpdated(DeviceIndex index, int chNum, int maxSampleCount); // resend to MW
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void setupChangedForce();
    void remoteCompletedStart(bool success, const RcRunId &runId);
    void remoteCompletedStop(bool success, const RcRunId &runId);
    void sendRunError(const RcRunId &runId, QString runError=QString());
    void sendRunWarning(const RcRunId &runId, QString warnStr);
    void startCalibration(int cal_key);
    void startAdcSrdDesCalibration(int cal_key, bool hardMode, const QString &resDir) const;

public slots:
    void setConfig(int config_key, const TqdcConfig &newConfig);
    void configSelectorChanged(const ConfigIndexName &conf);
    void performLocalStart();
    void performLocalStop();
    void remoteStartRequest(const RcRunId &runId_, quint64 startEvNum=0);
    void remoteStopRequest(const RcRunId &runId_, quint64 stopEvNum=0);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &status);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState);
    void deviceDisconnected(const DeviceIndex &index = DeviceIndex());
    void deviceStatusUpdated(const TqdcStatus &status);
    void startCalibration_();
    void requestAdcSerDesCalibration(bool hardMode, QString dir);

private:
    void initTqdcProgDescr();
    void updateTqdcProgDescrOptions();
    void addDevice(const DeviceIndex &index);

    RcRunId runId;
    AdcDeviceMgrFsm * const fsm;
    RemoteControlServer * const remoteControlServer;
    TqdcConfigDb * const tqdcConfigDb;
    CalConfigDb * const calConfigDb;
    PNPServer * const pnpServer;
    DeviceDispatcher * const deviceDispatcher;
    TqdcConfig config;
    ProgramDescription progDescr;
    LldpInfoCache lldpCache;
    QMap<DeviceIndex, TqdcDeviceGlobal*> devices;
    QMap<DeviceIndex, TqdcStatus> dev_status;
    bool lookForStop = false;
    int cal_key;
    bool adcSerDesCalHardMode = false;
    QString adcSerDesCalResDir;
    RemoteControlServerStatus lastRcStatus;
};

#endif // TQDCCORE_H
