#include "TqdcDeviceGlobal.h"
#include "TQDCDevice.h"

#include <QThread>

#include "base/threadQuit.h"
#include "mldiscover/LldpDiscovery.h"
#include "mstream-lib/MStreamDump.h"
#include "TQDCDevice.h"
#include "TqdcSignalRouter.h"

TqdcDeviceGlobal::TqdcDeviceGlobal(const DeviceIndex &_index, QObject *parent)
    :
      QObject(parent)
    , index(_index)
    , dev(new TQDCDevice(index))
    , devThread(new QThread(this))
    , tqdc_status()
    , mstream_thread(new QThread(this))
{
    devThread->setObjectName(index.getIdent());
    dev->moveToThread(devThread);
    devThread->start();


    connect(dev, &TQDCDevice::deviceDisconnected,
            this, &TqdcDeviceGlobal::deviceDisconnected);


    connect(dev, &TQDCDevice::destroyed,
            devThread, &QThread::quit);

    TqdcSignalRouter::instance().connect_TqdcDeviceGlobal(this);
}

TqdcDeviceGlobal::~TqdcDeviceGlobal()
{
    qInfo() << "Disconnect from:" << dev->getDeviceInfoStr();

    QElapsedTimer closeTimer;
    closeTimer.start();
    if(!threadQuiting(devThread, 2000, &closeTimer)) {
        qWarning() << "Can't stop device thread";
    }
}

void TqdcDeviceGlobal::deviceStatusUpdated(const TqdcStatus &status)
{
    Q_ASSERT(index == status.index);
    if(index != status.index)
        return;

    bool doMStreamReset =
            (tqdc_status.mstreamHwBufSize != status.mstreamHwBufSize) ||
            (tqdc_status.mstreamMultiAck != status.mstreamMultiAck);
    tqdc_status = status;

    if(status.connected) {
        if(mstream) {
            if(doMStreamReset)
                resetMStream();
        } else {
            addMStreamDump();
        }
    }
}

void TqdcDeviceGlobal::addMStreamDump()
{
    MStreamDumpParameters dump_parameters;
    dump_parameters.deviceIndex = index;
    dump_parameters.bigFragId = true;
    dump_parameters.hwBufSize = tqdc_status.mstreamHwBufSize;
    dump_parameters.ackSizeLimit = tqdc_status.mstreamMultiAck ? tqdc_status.mstreamHwBufSize : 1;
    dump_parameters.preRunInit();

    mstream = new MStreamDump(dump_parameters, false);
    connect(mstream, &MStreamDump::closing, this, &TqdcDeviceGlobal::closeProgram);
    connect(mstream, &MStreamDump::updateProgramDescription,
            this, &TqdcDeviceGlobal::updateProgramDescription);
    connect(mstream, &MStreamDump::stat_updated,
            this, &TqdcDeviceGlobal::stat_updated);

    LldpDiscovery::subscribe(mstream, {DEVICE_ID_TQDC16VS_E});
    connect(mstream, &MStreamDump::stat_from_receiver_updated,
            this, &TqdcDeviceGlobal::stat_from_receiver_updated);

    connect(this, &TqdcDeviceGlobal::msdump_reset, mstream, &MStreamDump::msdump_reset);

    mstream_thread->setObjectName("MSD_"+index.getSerialStr());
    connect(mstream, &MStreamDump::destroyed, mstream_thread, &QThread::quit);
    connect(this, &TqdcDeviceGlobal::deleteDevice, mstream, &MStreamDump::delete_dump);
    mstream->moveToThread(mstream_thread);
    mstream_thread->start();
}

void TqdcDeviceGlobal::resetMStream(bool doStatClear)
{
    if(!tqdc_status.connected)
        return;

    quint16 ack = tqdc_status.mstreamMultiAck ? tqdc_status.mstreamHwBufSize : 1;
    emit msdump_reset(index, true, tqdc_status.mstreamHwBufSize, ack,
                      QString(), doStatClear);
}
