#ifndef TQDCDEVICEGLOBAL_H
#define TQDCDEVICEGLOBAL_H

#include <QObject>

#include "TqdcStatus.h"
#include "base/DeviceIndex.h"

class QThread;

class MStreamDump;
class TQDCDevice;
class QHostAddress;
class DeviceConfig;
class ProgramDescription;
struct MStreamStat;
struct ReceiverStat;
struct TqdcConfig;
struct CalResult;
class CalConfig;

class TqdcDeviceGlobal : public QObject
{
    Q_OBJECT
public:
    explicit TqdcDeviceGlobal(const DeviceIndex &_index, QObject *parent = nullptr);
    virtual ~TqdcDeviceGlobal() override;
    TQDCDevice *getDevice() const { return dev; }
    DeviceIndex getDeviceIndex() const { return index; }
    QThread *getDeviceThread() const { return devThread; }
    TqdcStatus getTqdcStatus() const { return tqdc_status; }

signals:
    void deleteDevice(const DeviceIndex &);
    void msdump_reset(DeviceIndex index, bool bigFragId,
                      quint16 hwBufSize, quint16 ackSize,
                      QString fn=QString(), bool doStatClear = false);
    void deviceDisconnected(const DeviceIndex &);

    // resend from MStreamDump
    void closeProgram(const ProgramDescription&) const;
    void updateProgramDescription(const ProgramDescription&) const;
    void stat_updated(const MStreamStat &stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);

public slots:
    void resetMStream(bool doStatClear = false);
    void deviceStatusUpdated(const TqdcStatus &status);

private:
    void addMStreamDump();

private:
    DeviceIndex index;
    TQDCDevice * const dev;
    QThread * const devThread;
    TqdcStatus tqdc_status;
    MStreamDump * mstream = nullptr;
    QThread * const mstream_thread;
    bool lookForStop = false;
    quint32 stopEvNum = 0;
};

#endif // TQDCDEVICEGLOBAL_H
