#ifndef TQDCSIGNALROUTER_H
#define TQDCSIGNALROUTER_H

class AdcSerDesCalDialog;
class ConfigurationManager;
class AdcDeviceMgrFsm;
class TqdcConfigDb;
class TqdcConfigDispatcher;
class DeviceDispatcher;
class MainWindow;
class TqdcCore;
class PNPServer;
class RemoteControlServer;
class CalConfigDb;
class TqdcDeviceGlobal;

class TqdcSignalRouter
{
    TqdcSignalRouter() = default;
public:
    // Singleton
    static TqdcSignalRouter & instance()
    {
        // thread-safe since C++11
        static TqdcSignalRouter obj;
        return obj;
    }
    // delete copy and move constructors and assign operators
    TqdcSignalRouter(TqdcSignalRouter const &) = delete;
    TqdcSignalRouter(TqdcSignalRouter &&) = delete;
    TqdcSignalRouter &operator=(TqdcSignalRouter const &) = delete;
    TqdcSignalRouter &operator=(TqdcSignalRouter &&) = delete;

    void setConfigurationManager(ConfigurationManager *p)
    { configurationManager = p; }
    void setCore(TqdcCore *p)
    { core = p; }
    void setMainWindow(MainWindow *p);
    void setPnpServer(PNPServer *p)
    { pnpServer = p; }
    void setDeviceDispatcher(DeviceDispatcher *p)
    { deviceDispatcher = p;}
    void setCalConfigDb(CalConfigDb *p)
    { calConfigDb = p; }
    void setConfigDb(TqdcConfigDb *p)
    { tqdcConfigDb = p; }
    void setRemoteControlServer(RemoteControlServer *p)
    { remoteControlServer = p; }
    void setFsm(AdcDeviceMgrFsm *p)
    { fsm = p; }
    void setAdcSerDesCalDialog(AdcSerDesCalDialog *p)
    { adcSerDesCalDialog = p; }

    void connectAll();
    void connect_TqdcDeviceGlobal(const TqdcDeviceGlobal * devGlob);

private:
    void connect_ConfigurationDispatcher();
    void connect_deviceDispatcher();
    void connect_MainWindow();
    void connect_PnpServer();
    void connect_RemoteControlServer();
    void connect_FSM();

    bool mainConnectPassed = false;
    ConfigurationManager *configurationManager = nullptr;
    CalConfigDb * calConfigDb = nullptr;
    AdcDeviceMgrFsm *fsm = nullptr;
    TqdcConfigDb * tqdcConfigDb = nullptr;
    DeviceDispatcher * deviceDispatcher = nullptr;
    MainWindow *mainWindow = nullptr;
    TqdcCore *core = nullptr;
    PNPServer *pnpServer = nullptr;
    RemoteControlServer *remoteControlServer = nullptr;
    AdcSerDesCalDialog *adcSerDesCalDialog = nullptr;
};
#endif // TQDCSIGNALROUTER_H

