#include "TqdcThresholdModule.h"

#include "regio/regop.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

const int TQDC_CH_TO_DAC_CH_MAP[16] = {
    3,2,1,0, 4,5,6,7, 0,1,2,3, 7,6,5,4
};

TqdcThresholdModule::TqdcThresholdModule(QtMregDevice * const _dev)
    :Ad5328Module(_dev)
{
    useDoubleGain = DacConverter<TqdcThresholdModule>::getGain();
}

mlink::RegOpVector TqdcThresholdModule::setChThresholdRegOp(const QMap<int, int> &chThr_mV,
                                                            const QMap<int, int> &thrOffset,
                                                            const QMap<int, bool> &chInv) const
{
    QMap<int, int> chThr;
    for(int globCh=0 ; globCh<16; ++globCh) {
        int dacId=globCh/getChipCh();
        Q_ASSERT(dacId<getChipCnt());

        int dacVOut=TQDC_CH_TO_DAC_CH_MAP[globCh];
        Q_ASSERT(dacVOut<getChipCh());

        int mV = chThr_mV[globCh];
        if(chInv[globCh])
            mV *= -1;
        const int &chipGlobalCh = dacId*getChipCh() + dacVOut;
        int offset = thrOffset.value(globCh, DacConverter<TqdcThresholdModule>::getDacDefaultZero());
        chThr[chipGlobalCh] = DacConverter<TqdcThresholdModule>::dacValue(mV, offset);
    }
    return setChDacDataRegOp(chThr);
}

template<>
int DacConverter<TqdcThresholdModule>::getDacDefaultZero()
{
    return 1920; // -600,698125597
}

template<>
double DacConverter<TqdcThresholdModule>::getVRef()
{
    const double DAC_V_REF_TQDC16_THR = 3.3 * 82.5 / (82.5 + 130); // in V (1.2812)
    return DAC_V_REF_TQDC16_THR * 1e3;
}
