#include "DeviceConfig.h"

#include <QDataStream>
#include <QDebug>

DeviceConfig::DeviceConfig()
{
    setNCh(16);
}

void DeviceConfig::setThreshold(int thr)
{
    for(int ch=0; ch<nch; ++ch){
        if(chTdcEn[ch] || chAdcEn[ch])
            chThr[ch] = thr;
    }
}

void DeviceConfig::setNCh(int n)
{
    bool defTdcEn = chTdcEn.value(0, true);
    bool defAdcEn = chAdcEn.value(0, true);
    int defThr = chThr.value(0, 0);
    bool defInv = chInv.value(0, false);
    bool defGain = chGain.value(0, false);
    for(int ch=nch; ch<n; ++ch){
        chTdcEn[ch]  = defTdcEn;
        chAdcEn[ch]  = defAdcEn;
        chTrigEn[ch] = true;
        chThr[ch] = defThr;
        chInv[ch] = defInv;
        chGain[ch] = defGain;
    }
    nch = n;
}

bool DeviceConfig::isGain4() const
{
    return !chGain.isEmpty() && chGain.first();
}

int DeviceConfig::getSimilarThreshold() const
{
    return chThr.value(0, 0);
}

void DeviceConfig::setGain4(bool gain4)
{
    for(auto &v : chGain)
        v = gain4;
}

QDebug operator<<(QDebug dbg, const DeviceConfig &s)
{
    dbg << "DeviceSetup{"
        << "enabled:" << s.enabled
        << "run:" << s.run
        << "adcPreWin:" << s.adcPreWin
        << "readCellNumber:" << s.readCellNumber
        << "latency:" << s.latency
        << "matchWin:" << s.matchWin
        << "hptdcStandby:" << s.hptdcStandby
        << "invertInput:" << s.invertInput
        << "invert_thr_trig:" << s.invert_thr_trig
        << "invert_zs_thr:" << s.invert_zs_thr
        << "zsEn:" << s.zsEn
        << "trigPeriod:" << s.trigPeriod
        << "selfTrigDelay:" << s.selfTrigDelay
        << "maxSampleCnt:" << s.maxSampleCnt
        << "trigsetup_all:" << s.trigsetup.all
        << "index:" << s.index
        << "chTdcEn:" << s.chTdcEn
        << "chAdcEn:" << s.chAdcEn
        << "chInv:" << s.chInv
        << "chTrigEn:" << s.chTrigEn
        << "chThr:" << s.chThr
        << "chGain:" << s.chGain
        << "ttlIo:" << s.ttlIo
        << "serDesDelay:" << s.serDesDelay
        << "}";
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const DeviceConfig &r)
{
    out
        << r.enabled
        << r.run
        << r.adcPreWin
        << r.readCellNumber
        << r.latency
        << r.matchWin
        << r.hptdcStandby
        << r.invertInput
        << r.invert_thr_trig
        << r.invert_zs_thr
        << r.zsEn
        << r.trigPeriod
        << r.selfTrigDelay
        << r.maxSampleCnt
        << r.trigsetup.all
        << r.index
        << r.chTdcEn
        << r.chAdcEn
        << r.chInv
        << r.chTrigEn
        << r.chThr
        << r.chGain
        << r.ttlIo
        << r.serDesDelay;
    return out;
}

QDataStream &operator>>(QDataStream &in, DeviceConfig &r)
{
    in
        >> r.enabled
        >> r.run
        >> r.adcPreWin
        >> r.readCellNumber
        >> r.latency
        >> r.matchWin
        >> r.hptdcStandby
        >> r.invertInput
        >> r.invert_thr_trig
        >> r.invert_zs_thr
        >> r.zsEn
        >> r.trigPeriod
        >> r.selfTrigDelay
        >> r.maxSampleCnt
        >> r.trigsetup.all
        >> r.index
        >> r.chTdcEn
        >> r.chAdcEn
        >> r.chInv
        >> r.chTrigEn
        >> r.chThr
        >> r.chGain
            >> r.ttlIo
            >> r.serDesDelay;
    return in;
}
