#ifndef DEVICECONFIG_H
#define DEVICECONFIG_H

#include <QMetaType>
#include <QMap>

#include "base/DeviceIndex.h"
#include "device-modules/DeserializersControl/AdcSerDesDelay.h"
#include "device-modules/TtlIo/TtlIoConfig.h"

class QDataStream;

typedef union reg_trig_ctrl {
    quint16 all = 0;
    struct {
        unsigned int timer_en: 1;
        unsigned int ttl_en: 1;
        unsigned int ttc_en: 1;
        unsigned int selftrig_en: 1;
        unsigned int unused2: 12;
    } bit;
} reg_trig_ctrl;

struct DeviceConfig
{
    DeviceConfig();
    void setThreshold(int thr);
    int getNCh() const { return nch; }
    void setNCh(int n);
    bool isGain4() const;
    int getSimilarThreshold() const;
    void setGain4(bool gain4);
    void global_mode_copy(const DeviceConfig &ds) {
        auto candidate = ds;

        candidate.index = index;
        candidate.enabled = enabled;
        candidate.run = run;

        candidate.chTdcEn = chTdcEn;
        candidate.chAdcEn = chAdcEn;
        candidate.chInv = chInv;
        candidate.chThr = chThr;
        candidate.chTrigEn = chTrigEn;
        candidate.chGain = chGain;
        candidate.serDesDelay = serDesDelay;

        *this = candidate;
    }

    DeviceIndex index;
    bool enabled = true;
    bool run = false;
    int adcPreWin = 2;
    int readCellNumber = 8;
    int latency = 8; // in 24ns steps
    int matchWin = 8; // in 24ns steps
    bool hptdcStandby = false;  // not used
    bool invertInput = false; // not used
    bool invert_thr_trig = false; // not used
    bool invert_zs_thr = false; // not used

    bool zsEn = true;
    quint32 trigPeriod = 125e6; // in 8ns step
    quint16 selfTrigDelay = 0; // in 8ns step
    int maxSampleCnt = 0; // doesn't store //todo: remove from setup

    reg_trig_ctrl trigsetup;
    QMap<int, bool> chTdcEn;
    QMap<int, bool> chAdcEn;
    QMap<int, bool> chInv;
    QMap<int, int> chThr; // in mV
    QMap<int, bool> chTrigEn; // self-trig
    QMap<int, bool> chGain;
    TtlIoConfig ttlIo;
    DevSerDesDelay serDesDelay;

private:
    int nch = 0;
};
Q_DECLARE_METATYPE(DeviceConfig)

inline bool operator==(const DeviceConfig &l, const DeviceConfig &r)
{
    return l.index == r.index &&
            l.enabled == r.enabled &&
            l.run == r.run &&
            l.adcPreWin == r.adcPreWin &&
            l.readCellNumber == r.readCellNumber &&
            l.latency == r.latency &&
            l.matchWin == r.matchWin &&
            l.hptdcStandby == r.hptdcStandby &&
            l.invertInput == r.invertInput &&
            l.invert_thr_trig == r.invert_thr_trig &&
            l.invert_zs_thr == r.invert_zs_thr &&
            l.zsEn == r.zsEn &&
            l.trigPeriod == r.trigPeriod &&
            l.selfTrigDelay == r.selfTrigDelay &&
            l.maxSampleCnt == r.maxSampleCnt &&
            l.trigsetup.all == r.trigsetup.all &&
            l.chTdcEn == r.chTdcEn &&
            l.chAdcEn == r.chAdcEn &&
            l.chInv == r.chInv &&
            l.chThr == r.chThr &&
            l.chTrigEn == r.chTrigEn &&
            l.chGain == r.chGain &&
            l.ttlIo == r.ttlIo &&
            l.serDesDelay == r.serDesDelay;
}
QDebug operator<<(QDebug dbg, const DeviceConfig &s);
QDataStream &operator<<(QDataStream &out, const DeviceConfig &r);
QDataStream &operator>>(QDataStream &in, DeviceConfig &r);

#endif // DEVICECONFIG_H
