#include "TqdcConfig.h"
#include "base/DaqModule.h"
#include "base/DeviceIndexEnabled.h"

#include <QDataStream>
#include <QDebug>

DeviceIndexEnabledMap TqdcConfig::getDeviceIndexEnabledMap() const
{
    DeviceIndexEnabledMap map;
    for(auto cfg : knownConfigs.values()){
        auto &die = map[cfg.index];
        die.index = cfg.index;
        die.enabled = cfg.enabled;
    }
    return map;
}

bool TqdcConfig::isRun() const
{
    int on=0, off=0;
    for(auto &ds : knownConfigs) {
        if(ds.enabled)
            ++(ds.run ? on : off);
    }
    if(on && off)
        qWarning() << "Config has mix (running/stopped) devices";
    return on;
}

void TqdcConfig::setRunState(bool run)
{
    for(auto &ds : knownConfigs)
        ds.run = run && ds.enabled;
}

int TqdcConfig::getSimilarThreshold() const
{
    if(knownConfigs.isEmpty())
        return defaultSetup.getSimilarThreshold();

    for(auto &ds : knownConfigs)
        if(ds.enabled)
            return ds.getSimilarThreshold();

    return knownConfigs.first().getSimilarThreshold();
}

void TqdcConfig::setThreshold(int thr)
{
    defaultSetup.setThreshold(thr);
    for(auto &ds : knownConfigs)
        ds.setThreshold(thr);
}

bool TqdcConfig::hasEnabledDevices() const
{
    for(auto &ds : knownConfigs)
        if(ds.enabled)
            return true;
    return false;
}

QDebug operator<<(QDebug dbg, const TqdcConfig &s)
{
    dbg << "TqdcConfig{"
        << "separateChEn:" << s.separateChEn
        << "showAllWrInfo:" << s.showAllWrInfo
           << "matchWinLimitEn:" << s.matchWinLimitEn
        << "knownSetups:" << s.knownConfigs
        << "defaultSetup:" << s.defaultSetup
        << "}";
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const TqdcConfig &r)
{
    out
            << r.separateChEn
            << r.showAllWrInfo
            << r.matchWinLimitEn
            << r.knownConfigs
            << r.defaultSetup;
    return out;
}

QDataStream &operator>>(QDataStream &in, TqdcConfig &r)
{
    in
            >> r.separateChEn
            >> r.showAllWrInfo
            >> r.matchWinLimitEn
            >> r.knownConfigs
            >> r.defaultSetup;
    return in;
}
