#include "TqdcConfigConverter.h"

#include "base/ConfigConverterUtil.h"
#include "base/DeviceIndexConverter.h"
#include "device-modules/DeserializersControl/AdcSerDesDelayConvertor.h"
#include "device-modules/TtlIo/TtlIoConfigConverter.h"
//#include "histogramm/HistMngrCfgConverter.h"
//#include "zmq-lib/ZmqConfigConverter.h"

namespace {
constexpr const char *k_enabled = "enabled";
constexpr const char *k_run = "run";
constexpr const char *k_adcPreWin = "adcPreWin";
constexpr const char *k_readCellNumber = "readCellNumber";
constexpr const char *k_latency = "latency";
constexpr const char *k_matchWin = "matchWin";
constexpr const char *k_hptdcStandby = "hptdcStandby";
constexpr const char *k_invertInput = "invertInput";
constexpr const char *k_invert_thr_trig = "invert_thr_trig";
constexpr const char *k_invert_zs_thr = "invert_zs_thr";
constexpr const char *k_zsEn = "zsEn";
constexpr const char *k_trigPeriod = "trigPeriod";
constexpr const char *k_selfTrigDelay = "selfTrigDelay";
constexpr const char *k_maxSampleCnt = "maxSampleCnt";
constexpr const char *k_trigsetup = "trigsetup_all";
constexpr const char *k_index = "index";

constexpr const char *k_chTdcEn = "chTdcEn";
constexpr const char *k_chAdcEn = "chAdcEn";
constexpr const char *k_chTrigEn = "chTrigEn";
constexpr const char *k_chThr = "chThr";
constexpr const char *k_chInv = "chInv";
constexpr const char *k_chGain = "chGain";
constexpr const char *k_ttlIo = "ttlIo";
constexpr const char *k_serDesDelay = "serDesDelay";


constexpr const char *k_separateChEn = "separateChEn";
constexpr const char *k_showAllWrInfo = "showAllWrInfo";
constexpr const char *k_matchWinLimitEn = "matchWinLimitEn";
constexpr const char *k_knownSetups = "knownSetups";
constexpr const char *k_defaultSetup = "defaultSetup";
constexpr const char *k_allowRemoteControl = "AllowRemoteControl";
}

template<>
bool AbstractConfigConverter<DeviceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DeviceConfig>::toJsonObject(const DeviceConfig &in)
{
    QVariantMap m;
    m[k_enabled] = QVariant::fromValue(in.enabled);
    m[k_run] = QVariant::fromValue(in.run);
    m[k_adcPreWin] = QVariant::fromValue(in.adcPreWin);
    m[k_readCellNumber] = QVariant::fromValue(in.readCellNumber);
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_matchWin] = QVariant::fromValue(in.matchWin);
    m[k_hptdcStandby] = QVariant::fromValue(in.hptdcStandby);
    m[k_invertInput] = QVariant::fromValue(in.invertInput);
    m[k_invert_thr_trig] = QVariant::fromValue(in.invert_thr_trig);
    m[k_invert_zs_thr] = QVariant::fromValue(in.invert_zs_thr);
    m[k_zsEn] = QVariant::fromValue(in.zsEn);
    m[k_trigPeriod] = QVariant::fromValue(in.trigPeriod);
    m[k_selfTrigDelay] = QVariant::fromValue(in.selfTrigDelay);
    m[k_maxSampleCnt] = QVariant::fromValue(in.maxSampleCnt);
    m[k_trigsetup] = QVariant::fromValue(in.trigsetup.all);
    m[k_index] = QVariant::fromValue(in.index);

    m[k_chTdcEn] = ConfigConverterUtil::toJsonValue(in.chTdcEn);
    m[k_chAdcEn] = ConfigConverterUtil::toJsonValue(in.chAdcEn);
    m[k_chTrigEn] = ConfigConverterUtil::toJsonValue(in.chTrigEn);
    m[k_chThr] = ConfigConverterUtil::toJsonValue(in.chThr);
    m[k_chInv] = ConfigConverterUtil::toJsonValue(in.chInv);
    m[k_chGain] = ConfigConverterUtil::toJsonValue(in.chGain);
    m[k_ttlIo] = QVariant::fromValue(in.ttlIo);
    m[k_serDesDelay] = QVariant::fromValue(in.serDesDelay);

    return fromVariantMap(m);
}

template<>
DeviceConfig AbstractConfigConverter<DeviceConfig>::fromJsonObject(const QJsonObject &in)
{
    DeviceConfig out;
    update_value(in, k_enabled, &out.enabled);
    update_value(in, k_run, &out.run);
    update_value(in, k_adcPreWin, &out.adcPreWin);
    update_value(in, k_readCellNumber, &out.readCellNumber);
    update_value(in, k_latency, &out.latency);
    update_value(in, k_matchWin, &out.matchWin);
    update_value(in, k_hptdcStandby, &out.hptdcStandby);
    update_value(in, k_invertInput, &out.invertInput);
    update_value(in, k_invert_thr_trig, &out.invert_thr_trig);
    update_value(in, k_invert_zs_thr, &out.invert_zs_thr);
    update_value(in, k_zsEn, &out.zsEn);
    update_value(in, k_trigPeriod, &out.trigPeriod);
    update_value(in, k_selfTrigDelay, &out.selfTrigDelay);
    update_value(in, k_maxSampleCnt, &out.maxSampleCnt);
    update_value(in, k_trigsetup, &out.trigsetup.all);
    update_value(in, k_index, &out.index);

    out.chTdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTdcEn]);
    out.chAdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chAdcEn]);
    out.chTrigEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTrigEn]);
    out.chThr = ConfigConverterUtil::toMap<int, int>(in[k_chThr]);
    out.chInv = ConfigConverterUtil::toMap<int, bool>(in[k_chInv]);
    out.chGain = ConfigConverterUtil::toMap<int, bool>(in[k_chGain]);
    out.chTdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTdcEn]);
    update_value(in, k_ttlIo, &out.ttlIo);
    update_value(in, k_serDesDelay, &out.serDesDelay);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<DeviceConfig>::toJsonValue(const DeviceConfig &in)
{
    return toJsonObject(in);
}

template<>
DeviceConfig AbstractConfigConverter<DeviceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DeviceConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<TtlIoConfig>::registerTypes();
    AbstractConfigConverter<DevSerDesDelay>::registerTypes();
}

template<>
bool AbstractConfigConverter<TqdcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TqdcConfig>::toJsonObject(const TqdcConfig &in)
{
    QVariantMap m;
    m[k_separateChEn] = QVariant::fromValue(in.separateChEn);
    m[k_showAllWrInfo] = QVariant::fromValue(in.showAllWrInfo);
    m[k_matchWinLimitEn] = QVariant::fromValue(in.matchWinLimitEn);
    m[k_knownSetups] = ConfigConverterUtil::toJsonValue(in.knownConfigs);
    m[k_defaultSetup] = QVariant::fromValue(in.defaultSetup);
    m[k_allowRemoteControl] = QVariant::fromValue(in.allowRemoteControl);
    return fromVariantMap(m);
}

template<>
TqdcConfig AbstractConfigConverter<TqdcConfig>::fromJsonObject(const QJsonObject &in)
{
    TqdcConfig out;
    update_value(in, k_separateChEn, &out.separateChEn);
    update_value(in, k_showAllWrInfo, &out.showAllWrInfo);
    update_value(in, k_matchWinLimitEn, &out.matchWinLimitEn);
    out.knownConfigs = ConfigConverterUtil::toMap<DeviceIndex, DeviceConfig>(
        in[k_knownSetups]);
    update_value(in, k_defaultSetup, &out.defaultSetup);
    update_value(in, k_allowRemoteControl, &out.allowRemoteControl);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TqdcConfig>::toJsonValue(const TqdcConfig &in)
{
    return toJsonObject(in);
}

template<>
TqdcConfig AbstractConfigConverter<TqdcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TqdcConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<DeviceConfig>::registerTypes();
//    AbstractConfigConverter<ZmqConfig>::registerTypes();
//    AbstractConfigConverter<HistMngrCfg>::registerTypes();
//    AbstractConfigConverter<CalConfig>::registerTypes();
}
