#include "TqdcConfigDispatcher.h"
#include "base/DeviceIndexEnabled.h"

#include "base/util.h"

TqdcConfigDispatcher::TqdcConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new TqdcConfig()),
    cacheCalConfig(new CalConfig())
{}

void TqdcConfigDispatcher::sendCache() const {
    BaseConfigDispatcher::sendCache();
    sendConfigKeyCache();
    emit deviceListChanged(cache->getDeviceIndexEnabledMap());

    sendAdcSerDesCache();
}

void TqdcConfigDispatcher::setDevSerDesDelay(const DeviceIndex &index, const DevSerDesDelay &delay)
{
    auto it = cache->knownConfigs.find(index);
    if(it == cache->knownConfigs.end())
        return;
    if(it.value().serDesDelay == delay)
        return;
    it.value().serDesDelay = delay;
    incrementConfigKey();
    emit devSerDesDelayChanged(index, delay);
}

void TqdcConfigDispatcher::setConfig(const TqdcConfig &newConfig)
{
    if (newConfig == *cache)
        return;

    *cache = newConfig;
    incrementConfigKey();
    emit deviceListChanged(cache->getDeviceIndexEnabledMap());
    sendAdcSerDesCache();
}

void TqdcConfigDispatcher::setCalConfig(const CalConfig &newCalConfig)
{
    if (newCalConfig == *cacheCalConfig)
        return;

    *cacheCalConfig = newCalConfig;
    incrementConfigKey();
}

void TqdcConfigDispatcher::setCalResult(const CalResult &newCalResult)
{
    if (newCalResult == cacheCalConfig->value(newCalResult.index))
        return;
    *cacheCalConfig->insert(newCalResult.index, newCalResult);
    incrementConfigKey();
}

void TqdcConfigDispatcher::setDeviceList(const DeviceIndexEnabledMap &map)
{
    if(cache->getDeviceIndexEnabledMap() == map)
        return;

    for(auto index_it=map.cbegin(); index_it!=map.cend(); ++index_it){
        const DeviceIndex &index = index_it.key();
        const DeviceIndexEnabled &index_enabled = index_it.value();
        const auto it = cache->knownConfigs.find(index);
        if(it == cache->knownConfigs.cend()){
            cache->knownConfigs.insert(index, cache->defaultSetup);
        }
        cache->knownConfigs[index].enabled = index_enabled.enabled;
        cache->knownConfigs[index].index = index;
    }
    QSet<DeviceIndex> toDel;
    for(auto it=cache->knownConfigs.cbegin(); it!=cache->knownConfigs.cend(); ++it){
        if(!map.contains(it.key()))
            toDel << it.key();
    }
    if(!toDel.isEmpty())
        for(auto index : toDel){
            cache->knownConfigs.remove(index);
        }

    incrementConfigKey();
    emit deviceListChanged(cache->getDeviceIndexEnabledMap());
}

void TqdcConfigDispatcher::setTtlCfg(const TtlIoConfig &ttlCfg)
{
    if (ttlCfg == cache->defaultSetup.ttlIo)
        return;

    cache->defaultSetup.ttlIo = ttlCfg;
    for(auto &ds : cache->knownConfigs)
        ds.global_mode_copy(cache->defaultSetup);
    incrementConfigKey();
    emit deviceListChanged(cache->getDeviceIndexEnabledMap());
}
