#ifndef TQDCCONFIGDISPATCHER_H
#define TQDCCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"
#include "device-modules/DeserializersControl/AdcSerDesDelay.h"

#include "TqdcConfig.h"
#include "CalConfig.h"
#include "CalResult.h"

class DeviceIndexEnabledMap;

class TqdcConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
    explicit TqdcConfigDispatcher(QObject *parent = nullptr);
public:
    // Singleton
    static TqdcConfigDispatcher & instance()
    {
        // thread-safe since C++11
        static TqdcConfigDispatcher obj;
        return obj;
    }
    // delete copy and move constructors and assign operators
    TqdcConfigDispatcher(TqdcConfigDispatcher const &) = delete;
    TqdcConfigDispatcher(TqdcConfigDispatcher &&) = delete;
    TqdcConfigDispatcher &operator=(TqdcConfigDispatcher const &) = delete;
    TqdcConfigDispatcher &operator=(TqdcConfigDispatcher &&) = delete;

    virtual void sendCache() const override;

    template<class T>
    void subscribeConfig(T *obj) {
            connect(this, &TqdcConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &TqdcConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            qDebug() << "Config:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeDeviceConfig(T *obj) {
            connect(this, &TqdcConfigDispatcher::deviceConfigChanged,
                    obj, &T::setDeviceConfig,
                    Qt::UniqueConnection);
            qDebug() << "Config:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeCalConfig(T *obj) {
            connect(this, &TqdcConfigDispatcher::calConfigChanged,
                    obj, &T::setCalConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::calConfigChanged,
                    this, &TqdcConfigDispatcher::setCalConfig,
                    Qt::UniqueConnection);
            qDebug() << "CalConfig:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeCalResult(T *obj) {
            connect(this, &TqdcConfigDispatcher::calResultChanged,
                    obj, &T::setCalResult,
                    Qt::UniqueConnection);

            connect(obj, &T::calResultChanged,
                    this, &TqdcConfigDispatcher::setCalResult,
                    Qt::UniqueConnection);
            qDebug() << "CalResult:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeTtlCfg(T *obj) {
            connect(this, &TqdcConfigDispatcher::ttlCfgChanged,
                    obj, &T::setTtlCfg,
                    Qt::UniqueConnection);

            connect(obj, &T::ttlCfgChanged,
                    this, &TqdcConfigDispatcher::setTtlCfg,
                    Qt::UniqueConnection);
            qDebug() << "CalResult:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

// ===== DevSerDesDelay start =====
    template<class T>
    void subscribeSerDesDelay(T *obj) {
        connect(obj, &T::devSerDesDelayChanged,
                this, &TqdcConfigDispatcher::setDevSerDesDelay);
        connect(this, &TqdcConfigDispatcher::devSerDesDelayChanged,
                obj, &T::setDevSerDesDelay);
        qDebug() << "DevSerDesDelay:"
                 << metaObject()->className()
                 << "<->"
                 << obj->metaObject()->className();
    }

signals: void devSerDesDelayChanged(const DeviceIndex &index,
                                    const DevSerDesDelay &delay) const;
public slots: void setDevSerDesDelay(const DeviceIndex &index,
                                     const DevSerDesDelay &delay);
// ===== DevSerDesDelay end =====

signals:
    void configKeyChanged(int key) const;
    void configChanged(int key, const TqdcConfig &) const;
    void deviceConfigChanged(int key, const DeviceConfig &) const;
    void calConfigChanged(int key, const CalConfig &) const;
    void calResultChanged(int key, const CalResult &) const;
    void ttlCfgChanged(const TtlIoConfig &) const;
    void deviceListChanged(const DeviceIndexEnabledMap &) const;

public slots:
    void setConfig(const TqdcConfig &newConfig);
    void setCalConfig(const CalConfig &newCalConfig);
    void setCalResult(const CalResult &newCalResult);
    void setDeviceList(const DeviceIndexEnabledMap &map);
    void setTtlCfg(const TtlIoConfig &);

private:
    void sendConfigKeyCache() const {
        emit configKeyChanged(config_key);

        emit configChanged(config_key, *cache);
        for(auto cfg : cache->knownConfigs.values())
            emit deviceConfigChanged(config_key, cfg);

        emit calConfigChanged(config_key, *cacheCalConfig);
        for(auto cal : cacheCalConfig->values())
            emit calResultChanged(config_key, cal);

        emit ttlCfgChanged(cache->defaultSetup.ttlIo);
    }
    void sendAdcSerDesCache() const {
        for(const auto &cfg : cache->knownConfigs)
            emit devSerDesDelayChanged(cfg.index, cfg.serDesDelay);
    }
    void incrementConfigKey() {
        ++config_key;
        sendConfigKeyCache();
    }

    std::unique_ptr<TqdcConfig> cache;
    std::unique_ptr<CalConfig> cacheCalConfig;
    int config_key = 0;
};
#endif // TQDCCONFIGDISPATCHER_H
