﻿#ifndef DEVICEGLOB_H
#define DEVICEGLOB_H

#include <QObject>

#include "TtvxsConfig.h"
#include "TtvxsStatus.h"

class QHostAddress;
class QThread;

struct FeLinkStatus;
class MStreamDump;
struct MStreamStat;
class ProgramDescription;
class RcRunId;
class ReceiverStat;
class TtvxsAppConfig;
class TtvxsDevice;

class DeviceGlob : public QObject
{
    Q_OBJECT
public:
    DeviceGlob(const DeviceIndex &index_, QObject *parent = nullptr);
    ~DeviceGlob();

public slots:
    void setConfig(int config_key, const TtvxsAppConfig &app_config);
    void deleteHardware(DeviceIndex index);

private slots:
    void statusUpdated(const TtvxsStatus &status);

signals:
    void configChanged(const TtvxsAppConfig &config);
    void configChanged_(int config_key, const TtvxsConfig &config);
    void configWritten(const DeviceIndex &index, int configKey) const;
    void setHostAddress(const DeviceIndex &, const QHostAddress &);
    void statusUpdated_(const TtvxsStatus &st, const TtvxsStatus &prev_st);
    void feLinkStatusUpdated(const DeviceIndex &index, const QMap<int,FeLinkStatus> &stMap) const;

    //=== signals from MainWindow
    void deleteAll();
    void deleteHardware_(DeviceIndex=DeviceIndex());
    void resetTimestamp(DeviceIndex=DeviceIndex());
    void clearStat(DeviceIndex=DeviceIndex());
    void evNumReset(DeviceIndex=DeviceIndex());
    void softClear(DeviceIndex=DeviceIndex());
    void restartAll(const RcRunId &runId) const;

    void getTrigEvNum(int checkIndex, bool local=true);
    void trigEvNumResponse(int check_index, quint64 evNum);
    void devEvNumResponse(int check_index, quint64 devSerial, quint64 evNum);


    //=== MStreamDump
    void msdump_reset(DeviceIndex index, bool bigFragId,
                      quint16 hwBufSize, quint16 ackSize,
                      QString fn=QString(), bool doStatClear = false);
    void closeProgram(const ProgramDescription&);
    void updateProgramDescription(const ProgramDescription&);
    void stat_updated(const MStreamStat &stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);

private:
    void addMStreamDump();
    void writeConfig(const TtvxsConfig &config, int setup_key = 0);

    DeviceIndex index;
    TtvxsDevice * const dev;
    QThread * const dev_thread;
    MStreamDump * mstream = nullptr;
    QThread * const mstream_thread;
    TtvxsStatus prev_status;
    TtvxsConfig ttvxs_config;
};

#endif // DEVICEGLOB_H
