#ifndef TTVXSDEVICE_H
#define TTVXSDEVICE_H

#include <QObject>
#include <QMetaType>

#include "TtvxsConfig.h"
#include "TtvxsStatus.h"

#include "mregdev/qtmregdevice.h"
#include "base/DeviceIndex.h"

class RcRunId;

namespace mlink {
class RegOpVector;
}

class TtvxsDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit TtvxsDevice(DeviceIndex devIndex, QObject *parent=nullptr);

signals:
    void statusUpdated(const TtvxsStatus &status);
    void trigEvNumResponse(int check_index, quint64 evNum);
    void devEvNumResponse(int check_index, quint64 devSerial, quint64 evNum);
    void configWritten(const DeviceIndex &index, int setupSeqNum) const;

public slots:
    void writeConfig(int configKey_, const TtvxsConfig &cfg);
    void evNumReset(DeviceIndex index=DeviceIndex());
    void getTrigEvNum(int check_index, bool top_level);
    void onTempChanged(const DeviceIndex &index, double temp);
    void onRestartAll(const RcRunId &runId);
    void softClear(DeviceIndex i);
    void clearStat(DeviceIndex i);
    void resetTimestamp(DeviceIndex i=DeviceIndex());

protected slots:
    virtual void statusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

protected:
    virtual int getFeLinkCnt() const;

private:
    mlink::RegOpVector writeStatCfg(const TtvxsConfig &cfg);

    TtvxsConfig configCashe;
    TtvxsStatus status;
    bool configValid;
    int configKey = 0;
};

#endif // TTVXSDEVICE_H
