/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTVXSAPPCONFIG_H
#define TTVXSAPPCONFIG_H

#include "base/ClientConfig.h"
#include "base/DeviceIndex.h"
#include "base/RcRunId.h"
#include "TtvxsConfig.h"
#include "base/DeviceIndexEnabled.h"

#include <QMetaType>

class TtvxsAppConfig
{
public:
    void unify_config();
    DeviceIndexEnabledMap getDeviceIndexEnabledMap();

    friend bool operator==(const TtvxsAppConfig &l, const TtvxsAppConfig &r)
    {
        return l.devicesCfg == r.devicesCfg &&
               l.default_cfg == r.default_cfg &&
               l.runEnable == r.runEnable &&
               l.runId == r.runId &&
               l.localRunNumber == r.localRunNumber &&
               l.allowRemoteControl == r.allowRemoteControl &&
               l.clients == r.clients;
    }

    QMap<DeviceIndex, TtvxsConfig> devicesCfg;
    TtvxsConfig default_cfg;

    bool runEnable = false;
    RcRunId runId;
    int localRunNumber;
    bool allowRemoteControl = true;
    QMap<int, ClientConfig> clients;
};

Q_DECLARE_METATYPE(TtvxsAppConfig)

#endif // TTVXSAPPCONFIG_H
