/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TtvxsAppConfigDispatcher.h"

#include "TtvxsAppConfig.h"

TtvxsAppConfigDispatcher::TtvxsAppConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new TtvxsAppConfig())
{
    qRegisterMetaType<TtvxsAppConfig>();
}

TtvxsAppConfigDispatcher::~TtvxsAppConfigDispatcher()
{
    delete cache;
}

void TtvxsAppConfigDispatcher::setConfig(const TtvxsAppConfig &config)
{
    if (config == *cache)
        return;
    *cache = config;
    ++config_key;
    qDebug() << __PRETTY_FUNCTION__ << "config changed, new config key: " << config_key ;
    emit configChanged(config_key, config);
    emit deviceListChanged(cache->getDeviceIndexEnabledMap());
}

void TtvxsAppConfigDispatcher::setDeviceList(const DeviceIndexEnabledMap &map)
{
    if(cache->getDeviceIndexEnabledMap() == map)
        return;

    for(auto index_it=map.cbegin(); index_it!=map.cend(); ++index_it){
        const DeviceIndex &index = index_it.key();
        const DeviceIndexEnabled &index_enabled = index_it.value();
        const auto it = cache->devicesCfg.find(index);
        if(it == cache->devicesCfg.cend()){
            cache->devicesCfg.insert(index, cache->default_cfg);
        }
        cache->devicesCfg[index].enabled = index_enabled.enabled;
        cache->devicesCfg[index].device_index = index;
    }
    QSet<DeviceIndex> toDel;
    auto &devices = cache->devicesCfg;
    for(auto it=devices.cbegin(); it!=devices.cend(); ++it){
        if(!map.contains(it.key()))
            toDel << it.key();
    }
    if(!toDel.isEmpty())
        for(auto index : toDel){
            cache->devicesCfg.remove(index);
        }
    ++config_key;
    emit configChanged(config_key, *cache);
}
