/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTVXSAPPCONFIGDISPATCHER_H
#define TTVXSAPPCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"

class TtvxsAppConfig;
class DeviceIndexEnabledMap;

class TtvxsAppConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    TtvxsAppConfigDispatcher(QObject *parent = nullptr);
    virtual ~TtvxsAppConfigDispatcher();

    virtual void sendCache() const override{
        BaseConfigDispatcher::sendCache();
        emit configChanged(config_key, *cache);
    }

    template<class T>
    void subscribe(T *obj) {
        connect(obj, &T::configChanged,
                this, &TtvxsAppConfigDispatcher::setConfig);
        connect(this, &TtvxsAppConfigDispatcher::configChanged,
                obj, &T::setConfig);
        qDebug() << "TtvxsAppConfig:"
                 << metaObject()->className()
                 << "<->"
                 << obj->metaObject()->className();
    }

signals:
    void configChanged(int key, const TtvxsAppConfig &config) const;
    void deviceListChanged(const DeviceIndexEnabledMap &);

public slots:
    void setConfig(const TtvxsAppConfig &config);
    void setDeviceList(const DeviceIndexEnabledMap &map);

private:
    TtvxsAppConfig * const cache;
    int config_key = 0;
};

#endif // TTVXSAPPCONFIGDISPATCHER_H
