#ifndef TTVXSCONFIG_H
#define TTVXSCONFIG_H

#include <QObject>
#include <QDialog>

#include "StatPreset.h"

#include "base/DeviceIndex.h"

typedef union reg_trig_src {
    quint16 all=0;
    struct {
        unsigned int ttl: 1;
        unsigned int sfp: 1;
        unsigned int unused2: 12;
        unsigned int random_pulser: 1;
        unsigned int fixed_pulser: 1;
    } bit;
} reg_trig_src;
Q_DECLARE_METATYPE(reg_trig_src)

typedef union reg_rc_ctrl {
    quint16 all=0;
    struct {
        unsigned int clear: 1;
        unsigned int reserved: 11;
        unsigned int en_aux_cnt: 1;
        unsigned int reset_aux_cnt: 1;
        unsigned int sendTimeReset: 1;
        unsigned int run: 1;
    } bit;
} reg_rc_ctrl;

typedef union reg_base_ctrl {
    quint16 all=0;
    struct {
        unsigned int softClear: 1;
        unsigned int cntLock: 1;
        unsigned int reserved: 13;
        unsigned int run_csr: 1;
    } bit;
} reg_base_ctrl;

class TtvxsConfig
{
public:
    TtvxsConfig(DeviceIndex i = DeviceIndex());
    // TODO: bool operator==(const TtvxsConfig r) const;
    void unify_config(const TtvxsConfig &cfg) {
        // except device_index, enabled, vxsChMask, slot, custom_name
        run = cfg.run;
        trig_open = cfg.trig_open;
        trigSrc = cfg.trigSrc;
        trigTimerFreq = cfg.trigTimerFreq;
        trigRandomFreq = cfg.trigRandomFreq;
        trigRandomDeadTimeUs = cfg.trigRandomDeadTimeUs;
        aux_cnt_for_trig = cfg.aux_cnt_for_trig;
        stat_readout = cfg.stat_readout;
        stat_period = cfg.stat_period;
        stat_preset = cfg.stat_preset;
    }
    friend bool operator==(const TtvxsConfig &l, const TtvxsConfig &r)
    {
        return
        l.device_index == r.device_index &&
        l.enabled == r.enabled &&
        l.run == r.run &&
        l.trig_open == r.trig_open &&
        l.trigSrc.all == r.trigSrc.all &&
        l.vxsChMask == r.vxsChMask &&
        l.trigTimerFreq == r.trigTimerFreq &&
        l.trigRandomFreq == r.trigRandomFreq &&
        l.trigRandomDeadTimeUs == r.trigRandomDeadTimeUs &&
        l.aux_cnt_for_trig == r.aux_cnt_for_trig &&
        l.stat_readout == r.stat_readout &&
        l.stat_period == r.stat_period &&
        l.stat_preset == r.stat_preset &&
        l.slot == r.slot &&
        l.custom_name == r.custom_name;
    }

    DeviceIndex device_index;
    bool enabled = true;
    bool run = false;
    bool trig_open = false;
    reg_trig_src trigSrc;
    quint32 vxsChMask = 0;
    int trigTimerFreq = 1000;  // Hz
    int trigRandomFreq = 1000; // Hz
    double trigRandomDeadTimeUs = 10.; // us

    bool aux_cnt_for_trig = false;

    bool stat_readout = false;
    quint16 stat_period = 1000;
    StatPreset stat_preset;

    int slot = 0;
    QString custom_name;
};
Q_DECLARE_METATYPE(TtvxsConfig)

#endif // TTVXSCONFIG_H
