#include "TtvxsConfigDb.h"

#include "TtvxsAppConfigConverter.h"
#include "base/DeviceIndexEnabled.h"

#include <QJsonDocument>

TtvxsConfigDb::TtvxsConfigDb(const ConfigSelector &cs, QObject *parent) :
    AbstractConfigDb(cs, parent)
{
    AbstractConfigConverter<TtvxsAppConfig>::registerTypes();
}

void TtvxsConfigDb::setConfig(int config_key, const TtvxsAppConfig &newConfig)
{
    Q_UNUSED(config_key);
    if(!initialized)
        return;
    if (newConfig == config)
        return;
    config = newConfig;
    emit configChanged(config);
    const auto &json = QVariant::fromValue(config).value<QJsonObject>();
    write_config(json);
}

void TtvxsConfigDb::update_config_from_db()
{
    const auto &json = read(cs);
    TtvxsAppConfig newConfig = QVariant::fromValue(json).value<TtvxsAppConfig>();
    newConfig.runEnable = false;
    newConfig.default_cfg.run = false;
    newConfig.unify_config();
    if(newConfig.devicesCfg.contains(DeviceIndex())) {
        newConfig.devicesCfg.remove(DeviceIndex());
    }
    for(DeviceIndex index : newConfig.devicesCfg.keys()){
        if(index != newConfig.devicesCfg[index].device_index)
            newConfig.devicesCfg.remove(index);
    }
    emit configChanged(newConfig);
}
