#include "TrigDeviceMgrFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.TrigDeviceMgrFsm")
}

TrigDeviceMgrFsm::TrigDeviceMgrFsm(QObject *p):
    DeviceMgrFsm(p)
{
    setObjectName("TrigDeviceMgrFsm");
    recreateInput(new TrigDeviceMgrFsmInput());
}

bool TrigDeviceMgrFsm::evaluate()
{
    if(getInput()->devSet.isEmpty()) {
        setState(FsmState::Error);
        return true;
    }

    if(DeviceMgrFsm::evaluate())
        return true;

    switch (getState()) {
    case FsmState::Starting:
        if(allDevicesWritten() && getRequestState() == FsmState::CloseGlobTrig) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::CloseGlobTrig:
        if (allDevicesWritten() &&
                getRequestState() == FsmState::OpenGlobTrig) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::OpenGlobTrig:
        if(allDevicesWritten() &&
                getRequestState() == FsmState::Run) {
            acceptRequestedState();
            return true;
        }
        break;
    case FsmState::Stopping:
        if(allDevicesWritten() && getInput()->hasStopEvNumber &&
                getRequestState() == FsmState::Ready) {
            acceptRequestedState();
            return true;
        }
        break;
    default:
    {}
    }

    return false;
}

void TrigDeviceMgrFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

void TrigDeviceMgrFsm::markStopEvNumber()
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName();
    getInput()->hasStopEvNumber = true;
    evaluate();
}

qint64 TrigDeviceMgrFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
//    case FsmState::CloseGlobTrig: return 20e3;
    case FsmState::OpenGlobTrig: return DEVICE_CONFIG_WRITE_TIMEOUT;
    default:
        return DeviceMgrFsm::stateTimeoutMs(state);
    }
}
