#ifndef TDCVIEWERCONFIG_H
#define TDCVIEWERCONFIG_H

#include <QMetaType>

#include "base/DaqModule.h"
#include "base/ProgramIndex.h"
#include "file-reader-dialog/FileReaderConfig.h"
#include "histogramm/HistMngrCfg.h"
#include "histogramm/DeviceCalibration.h"
#include "zmq-lib/ZmqTypes.h"

#include "WaveformViewerConfig.h"

using DevChInvert = QVector<bool>;

struct AfiViewerConfig
{
    enum class SOURCE_DATA {
        SOURCE_IDLE=0,
        SOURCE_STREAM=1,
        SOURCE_FILE=2
    };

    DaqModule sourse_program_type;
    ProgramIndex sourse_program_index;
    bool rawDataEveryEv = false;
    int rawDataLimit = 1;
    bool rawDataPeriodic = false;
    int rawDataPeriod = 5;
    QString progInterfaceType; // FIXME: replace QString with ...
    QString separateInlFolder;

    SOURCE_DATA source = SOURCE_DATA::SOURCE_IDLE;
    QMap<DeviceIndex, DevChInvert > invertMap;
    FileReaderConfig file_reader_config;
    TdcTtvxsMapType tdc_ttvxs_map;

    HistMngrCfg hist_manager_config;
    WaveformViewerConfig wf_config;
    ZmqConfig zmq_config;
};
Q_DECLARE_METATYPE(AfiViewerConfig)

inline bool operator==(const AfiViewerConfig &l, const AfiViewerConfig &r)
{
    return l.sourse_program_type == r.sourse_program_type &&
            l.sourse_program_index == r.sourse_program_index &&
            l.rawDataEveryEv == r.rawDataEveryEv &&
            l.rawDataLimit == r.rawDataLimit &&
            l.rawDataPeriodic == r.rawDataPeriodic &&
            l.rawDataPeriod == r.rawDataPeriod &&
            l.progInterfaceType == r.progInterfaceType &&
            l.separateInlFolder == r.separateInlFolder &&
            l.source == r.source &&
            l.invertMap == r.invertMap &&
            l.file_reader_config == r.file_reader_config &&
            l.tdc_ttvxs_map == r.tdc_ttvxs_map &&
            l.hist_manager_config == r.hist_manager_config &&
            l.wf_config == r.wf_config &&
            l.zmq_config == r.zmq_config;
}

#endif // TDCVIEWERCONFIG_H


