#include "AfiViewerConfigConverter.h"

#include "base/ConfigConverterUtil.h"
#include "base/DaqModuleConverter.h"
#include "base/DeviceIndexConverter.h"
#include "file-reader-dialog/FileReaderConfigConvertor.h"
#include "histogramm/HistMngrCfgConverter.h"
#include "zmq-lib/ZmqConfigConverter.h"

#include "AfiViewerConfig.h"

namespace {
constexpr const char *k_en = "en";
constexpr const char *k_selectedDevice = "selectedDevice";
constexpr const char *k_minCh = "minCh";
constexpr const char *k_maxCh = "maxCh";

constexpr const char *k_sourse_program_type = "sourse_program_type";
constexpr const char *k_sourse_program_index = "sourse_program_index";
constexpr const char *k_rawDataEveryEv = "rawDataEveryEv";
constexpr const char *k_rawDataLimit = "rawDataLimit";
constexpr const char *k_rawDataPeriodic = "rawDataPeriodic";
constexpr const char *k_rawDataPeriod = "rawDataPeriod";
constexpr const char *k_progInterfaceType = "progInterfaceType";
constexpr const char *k_separateInlFolder = "separateInlFolder";
constexpr const char *k_source = "source";
constexpr const char *k_invertMap = "invertMap";
constexpr const char *k_file_reader_config = "file_reader_config";
constexpr const char *k_tdc_ttvxs_map = "tdc_ttvxs_map";
constexpr const char *k_hist_manager_config = "hist_manager_config";
constexpr const char *k_zmq_config = "zmq_config";
}

// ----- WaveformViewerConfig
template<>
bool AbstractConfigConverter<WaveformViewerConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<WaveformViewerConfig>::toJsonObject(const WaveformViewerConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_selectedDevice] = QVariant::fromValue(in.selectedDevice);
    m[k_minCh] = QVariant::fromValue(in.minCh);
    m[k_maxCh] = QVariant::fromValue(in.maxCh);
    return fromVariantMap(m);
}

template<>
WaveformViewerConfig AbstractConfigConverter<WaveformViewerConfig>::fromJsonObject(const QJsonObject &in)
{
    WaveformViewerConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_selectedDevice, &out.selectedDevice);
    update_value(in, k_minCh, &out.minCh);
    update_value(in, k_maxCh, &out.maxCh);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<WaveformViewerConfig>::toJsonValue(const WaveformViewerConfig &in)
{
    return toJsonObject(in);
}

template<>
WaveformViewerConfig AbstractConfigConverter<WaveformViewerConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<WaveformViewerConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
}


// ----- DevChInvert
template<>
bool AbstractConfigConverter<DevChInvert>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DevChInvert>::toJsonObject(const DevChInvert &in)
{
    return ConfigConverterUtil::toJsonValue(in.toList()).toObject();
}

template<>
DevChInvert AbstractConfigConverter<DevChInvert>::fromJsonObject(const QJsonObject &in)
{
    return ConfigConverterUtil::toList<bool>(in).toVector();
}

template<>
QJsonValue AbstractConfigConverter<DevChInvert>::toJsonValue(const DevChInvert &in)
{
    return toJsonObject(in);
}

template<>
DevChInvert AbstractConfigConverter<DevChInvert>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DevChInvert>::registerSubTypes()
{}

// ----- TdcViewerConfig
template<>
bool AbstractConfigConverter<AfiViewerConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AfiViewerConfig>::toJsonObject(const AfiViewerConfig &in)
{
    QVariantMap m;
    m[k_sourse_program_type] = QVariant::fromValue(in.sourse_program_type);
    m[k_sourse_program_index] = QVariant::fromValue(in.sourse_program_index);
    m[k_rawDataEveryEv] = QVariant::fromValue(in.rawDataEveryEv);
    m[k_rawDataLimit] = QVariant::fromValue(in.rawDataLimit);
    m[k_rawDataPeriodic] = QVariant::fromValue(in.rawDataPeriodic);
    m[k_rawDataPeriod] = QVariant::fromValue(in.rawDataPeriod);
    m[k_progInterfaceType] = QVariant::fromValue(in.progInterfaceType);
    m[k_separateInlFolder] = QVariant::fromValue(in.separateInlFolder);
    m[k_source] = QVariant::fromValue(static_cast<int>(in.source));
//    m[k_invertMap] = QVariant::fromValue(in.invertMap);
    m[k_invertMap] = ConfigConverterUtil::toJsonValue<DeviceIndex, DevChInvert>(in.invertMap);
    m[k_file_reader_config] = QVariant::fromValue(in.file_reader_config);
//    m[k_tdc_ttvxs_map] = QVariant::fromValue(in.tdc_ttvxs_map);
    m[k_tdc_ttvxs_map] = ConfigConverterUtil::toJsonValue<DeviceIndex,DeviceIndex>(in.tdc_ttvxs_map);
    m[k_hist_manager_config] = QVariant::fromValue(in.hist_manager_config);
    m[k_zmq_config] = QVariant::fromValue(in.zmq_config);
    return fromVariantMap(m);
}

template<>
AfiViewerConfig AbstractConfigConverter<AfiViewerConfig>::fromJsonObject(const QJsonObject &in)
{
    AfiViewerConfig out;
    update_value(in, k_sourse_program_type, &out.sourse_program_type);
    update_value(in, k_sourse_program_index, &out.sourse_program_index);
    update_value(in, k_rawDataEveryEv, &out.rawDataEveryEv);
    update_value(in, k_rawDataLimit, &out.rawDataLimit);
    update_value(in, k_rawDataPeriodic, &out.rawDataPeriodic);
    update_value(in, k_rawDataPeriod, &out.rawDataPeriod);
    update_value(in, k_progInterfaceType, &out.progInterfaceType);
    update_value(in, k_separateInlFolder, &out.separateInlFolder);
//    update_value(in, k_source, &out.source);
    out.source = static_cast<AfiViewerConfig::SOURCE_DATA>(in[k_source].toInt());
//    update_value(in, k_invertMap, &out.invertMap);
    out.invertMap = ConfigConverterUtil::toMap<DeviceIndex, DevChInvert>(in[k_invertMap]);
    update_value(in, k_file_reader_config, &out.file_reader_config);
//    update_value(in, k_tdc_ttvxs_map, &out.tdc_ttvxs_map);
    out.tdc_ttvxs_map = ConfigConverterUtil::toMap<DeviceIndex, DeviceIndex>(in[k_tdc_ttvxs_map]);
    update_value(in, k_hist_manager_config, &out.hist_manager_config);
    update_value(in, k_zmq_config, &out.zmq_config);
    return out;
}

template<>
void AbstractConfigConverter<AfiViewerConfig>::registerSubTypes()
{
    AbstractConfigConverter<DaqModule>::registerTypes();
    AbstractConfigConverter<ProgramIndex>::registerTypes();
    AbstractConfigConverter<FileReaderConfig>::registerTypes();
    AbstractConfigConverter<HistMngrCfg>::registerTypes();
    AbstractConfigConverter<DevChInvert>::registerTypes();
    AbstractConfigConverter<ZmqConfig>::registerTypes();
}

template<>
QJsonValue AbstractConfigConverter<AfiViewerConfig>::toJsonValue(const AfiViewerConfig &in)
{
    return toJsonObject(in);
}

template<>
AfiViewerConfig AbstractConfigConverter<AfiViewerConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}
