#include "AfiViewerConfigDispatcher.h"

#include "base/util.h"

AfiViewerConfigDispatcher::AfiViewerConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new AfiViewerConfig())
{
}

void AfiViewerConfigDispatcher::setConfig(const AfiViewerConfig &newConfig)
{
    if (newConfig == *cache)
        return;
    bool histChanged = !(newConfig.hist_manager_config == cache->hist_manager_config);
    bool zmqChanged = !(newConfig.zmq_config == cache->zmq_config);
    *cache = newConfig;
    qDebug() << "configChanged by" << qObjectAncestryStr(sender());
    emit configChanged(*cache);
    Q_ASSERT(!newConfig.hist_manager_config.hist_groups.contains(HistId::unknown));
    if(histChanged)
        emit histConfigChanged(cache->hist_manager_config);
    if(zmqChanged)
        emit zmqConfigChanged(cache->zmq_config);
}

void AfiViewerConfigDispatcher::setHistConfig(const HistMngrCfg &newConfig)
{
    if (newConfig == cache->hist_manager_config)
        return;
    cache->hist_manager_config = newConfig;
    qDebug() << "histConfigChanged by" << qObjectAncestryStr(sender());
    emit histConfigChanged(cache->hist_manager_config);
    emit configChanged(*cache);
}

void AfiViewerConfigDispatcher::setZmqConfig(const ZmqConfig &newConfig)
{
    if (newConfig == cache->zmq_config)
        return;
    cache->zmq_config = newConfig;
    qDebug() << "zmqConfigChanged by" << qObjectAncestryStr(sender());
    emit zmqConfigChanged(cache->zmq_config);
    emit configChanged(*cache);
}
