#ifndef TDCVIEWERCONFIGDISPATCHER_H
#define TDCVIEWERCONFIGDISPATCHER_H

#include "base/BaseConfigDispatcher.h"

#include "AfiViewerConfig.h"

class AfiViewerConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit AfiViewerConfigDispatcher(QObject *parent = nullptr);

    virtual void sendCache() const override{
        BaseConfigDispatcher::sendCache();
        emit configChanged(*cache);
        emit histConfigChanged(cache->hist_manager_config);
    }

    template<class T>
    void subscribe(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &AfiViewerConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            qDebug() << "TdcViewerConfig:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeHistCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::histConfigChanged,
                    obj, &T::setHistConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::histConfigChanged,
                    this, &AfiViewerConfigDispatcher::setHistConfig,
                    Qt::UniqueConnection);
            qDebug() << "HistMngrCfg:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

    template<class T>
    void subscribeZmqCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::zmqConfigChanged,
                    obj, &T::setZmqConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::zmqConfigChanged,
                    this, &AfiViewerConfigDispatcher::setZmqConfig,
                    Qt::UniqueConnection);
            qDebug() << "ZmqConfig:"
                     << metaObject()->className()
                     << "<->"
                     << obj->metaObject()->className();
    }

signals:
    void configChanged(const AfiViewerConfig &) const;
    void histConfigChanged(const HistMngrCfg &) const;
    void zmqConfigChanged(const ZmqConfig &) const;

public slots:
    void setConfig(const AfiViewerConfig &newConfig);
    void setHistConfig(const HistMngrCfg &newConfig);
    void setZmqConfig(const ZmqConfig &newConfig);

private:
    std::unique_ptr<AfiViewerConfig> cache;
};

#endif // TDCVIEWERCONFIGDISPATCHER_H
