#include "AfiViewerSignalRouter.h"

#include "MainWindow.h"
#include "AfiViewerConfigDb.h"
#include "AfiViewerConfigDispatcher.h"
#include "pnp-server/PNPServer.h"
#include "mldiscover/LldpDiscovery.h"
#include "configuration-manager/ConfigurationManager.h"
#include "histogramm/HistWorker.h"
#include "histogramm/HistMngrWidget.h"
#include "zmq-lib/ZmqDialog.h"
#include "zmq-lib/ZmqPublisher.h"

namespace {
template <typename Func1, typename Func2>
inline void connect(const typename QtPrivate::FunctionPointer<Func1>::Object *sender, Func1 signal,
                    const typename QtPrivate::FunctionPointer<Func2>::Object *receiver, Func2 slot)
{
    QObject::connect(sender, signal, receiver, slot);
}
}

void AfiViewerSignalRouter::connectAll()
{
    Q_ASSERT(configDispatcher);
    Q_ASSERT(pnpServer);

    connect_ConfigurationDispatcher();
    connect_MainWindow();
    connect_PnpServer();
    connect_HistWorker();
    connect_HistWidget();
    connect_ZMQ();
}

void AfiViewerSignalRouter::connect_MainWindow()
{

}

void AfiViewerSignalRouter::connect_PnpServer()
{
    if(!pnpServer || !mainWindow)
        return;
    connect(pnpServer, &PNPServer::programDiscovered, mainWindow, &MainWindow::gotProgramDescription);
    connect(pnpServer, &PNPServer::programClosed, mainWindow, &MainWindow::gotClosedProgram);
    connect(mainWindow, &MainWindow::closeProgram, pnpServer, &PNPServer::closeProgram);
    connect(mainWindow, &MainWindow::changeProgramDescription, pnpServer, &PNPServer::changeProgramDescription);
    connect(mainWindow, &MainWindow::closeAll, pnpServer, &PNPServer::deleteLater);
}

void AfiViewerSignalRouter::connect_ConfigurationDispatcher()
{
    if(!configDispatcher)
        return;
    if(mainWindow) {
        configDispatcher->subscribe(mainWindow);
        configDispatcher->subscribeCS(mainWindow);
    }
    if(configurationManager)
        configDispatcher->subscribeCS(configurationManager);
    if(configDb) {
        configDispatcher->subscribe(configDb);
        configDispatcher->subscribeCS(configDb);
    }
    if(histWorker)
        configDispatcher->subscribeHistCfg(histWorker);
    if(histMngrWidget)
        configDispatcher->subscribeHistCfg(histMngrWidget);
}

void AfiViewerSignalRouter::connect_HistWorker()
{
    if(!histWorker)
        return;

    if(mainWindow) {
        connect(mainWindow, &MainWindow::inlConfigUpdated,
                histWorker, &HistWorker::inlConfigUpdated);
        connect(mainWindow, &MainWindow::tdcTtvxMapUpdated,
                histWorker, &HistWorker::tdcTtvxMapUpdated);
        connect(mainWindow, &MainWindow::requestInlResults,
                histWorker, &HistWorker::requestInlResults);
        connect(histWorker, &HistWorker::inlResultsUpdated,
                mainWindow, &MainWindow::inl_corrections_updated);
    }

    if(histMngrWidget) {
        connect(histMngrWidget, &HistMngrWidget::clearHist,
                histWorker, &HistWorker::clearHist);
        connect(histWorker, &HistWorker::devListUpdated,
                histMngrWidget, &HistMngrWidget::devListUpdated);
        connect(histWorker, &HistWorker::hist_updated,
                histMngrWidget, &HistMngrWidget::hist_updated);
        connect(histWorker, &HistWorker::taiReperUpdated,
                histMngrWidget, &HistMngrWidget::taiReperUpdated);
    }
}

void AfiViewerSignalRouter::connect_HistWidget()
{
    if(!histWorker || !histMngrWidget)
        return;
}

void AfiViewerSignalRouter::connect_ZMQ()
{
    if(configDispatcher) {
        if(zmqDialog)
            configDispatcher->subscribeZmqCfg(zmqDialog);
        if(zmqPublisher)
            configDispatcher->subscribeZmqCfg(zmqPublisher);
    }
    if(zmqDialog && zmqPublisher)
        connect(zmqPublisher, &ZmqPublisher::statusUpdated,
                zmqDialog, &ZmqDialog::statusUpdated);
}
