#ifndef WAVEFORMVIEWER_H
#define WAVEFORMVIEWER_H

#include <QWidget>

#include "histogramm/DeviceData.h"

#include "WaveformViewerConfig.h"

namespace Ui {
class WaveformViewer;
}

class WaveformViewer : public QWidget
{
    Q_OBJECT

public:
    explicit WaveformViewer(QWidget *parent = nullptr);
    ~WaveformViewer() override;

signals:
    void configUpdated(const WaveformViewerConfig &cfg) const;

public slots:
    void deviceDataUpdated(const DecodedData &data,
                           bool writeDecodedEnabled);
    void refreshPlot(bool force = false);
    void reset(const WaveformViewerConfig &config = WaveformViewerConfig());

private slots:
    void on_checkBoxEn_toggled(bool checked);
    void on_comboBoxDev_currentIndexChanged(int index);
    void on_spinBoxChMin_valueChanged(int arg1);
    void on_spinBoxChMax_valueChanged(int arg1);

private:
    void initGui();
    void addDevice(const DeviceIndex &index);
    void showWaveform(const CompleteEvent &ev);

    Ui::WaveformViewer *ui;
    DecodedData lastData;
    DecodedData pendingData;
    bool hasPendingData;
    QTimer * const refreshTimer;
    WaveformViewerConfig cfg;
    DeviceIndexSet addedDevices;
    bool hasSeveralDev = false;
    bool hasSeveralTypes = false;
};

#endif // WAVEFORMVIEWER_H
