#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QDateTime>
#include <QMainWindow>

#include "Config.h"

class QTimer;
class QLabel;
class PexmlFlashDev;
class DataBase;

namespace Ui {
class MainWindow;
}


class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    void init(QString filePath, bool golden, int interfaceIndex);

private Q_SLOTS:
    void on_actionWrite_Golden_Image_triggered();
    void on_actionGet_record_logs_triggered();
    void on_actionVerify_after_write_toggled(bool doCheck);
    void on_actionVerify_Golden_image_triggered();
    void on_actionVerify_regular_image_triggered();
    void on_actionAbout_triggered();

    void on_pushButtonFile_clicked();
    void on_pushButtonWrite_clicked();

    void deviceTempChanged(double temp);
    void deviceStatusUpdated(QString s);
    void progressFormatChanged(QString format);
    void progressUpdated(int p);
    void taskFinished(uint id, bool isWrite, bool ok, bool isGoldenImage);

signals:
    void startDevWrite(uint id, QByteArray file, int headerSize, bool isGold, bool verify=true);
    void startDevVerify(uint id, QByteArray file, int headerSize, bool isGold);
    void closeDevices();

private:
    void disconnectFromAll();
    void readConfig();
    void saveConfig();
    void updateWriteButtonState() const;

    bool readFile();
    void startWrite(bool isGoldenImage=false);
    void startVerify(bool isGoldenImage=false);
    void startTask(bool isWrite, bool isGoldenImage=false);
    void initTable();
    void updateTableSize();
    void addDevice(int interfaceIndex);
    void setPushButtonEnabled(bool enabled) const;

    Ui::MainWindow *ui;
    Config config;
    int taskId;
    int devTaskId; // -1: err; 0: unknown; 1-1000: ok
    bool connectedState;
    quint32 pexmlSerial;
    PexmlFlashDev * device;
    QThread * devThread;
    quint64 curDevSerial;
    QLabel * const tempLabel;
    QLabel * const fwLabel;
    QLabel * const lastStatusLabel;
    DataBase * dataBase;

    QByteArray fileData;
    int fileHeaderSize;
    int fileSize;
    QDateTime fileModified;
    bool batchMode;
};

#endif // MAINWINDOW_H
