#ifndef PEXMLFLASHDEV_H
#define PEXMLFLASHDEV_H

#include <QObject>

#include "flash-prog/FlashDev.h"
#include "mlink/mlink.h"
#include "mlink/MlinkDeviceInterfaceDebug.h"

namespace mlink {
class MlinkDeviceInterfaceDebug;
}

class PexmlFlashDev : public QObject, public FlashDev
{
    Q_OBJECT
public:
    PexmlFlashDev(QObject * parent, int interfaceIndex = -1);
    ~PexmlFlashDev();
    std::string getDevName() const { return devName; }
    quint32 getDevSerial() const { return devSerial; }
    bool isOpened() const { return isOpen; }
    QString getDrvFwStr() const;
    QString getDevFwStr() const;

private slots:
    void write(uint id, QByteArray fileData, int headerSize, bool isGoldenImage=false, bool verify=true);
    void verify(uint id, QByteArray fileData, int headerSize, bool isGoldenImage=false);

signals:
    void progressFormatChanged(QString format);
    void taskFinished(uint id, bool isWrite, bool ok, bool isGold) const;
    void progressUpdated(int p);
    void setStatus(QString);
    void TempChanged(double t);
//    void statusUpdated(bool connected, std::string devName, quint32 devSerial);

protected:
    virtual bool sectorErase(bool isGoldenImage, uint32_t sizeToErase);
    virtual bool writeFlash(const char * fileData, bool isGoldenImage=false);
    virtual bool verifyFlash(bool isGoldenImage, uint sizeToRead, const char *data);

    virtual quint16 devReadReg(int regAddr);
    virtual void devWriteReg(int regAddr, uint16_t data);
    virtual void devWriteReg32(int regAddr, uint32_t data);
    virtual bool devBlkWriteReg(uint count, const uint *addr, uint16_t *data);
    virtual bool devBlkReadReg(uint count, const uint *addr, uint16_t *data);
    virtual int getMaxBlockSize() const; // max number of 32-bit reg write operations

    virtual void sendProgress(int p);
    virtual std::string getPrefix() const;
    virtual uint getRegBase() const;
    virtual bool canReadFlashId() const;

private:
    mlink::MlinkDeviceInterfaceDebug * const mlinkDevice;
    std::string devName;
    bool isOpen;
    quint64 devSerial;
    unsigned long drvVer;
//    unsigned long drvRev;
    board_info_t board_info;
};

#endif // PEXMLFLASHDEV_H
