#include "MainWindow.h"
#include <QApplication>
#include <QCommandLineParser>
#include <QString>
#include <QDebug>
#include <iostream>

int main(int argc, char *argv[])
{
    QCoreApplication::setOrganizationName("AFI Electronics");
    QCoreApplication::setOrganizationDomain("local");
    QCoreApplication::setApplicationName("PexmlFlashProg");

    int ret;
    try{
        QApplication a(argc, argv);

        QCommandLineParser parser;
        parser.setApplicationDescription("PexmlFlashProg is a program for writing firmware to PEXML PCI-Express boards.\n"
                                         "The software is capable to write golden and regular image.");
        parser.addHelpOption();
        parser.addPositionalArgument("pathToFile", "Location of bit-file with firmware image");

        QCommandLineOption interfaceOption(QStringList() << "i" << "interface",
                                           "set M-Link device driver interface index (default 0)",
                                           "interfaceIndex");
        parser.addOption(interfaceOption);

        QCommandLineOption goldenOption(QStringList() << "g" << "golden",
                                     "write golden image");
        parser.addOption(goldenOption);


        parser.process(QCoreApplication::arguments());

        bool argErr = false;
        QString filePath;
        bool golden=false;
        int interfaceIndex = 0;

        const QStringList positionalArgs = parser.positionalArguments();
        if(!positionalArgs.isEmpty()) {
            if (positionalArgs.size() > 1) {
                qWarning() << "undefined positional arguments:" << positionalArgs;
                argErr = true;
            } else
                filePath = positionalArgs.first().trimmed();
        }

        if(parser.isSet(interfaceOption)) {
            QString valStr = parser.value(interfaceOption);
            bool valOk;
            interfaceIndex = valStr.toInt(&valOk);
            if (!valOk || interfaceIndex<0) {
                qWarning() << QString("bad option '%1' value: %2")
                              .arg(interfaceOption.names().first()).arg(valStr);
                argErr = true;
            }
        }

        if(parser.isSet(goldenOption))
            golden = true;

        if (argErr)
            parser.showHelp(1);

//        qInfo() << QString("Write %1 file as %2 image into %3 mlink").arg(filePath).arg(golden?"golden":"regular").arg(interfaceIndex);

        MainWindow w;
        bool batchMode = !filePath.isEmpty();
        w.init(filePath, golden, interfaceIndex);
        if(batchMode) {
            ret = 0;
        } else {
            w.show();
            ret = a.exec();
        }
    } catch (std::exception &e) {
        std::cerr << "Exception: " << e.what () << std::endl;
    }

    return ret;
}
