#include "ClientConfigConverter.h"

#include "DaqModuleConverter.h"

#include "ClientConfig.h"

namespace {
constexpr const char *k_id = "id";
constexpr const char *k_daq_module = "daq_module";
constexpr const char *k_program_index = "program_index";
constexpr const char *k_readout = "readout";
constexpr const char *k_evNumCheck = "evNumCheck";
constexpr const char *k_has_vme = "has_vme";
constexpr const char *k_protoId = "protoId";
}

template<>
bool AbstractConfigConverter<ClientConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ClientConfig>::toJsonObject(const ClientConfig &in)
{
    QVariantMap m;
    m[k_id] = QVariant::fromValue(in.id);
    m[k_daq_module] = QVariant::fromValue(in.daq_module);
    m[k_program_index] = QVariant::fromValue(in.program_index);
    m[k_readout] = QVariant::fromValue(in.readout);
    m[k_evNumCheck] = QVariant::fromValue(in.evNumCheck);
    m[k_has_vme] = QVariant::fromValue(in.has_vme);
    m[k_protoId] = QVariant::fromValue(static_cast<int>(in.protoId));
    return fromVariantMap(m);
}

template<>
ClientConfig AbstractConfigConverter<ClientConfig>::fromJsonObject(const QJsonObject &in)
{
    ClientConfig out;
    update_value(in, k_id, &out.id);
    update_value(in, k_daq_module, &out.daq_module);
    update_value(in, k_program_index, &out.program_index);
    update_value(in, k_readout, &out.readout);
    update_value(in, k_evNumCheck, &out.evNumCheck);
    update_value(in, k_has_vme, &out.has_vme);
    out.protoId = static_cast<CLIENT_PROTO_ID>(QVariant::fromValue(in[k_protoId]).value<int>());
    return out;
}

template<>
QJsonValue AbstractConfigConverter<ClientConfig>::toJsonValue(const ClientConfig &in)
{
    return toJsonObject(in);
}

template<>
ClientConfig AbstractConfigConverter<ClientConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ClientConfig>::registerSubTypes()
{
    AbstractConfigConverter<DaqModule>::registerTypes();
    qRegisterMetaTypeStreamOperators<ClientConfig>();
}
