/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ClientIndexEnabled.h"

#include <QDebug>

QDebug operator<<(QDebug dbg, const ClientIndexEnabled &r)
{
    dbg.noquote() << QString("{index:\"%1\", enabled:%2}")
                         .arg(r.index.toString())
                         .arg(r.enabled);
    return dbg;
}

ClientIndexEnabledMap::operator ClientIndexSet() const
{
    const auto keys = this->keys();

#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
    return ClientIndexSet(keys.begin(), keys.end());
#else
    ClientIndexSet set;
    for (const auto &i: keys)
        set << i;
    return set;
#endif
}

QString ClientIndexEnabledMap::toString() const
{
    QString to_return;
    for(ClientIndex index : this->keys()){
        to_return.append(QString("Client index: %1; Client index enabled: %2)")
                                 .arg(index.toString()).arg(this->value(index).toString()));
    }
    return to_return;
}

ClientIndexSet ClientIndexEnabledMap::getEnabled() const
{
    ClientIndexSet result;
    for (auto it = cbegin(); it != cend(); ++it)
        if(it->enabled)
            result << it.key();
    return result;
}

ClientIndexEnabledMap::operator ClientIndexEnabledVector() const
{
    ClientIndexEnabledVector result;
    for (auto it = begin(); it != end(); ++it)
        result.push_back(it.value());
    return result;
}

ClientIndexEnabledVector::operator ClientIndexSet() const
{
    ClientIndexSet result;
    for (auto it = begin(); it != end(); ++it)
        result.insert(it->index);
    return result;
}

ClientIndexEnabledVector::operator ClientIndexEnabledMap() const
{
    ClientIndexEnabledMap result;
    for (auto it = begin(); it != end(); ++it)
        result[it->index] = *it;
    return result;
}


