/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CLIENTINDEXENABLED_H
#define CLIENTINDEXENABLED_H

#include <QVector>
#include <QMap>
#include <QMetaType>

#include "ClientIndex.h"

class QDebug;

struct ClientIndexEnabled
{
    ClientIndexEnabled(const ClientIndex &index = ClientIndex(), bool enabled = false)
        : index(index), enabled(enabled) { }

    QString toString() const {return QString("Client index: %1; enabled:%2")
                                     .arg(index.toString()).arg(QString::number(enabled)); }
    bool isEmpty() const { return index.isEmpty(); }

    friend QDebug operator<<(QDebug dbg, const ClientIndexEnabled &r);
    bool operator==(const ClientIndexEnabled &r) const {
        return index == r.index && enabled == r.enabled;
    }
    bool operator!=(const ClientIndexEnabled &r) const {
        return ! (*this == r);
    }

    ClientIndex index;
    bool enabled = false;
};

struct ClientIndexEnabledMap;
struct ClientIndexEnabledVector;

struct ClientIndexEnabledMap : public QMap<ClientIndex, ClientIndexEnabled>
{
    QString toString() const;
    explicit operator ClientIndexSet() const;
    explicit operator ClientIndexEnabledVector() const;
    ClientIndexSet getEnabled() const;
};

Q_DECLARE_METATYPE(ClientIndexEnabledMap)

struct ClientIndexEnabledVector : public QVector<ClientIndexEnabled>
{
    explicit operator ClientIndexSet() const;
    explicit operator ClientIndexEnabledMap() const;
};

Q_DECLARE_METATYPE(ClientIndexEnabledVector)

#endif // CLIENTINDEXENABLED_H
