/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DaqModule.h"

#include <QDataStream>
#include <QMap>

const QMap<DaqModule::Type, QPair<std::string, QString>> DAQ_MODULE_DESCR =
{
    {DaqModule::Type::Adc64,            {"Adc64",               "ADC64"}},
    {DaqModule::Type::Adc64System,      {"Adc64System",         "ADC64System"}},
    {DaqModule::Type::AfiViewer,        {"AfiViewer",           "AFI Viewer"}},
    {DaqModule::Type::DeviceBrowser,    {"DeviceBrowser",       "Device Browser"}},
    {DaqModule::Type::EventBuilder,     {"EvB",                 "EventBuilder"}},
    {DaqModule::Type::Generator,        {"EvbGen",              "Generator"}},
    {DaqModule::Type::HrbAnalyzer,      {"HrbAnalyzer",         "HRBAnalyzer"}},
    {DaqModule::Type::HrbSystem,        {"HrbSystem",           "HRBSystem"}},
    {DaqModule::Type::Ideas,            {"Ideas",               "IDEAS"}},
    {DaqModule::Type::Land,             {"Land",                "LAND"}},
    {DaqModule::Type::MetricAggregator, {"MetricAggregator",    "Metric Aggregator"}},
    {DaqModule::Type::MongoViewer,      {"MongoViewer",         "MongoDB Viewer"}},
    {DaqModule::Type::MpdRawstat,       {"MpdRawstat",          "MPD Rawstat"}},
    {DaqModule::Type::Msc16Ve,          {"MSC16VE",             "MSC16VE"}},
    {DaqModule::Type::Msc16VeTm,        {"MSC16VE-TM",          "MSC16VE-TM"}},
    {DaqModule::Type::MStream,          {"MStream",             "MStream"}},
    {DaqModule::Type::PhosFec,          {"PhosFec",             "PHOS FEC"}},
    {DaqModule::Type::PnpBrowser,       {"PnpBrowser",          "PnP Browser"}},
    {DaqModule::Type::RunControl,       {"RunControl",          "RunControl"}},
    {DaqModule::Type::Scheduler,        {"Scheduler",           "Scheduler"}},
    {DaqModule::Type::TdcControl,       {"TDC2-CTRL",           "TdcControl"}},
    {DaqModule::Type::TQDC,             {"TQDC2",               "TQDC"}},
    {DaqModule::Type::Ttvxs,            {"Ttvxs",               "TTVXS"}},
    {DaqModule::Type::VmeDaq,           {"VmeDaq",              "VMEDAQ"}},
};

std::string DaqModule::toStdString() const
{
    return DAQ_MODULE_DESCR.value(type).first;
}

DaqModule DaqModule::fromStdString(const std::string &s)
{
    for(auto it=DAQ_MODULE_DESCR.cbegin(); it!=DAQ_MODULE_DESCR.cend(); ++it) {
        if(s == it.value().first)
            return DaqModule(it.key());
    }

    return DaqModule(Type::Default);
}

DaqModule DaqModule::fromString(const QString &s)
{
    return fromStdString(s.toStdString());
}

QString DaqModule::prettyName() const
{
    return DAQ_MODULE_DESCR.value(type).second;
}

QDebug operator<<(QDebug dbg, const DaqModule &r)
{
    dbg << r.prettyName();
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const DaqModule &value)
{
    out << value.toString();
    return out;
}

QDataStream &operator>>(QDataStream &in, DaqModule &value)
{
    QString s;
    in >> s;
    if (in.status() == QDataStream::Ok) {
        value = DaqModule::fromString(s);
    }
    return in;
}
