/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BASEMAINWINDOW_H
#define BASEMAINWINDOW_H

#include <QMainWindow>

class QSettings;

class BaseMainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit BaseMainWindow(QWidget *parent = nullptr);
    virtual ~BaseMainWindow() override;

protected slots:
    void actionQuit();
    void actionAbout();
    virtual void addLog(QtMsgType severity, const QString &log, const QString &thrName);
    virtual void addLogMessage(QtMsgType severity, const QString &category, const QString &log, const QString &thrName);
    virtual void delayedInit();

protected:
    virtual void closeEvent(QCloseEvent *event) override;
    virtual void setupMenu(QMenu *file = nullptr, QMenu *help = nullptr, QMenu *options = nullptr);
    void saveMainWindowPosition() const;
    void loadMainWindowPosition();
    virtual void saveAppGuiSettings(QSettings &) const {}
    virtual void loadAppGuiSettings(const QSettings &) {}

    const QString SETTING_GROUP_MAINWINDOW = "MainWindow";

private:
    QMenu *menuFile = nullptr;
    QMenu *menuHelp = nullptr;
    QMenu *menuOptions = nullptr;
};

#endif // BASEMAINWINDOW_H
