/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef GUI_UTILS_H
#define GUI_UTILS_H

#include <QMap>
#include <QWidget>

class WidgetSignalBlocker {
public:
    WidgetSignalBlocker(QWidget *widget)
        : w(widget), state(w ? w->blockSignals(true) : false) { }
    ~WidgetSignalBlocker() {
        if (w)
            w->blockSignals(state);
    }
private:
    QWidget * const w;
    bool state;
};

struct ObjectBlockState
{
    ObjectBlockState(QObject *p, bool state) : p(p), state(state) { }
    QObject *p;
    bool state;
};

class MultiSignalBlocker {
public:
    MultiSignalBlocker(const QList<QObject *> &list) {
        for (auto p : list) {
            if (!p)
                continue;
            QObject *obj = p;
            blockStates.push_back(ObjectBlockState(obj, obj->blockSignals(true)));
        }
    }
    ~MultiSignalBlocker() {
        for (auto i : blockStates)
            i.p->blockSignals(i.state);
    }
private:
    QList<ObjectBlockState> blockStates;
};

#endif // GUI_UTILS_H
