#ifndef BASEFSM_H
#define BASEFSM_H

#include <QElapsedTimer>
#include <QObject>

#include "FsmState.h"
#include <QSharedData>

struct BaseFsmInput : public QSharedData
{
    //    static QString toString(const RcFsmInput &r);
    void clearRequests() {
        requestState = FsmState::Init;
    }

    FsmState requestState = FsmState::Init;
//    bool underRemoteControl = false;
};

class BaseFsm: public QObject
{
    Q_OBJECT

public:
    explicit BaseFsm(QObject *parent = nullptr);
    FsmState getState() const { return state; }


public slots:
    void goState(const FsmState &nextState);
    virtual void printFsmInput() const;
//    void remoteStatusUpdated(bool underRemoteControl);

signals:
    void stateChanged(const FsmState &cur, const FsmState &old) const;

protected slots:
    virtual bool evaluate();

protected:
    virtual void printInputString(QDebug dbg) const;
    FsmState getRequestState() const { return input->requestState; }
    virtual qint64 stateTimeoutMs(const FsmState &state) const;
    void setState(const FsmState &nextState);
    void acceptRequestedState();
    BaseFsmInput* getInput() { return input; }
    const BaseFsmInput* getInput() const { return input; }
    void recreateInput(BaseFsmInput* p);

private:
    bool stateTimeout() const;

    QElapsedTimer stateTime;
    FsmState state = FsmState::Init;
    QSharedDataPointer<BaseFsmInput> input;
};

QDebug operator<<(QDebug dbg, const BaseFsmInput &state);

#endif // BASEFSM_H
