#ifndef DEVICEMGRFSM_H
#define DEVICEMGRFSM_H

#include "BaseFsm.h"
#include <QMap>

#include "base/DeviceIndex.h"

class DeviceIndexEnabledMap;

struct DeviceMgrFsmInput : public BaseFsmInput
{
    int setupKey = 0;
    QElapsedTimer lastSetupKeyUpdate;
    DeviceIndexSet devSet;
    QMap<DeviceIndex, int> deviceSetupKeys;
};

class DeviceMgrFsm : public BaseFsm
{
    Q_OBJECT
public:
    explicit DeviceMgrFsm(QObject *parent = nullptr);

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &newList);
    void setMainSetupKey(int key);
    void setDevSetupKey(const DeviceIndex &index, int key);

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline DeviceMgrFsmInput *getInput() {
        return static_cast<DeviceMgrFsmInput*>(BaseFsm::getInput());
    }
    inline const DeviceMgrFsmInput *getInput() const {
        return static_cast<const DeviceMgrFsmInput*>(BaseFsm::getInput());
    }

    bool checkDevKey(int mainKey, const QMap<DeviceIndex, int> &keyMap) const;
    bool allDevicesWritten() const;
    bool setupKeyGuargExpired() const;

    const int DEVICE_CONFIG_WRITE_TIMEOUT = 600; // in ms
};

QDebug operator<<(QDebug dbg, const DeviceMgrFsmInput &in);


#endif // DEVICEMGRFSM_H
