/*
**    Merge two table models by rows
**
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MERGECOLUMNSTABLEMODEL_H
#define MERGECOLUMNSTABLEMODEL_H

#include <QAbstractTableModel>

class MergeColumnsTableModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    MergeColumnsTableModel(QObject *parent = nullptr);

    QModelIndex mapFromSource(const QModelIndex &source) const;
    QModelIndex mapToSource(const QModelIndex & proxy) const;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;

    virtual QVariant data(const QModelIndex &index, int role) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;

    void setSourceModelA(QAbstractTableModel *modelA);
    void setSourceModelB(QAbstractTableModel *modelB);

signals:
    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight,
                       const QVector<int> &roles = QVector<int>());
    void modelReset();

private slots:
    void onDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight,
                       const QVector<int> &roles = QVector<int>());

    void sourceModelReset();

private:
    void connectSourceModel(QAbstractTableModel *model);
    QAbstractTableModel *modelA = nullptr;
    QAbstractTableModel *modelB = nullptr;
};

#endif // MERGECOLUMNSTABLEMODEL_H
