#include "DeviceIdListWidget.h"

#include "mldiscover/LldpDiscovery.h"

DeviceIdListWidget::DeviceIdListWidget(QWidget *parent)
    : QListWidget(parent)
{
    setSelectionMode(QListWidget::ExtendedSelection);
    addDeviceId(DEVICE_ID_AnyDevice, "_All_");
}

void DeviceIdListWidget::lldpCacheUpdated(const LldpInfoCache &cache)
{
    for(auto &lldp : cache) {
        if(!knownTypes.value(lldp.device_id).contains(lldp.getModelStr())) {
            addDeviceId(lldp.device_id, lldp.getModelStr());
        }
    }
}

void DeviceIdListWidget::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
    QListWidget::selectionChanged(selected, deselected);

    DeviceIdSet id_set;
    const auto &indices = selectedIndexes();
    for (auto it = indices.begin(); it != indices.end(); ++it) {
        const auto &id = it->data(Qt::UserRole).toUInt();
        id_set << static_cast<DeviceId>(id);
    }
    emit setDeviceIdFilter(id_set);
}

void DeviceIdListWidget::addDeviceId(quint16 id, QString name)
{
    bool new_id = !knownTypes.contains(id);
    auto &list = knownTypes[id];
    if(!name.isEmpty() && !list.contains(name)) {
        list << name;
    }

    auto item_name = list.join(' ');
    if(item_name.isEmpty())
        item_name = getDeviceTypeName(id);

    if(new_id) {
        auto id_item = new QListWidgetItem(item_name, this);
        id_item->setData(Qt::UserRole, id);
        id_item->setToolTip(QString("0x%1").arg(id, 2, 16, QChar('0')));
        addItem(id_item);
        id_item->setSelected(true);
    } else {
        for(int row=0; row<count(); ++row) {
            auto i = item(row);
            if(i->data(Qt::UserRole) == id) {
                i->setText(item_name);
                return;
            }
        }
        Q_ASSERT(false);
    }
}
