/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef LLDPINFO_H
#define LLDPINFO_H

#include <QDebug>
#include <QMetaType>
#include <QPair>
#include <QString>

#include "mldp.h"

class DeviceIndex;

struct LldpInfo
{
    bool isNull() const { return !valid; }
    static QString formatVersion(const QString &input);

    bool operator==(const LldpInfo &r) const;
    explicit operator DeviceIndex() const;

    QString getIdent() const;
    QString getSerialStr() const;
    QString getModelStr() const;
    QString getModelVerStr() const; // "BOARD-1.2"
    QString getFirmWareStr() const;
    int getHwMajVer() const;
    static int getHwMajVer(const QString &str);
    QPair<int,int> getHwVer() const;
    static QPair<int,int> getHwVer(const QString &str);

    // LLDP
    quint64 haddr = 0; // mac-address: 1811275143411
    quint32 ip_addr = 0; // 173897917 = 0x0A5D78BD = 10.93.120.189
    quint32 ttl = 0; // Time-to-live: 8

    // LLDP-MED
    QString portID; // id for the port from which this LLDPDU was transmitted: "2"
    //    QString hw;
    QString hw_str; // 5 Hardware Revision, formatted: "1.0"
    QString fw_str; // 6 Firmware Revision, formatted: "2.0.19676"
    QString sn_str; // 8 Serial Number: "0000-0569-59F2"
    QString mn_str; // 9 Manufacturer Name: "JINR AFI"
    QString modelname_str; // 10 Model Name: "        U40VE-RC"

    // OUI_AFI - 1
    quint16 device_id = 0; // (hex) cc
    quint64 serial_id = 0; // (hex) 56959f2
    quint16 chassis_slot = 0;

    // OUI_AFI - 2
    quint64 master_mac = 0;
    quint32 master_ip = 0;
    quint16 master_udp_port = 0;
    quint64 mstream_mac = 0;
    quint32 mstream_ip = 0;
    quint16 mstream_udp_port = 0;
    bool mstream_locked = false;
    bool master_locked = false;

    bool valid = false;
};

Q_DECLARE_METATYPE(LldpInfo)

QDebug operator<<(QDebug dbg, const LldpInfo &r);

#endif // LLDPINFO_H
