#include "Ad5328Module.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "device-utils/DacHelper.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;


enum {
    REG_DAC_CTRL_BASE = 0x0, // for channels 0..7
//    REG_DAC_CTRL_DAC2 = 0x1  // for channels 8..15
};

enum {
    DAC_CMD_RESET           = 0xF000,
    DAC_CMD_GAIN            = 0x8000,
    DAC_CMD_SINGLE_UPDATE   = 0xA002
};


Ad5328Module::Ad5328Module(quint16 bAddr, QtMregDevice * const _dev)
    :BaseDeviceModule(_dev)
{
    if(dev->getDeviceId() == DEVICE_ID_PHOS_FEC) {
        setBaseAddr(bAddr, 0xFF);
    }
}

mlink::RegOpVector Ad5328Module::setChDacDataRegOp(const QMap<int, int> &chDacData) const
{
    RegOpVector regOp;

    for(int dacId=0 ; dacId<getChipCnt(); ++dacId) {
        for(int dacVOut=0 ; dacVOut<getChipCh(); ++dacVOut) {
            const int &globCh = dacId*getChipCh() + dacVOut;

            quint16 dacData = chDacData[globCh] & 0xFFF;
            dacData |= dacVOut<<12;
            regOp << RegWrite(baseAddr() + REG_DAC_CTRL_BASE + dacId, dacData);
        }
        regOp << RegWrite(baseAddr() + REG_DAC_CTRL_BASE + dacId, DAC_CMD_SINGLE_UPDATE);
    }
    return regOp;

}
/*
 * 0x8000 - Gain of output amplifier and reference selection
 *     03 - Vdd as ref.
 *     0C - buffered
 *     30 - GAIN ctrl
 */
void Ad5328Module::resetDac()
{
    RegOpVector regOp;
    for(int daqId=0; daqId<getChipCnt(); ++daqId) {
        regOp << RegWrite(baseAddr() + REG_DAC_CTRL_BASE + daqId, DAC_CMD_RESET); // DAC Data and control reset

        quint16 gainCmd = DAC_CMD_GAIN; // default GAIN control
//        gainCmd |= 0x03; // set Vdd as ref.
        gainCmd |= 0x0C; // set Buffered input
        if(useDoubleGain)
            gainCmd |= 0x30; // set double GAIN
        regOp << RegWrite(baseAddr() + REG_DAC_CTRL_BASE + daqId, gainCmd);
    }

    dev->regOpExec(regOp);
}
