#include "AdcSpiAds52J90Module.h"

#include <QDebug>
#include <QTime>
#include <QThread>

#include "mregdev/qtmregdevice.h"

namespace  {
enum {
    REG_ADS52J90_SPI_ADDR = 0x0,
    REG_ADS52J90_SPI_WR_DATA = 0x1,
    REG_ADS52J90_SPI_READ_DATA = 0x2,
    REG_ADS52J90_ACTIVE_ADC_SELECT = 0x4
};
}

AdcSpiAds52J90Module::AdcSpiAds52J90Module(QtMregDevice *_qtMregDevice)
    :AdcSpiBaseModule(_qtMregDevice)
{
    if(dev->getDeviceId() == DEVICE_ID_PHOS_FEC)
        setBaseAddr(0x4100, 0xFF);
}

void AdcSpiAds52J90Module::spi_test()
{
    qDebug() << __PRETTY_FUNCTION__ << "started";

    qsrand(QTime::currentTime().msecsSinceStartOfDay());

//    QSet<quint16> randVal;
//    randVal << 0xa5a5;
    for(int adcId = 0; adcId<adcChips; ++adcId) {
        quint16 xormask = 0;
        quint16 error_number = 0;
        const auto &adcPrefix = QString("[adcId:%1]").arg(adcId);
        quint16 active_adc_mask = 1<<adcId;
        for(int j=0; j<100; j++)
        {
            quint16 rand_val = qrand() & 0xFFF;
            spi_write(0x5, rand_val, active_adc_mask);
            quint16 readregval = spi_read(0x5, active_adc_mask);
            if (rand_val != readregval)
            {
                xormask |= rand_val ^ readregval;
                error_number++;
                if (error_number < 30)
                    qInfo() << adcPrefix << "write " << hex << rand_val << "read" << readregval << "XOR" << (rand_val ^ readregval);
            }
        }

        if (error_number == 0)
            qDebug() << adcPrefix << "SPI random data test OK";
        else
            qWarning() << adcPrefix << "Error number = " << error_number
                       << " xormask=0x" << hex << xormask;
    }
    qDebug() << __PRETTY_FUNCTION__ << "finished";
}

void AdcSpiAds52J90Module::setupAdc(quint8 adcBits, quint8 adcChPerChip)
{
    qDebug() << "ADS52J90 setup";

    const quint16 &adc_mask=getAdcMask();
    spi_write(0xA, 0x3000, adc_mask); //after reset
    quint16 data1h = 0;
    switch (adcChPerChip) {
    case 8: data1h = 0x0094; break;
    case 16: data1h = 0x0014; break;
    case 32: data1h = 0x0000; break;
    default:;
    }
    spi_write(0x1, data1h, adc_mask);
    spi_write(0x2, 0, adc_mask); // normal data operation

    quint16 adcSerialization;
    quint16 adcResolution;
    switch(adcBits) {
    case 10:
        adcSerialization = 0x6000; // 10X
        adcResolution = 0x3;
        break;
    case 16:
        adcSerialization = 0x8000; // 16X
        adcResolution = 0x1;
        break;
    case 12:
    default:
        adcSerialization = 0; // 12X
        adcResolution = 0;
    }
    spi_write(0x3, adcSerialization, adc_mask);
    spi_write(0x4, adcResolution, adc_mask);

    spi_test();

    //setCustomPatternMode(true);
    //setCustomPattern(0x40);
}

void AdcSpiAds52J90Module::setCustomPatternMode(bool mode)
{
    spi_write(0x2, mode ? 0x0180 : 0, 0xFF);
}

void AdcSpiAds52J90Module::setCustomPattern(quint16 data)
{
    spi_write(0x5, data, 0xFF);
}

void AdcSpiAds52J90Module::spi_write(quint16 addr, quint16 data, quint16 device_mask)
{
//const quint16 device_mask_bak = device_mask;
//for(int adcId=0; adcId<adcChips; ++adcId) {
//        device_mask = device_mask_bak & (1<<adcId);
//        if(device_mask == 0) continue;
    dev->regWrite(baseAddr() + REG_ADS52J90_ACTIVE_ADC_SELECT, device_mask);
    dev->regWrite(baseAddr() + REG_ADS52J90_SPI_WR_DATA, data);
    dev->regWrite(baseAddr() + REG_ADS52J90_SPI_ADDR, addr);

    for(int i=0; i<10; i++)
        dev->regRead(baseAddr() + REG_ADS52J90_ACTIVE_ADC_SELECT);
//}
}

quint16 AdcSpiAds52J90Module::spi_read(quint16 addr, quint16 device_mask)
{
    spi_write(0x0, 0x2, device_mask);
    dev->regWrite(baseAddr() + REG_ADS52J90_ACTIVE_ADC_SELECT, device_mask);
//    dev->regWrite(spiAdcBase + REG_ADS52J90_SPI_WR_DATA, 0);
    dev->regWrite(baseAddr() + REG_ADS52J90_SPI_ADDR, addr);

    for(int i=0; i<10; i++)
        dev->regRead(baseAddr() + REG_ADS52J90_ACTIVE_ADC_SELECT);

    quint16 readval = dev->regRead(baseAddr() + REG_ADS52J90_SPI_READ_DATA);
    spi_write(0x0, 0x0, device_mask);

    return readval;
}
