#ifndef BASEDEVICEMODULE_H
#define BASEDEVICEMODULE_H

#include <QtCore>

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

class QtMregDevice;

class BaseDeviceModule
{
public:
    BaseDeviceModule(QtMregDevice *qtMregDevice = nullptr)
        :dev(qtMregDevice) {}
    virtual ~BaseDeviceModule() {}
    void init();
    virtual void post_init() {};
    bool isAvailable() const { return addrRange.second; }

protected:
    virtual AFI_SDB_ID getSdbId() const = 0;
    void setBaseAddr(quint16 baseAddr, quint16 mask);
    quint16 baseAddr() const;

    template <typename T>
    void checkRegResValid(mlink::RegOpVector (T::* regOpPtr)() const,
               const mlink::RegOpVector &regRes,
               int pos) const
    {
#ifdef QT_DEBUG
        const auto &regOp = (static_cast<const T*>(this)->*regOpPtr)();
        Q_ASSERT(regRes.size() >= pos+regOp.size());
//        Q_ASSERT(regRes[pos].addr == regOp.first().addr
//                 && regRes[pos+regOp.size()-1].addr == regOp.last().addr);
        for(auto op : regOp)
            Q_ASSERT(regRes[pos++].addr == op.addr);
#else
        Q_UNUSED(regOpPtr) Q_UNUSED(regRes) Q_UNUSED(pos)
#endif
    }

    QPair<quint16, quint16> addrRange;
    QtMregDevice * const dev;
};

#endif // BASEDEVICEMODULE_H
