#include "BmcModule.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "BmcStatus.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

// MCU SPI Slave
enum {
    REG_BMC_VERSION    = 0x00,
    REG_PLL_STATUS     = 0x01,
    REG_PLL_UNLOCK_CNT = 0x02, // [12:0] - unlock counter
    REG_BMC_TEMP_1     = 0x03,
    REG_BMC_TEMP_2     = 0x04,
    REG_BMC_TEMP_3     = 0x05,
    REG_BMC_TEMP_4     = 0x06,
    REG_BMC_REVERSION  = 0x07
};

BmcModule::BmcModule(QtMregDevice * const dev)
    :BaseDeviceModule(dev)
{
    if(dev->getDeviceId() == DEVICE_ID_PHOS_FEC)
        setBaseAddr(0x6000, 0xFF);
}

mlink::RegOpVector BmcModule::getStatusRegOp() const
{
    RegOpVector regOp;
    regOp << RegRead(baseAddr() + REG_BMC_VERSION);
    regOp << RegRead(baseAddr() + REG_PLL_STATUS);
    regOp << RegRead(baseAddr() + REG_PLL_UNLOCK_CNT);
    regOp << RegRead(baseAddr() + REG_BMC_TEMP_1);
    regOp << RegRead(baseAddr() + REG_BMC_TEMP_2);
    regOp << RegRead(baseAddr() + REG_BMC_TEMP_3);
    regOp << RegRead(baseAddr() + REG_BMC_TEMP_4);
    regOp << RegRead(baseAddr() + REG_BMC_REVERSION);

    return regOp;
}

BmcStatus BmcModule::readStatus() const
{
    const auto &regOp = getStatusRegOp();
    auto regRes = dev->regOpExec(regOp);
    if(regRes.size() != regOp.size())
        return BmcStatus();

    const auto &bmcSt = getStatus(regRes, 0);
    return bmcSt;
}

BmcStatus BmcModule::getStatus(const mlink::RegOpVector &regRes, int posOffset) const
{
    checkRegResValid(&BmcModule::getStatusRegOp, regRes, posOffset);

    BmcStatus st;
    int p = posOffset;
    st.fw_ver = regRes[p++].data;
    st.pllStatus = regRes[p++].data & 0xFFF;
    st.pllUnlockCnt = regRes[p++].data & 0xFFF;
    st.temp1 = regRes[p++].data;
    st.temp2 = regRes[p++].data;
    st.temp3 = regRes[p++].data;
    st.temp4 = regRes[p++].data;
    st.fw_rev = regRes[p++].data;
    return st;
}
