#ifndef DESERIALIZERSCONTROLMODULE_H
#define DESERIALIZERSCONTROLMODULE_H

#include "device-modules/BaseDeviceModule.h"

#include "DeserializersControlStatus.h"
#include "AdcSerDesDelay.h"

class DeserializersControlModule : public BaseDeviceModule
{
public:
    DeserializersControlModule(QtMregDevice * const dev);
    void post_init() override;
    mlink::RegOpVector getClockDelaySetRegOp(quint16 clockDelay) const;
    mlink::RegOpVector getDataDelaySetRegOp(quint16 dataDelay) const;
    mlink::RegOpVector getStatusRegOp() const;
    DeserializersControlStatus getStatus(mlink::RegOpVector regRes, int pos) const;
    DeserializersControlStatus readStatus();
    QVector<quint16> getChannelData();
    void setDeserReset(bool en);
    void ResetAdcPin();
    void setAdcSync();
    void ResetDeser();

    void ResetAdcStatus();
    void setClockDelay(quint16 clkDelay);
    void setDataDelay(quint16 dataDelay);
    void setSerDesDelay(const DevSerDesDelay &delay);

    void ResetAdcClkDelay();
    quint16 lockTest();
    bool isLockOk(quint16 des_status, int desId = -1) const
    {
        for(int i=0; i<desCnt; ++i) {
            bool desOk = ((des_status>>(2*i)) & 3) == 1;
            if(desId == i) {
                if(desOk)
                    return desOk;
                else
                    return desOk;
            }
            if(!desOk)
                return false;
        }
        return true;
    }

protected:
    AFI_SDB_ID getSdbId() const final { return AFI_SDB_Deserializers_Contr; }

private:
    void changeDesDelay(quint16 channel_mask, quint16 taps);
    void readBuildParams();
    void updateDesCtrlVal();
    void resetCtrlBit(quint16 bit);

    quint8 desCnt=0;
    quint8 adcChPerChip=0;

    quint16 curDesCtrlVal = 0;
    DeserializersControlStatus desSt;
};

#endif // DESERIALIZERSCONTROLMODULE_H
