#include "RunStatisticModule.h"


#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "RunStatistic.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

// TADC Hit Core Registers
enum {
    REG_RUN_UPTIME              = 0x00, // Level || Run   ||
    REG_LEMO_INPUT_PULSE_CNT    = 0x04, // Edge  || Total ||
    REG_TTC_INPUT_PULSE_CNT     = 0x08, // Edge  || Total ||
    REG_VXS_INPUT_PULSE_CNT     = 0x0C, // Edge  || Total ||
    REG_FELINK_INPUT_PULSE_CNT  = 0x10, // n/a   || n/a   ||
    REG_TRIGGER_CANDIDATES      = 0x14, // Edge  || Run   ||
    REG_ACCEPTED_TRIGGERS       = 0x18, // Edge  || Run   ||
    REG_BLOCKED_TRIGGERS        = 0x1C, // Edge  || Run   ||

    REG_TOTAL_BUSY_TIME         = 0x40, // Level || Run   ||
    REG_ADC_BUSY_TIME           = 0x44, // Level || Run   ||
    REG_TDC_BUSY_TIME           = 0x48, // Level || Run   ||
    REG_READOUT_BUSY_TIME       = 0x4C, // Level || Run   ||
};

enum {
    REG_HIT_COUNTER_LOCK_BIT = 0x0001
};

RunStatisticModule::RunStatisticModule(QtMregDevice * const dev)
    :BaseDeviceModule(dev)
{
    if(dev->getDeviceId() == DEVICE_ID_PHOS_FEC) {
        setBaseAddr(0x700, 0xFF);
    }
}

mlink::RegOpVector RunStatisticModule::getRunStatisticRegOp() const
{
    RegOpVector regOp;
    for(int reg=REG_RUN_UPTIME; reg<=REG_BLOCKED_TRIGGERS; reg+=4)
        regOp << RegRead64(baseAddr() + reg);

    for(int reg=REG_TOTAL_BUSY_TIME; reg<=REG_READOUT_BUSY_TIME; reg+=4)
        regOp << RegRead64(baseAddr() + reg);

    return regOp;
}

RunStatistic RunStatisticModule::getRunStatistic(const mlink::RegOpVector &regRes, int posOffset) const
{
    checkRegResValid(&RunStatisticModule::getRunStatisticRegOp, regRes, posOffset);

    RunStatistic s;
    int pos = posOffset;
    s.runUptime = QtMregDevice::get64(regRes, pos); pos += 4;
    s.lemoInputPulses = QtMregDevice::get64(regRes, pos); pos += 4;
    s.ttcInputPulses = QtMregDevice::get64(regRes, pos); pos += 4;
    s.vxsInputPulses = QtMregDevice::get64(regRes, pos); pos += 4;
    s.felinkInputPulses = QtMregDevice::get64(regRes, pos); pos += 4;
    s.triggerCandidates = QtMregDevice::get64(regRes, pos); pos += 4;
    s.acceptedTrig = QtMregDevice::get64(regRes, pos); pos += 4;
    s.blockedTriggers = QtMregDevice::get64(regRes, pos); pos += 4;

    s.totalBusyTime = QtMregDevice::get64(regRes, pos); pos += 4;
    s.adcBusyTime = QtMregDevice::get64(regRes, pos); pos += 4;
    s.tdcBusyTime = QtMregDevice::get64(regRes, pos); pos += 4;
    s.readoutBusyTime = QtMregDevice::get64(regRes, pos); pos += 4;

    return s;
}
