#ifndef TDCCOREMODULE_H
#define TDCCOREMODULE_H

#include "device-modules/BaseDeviceModule.h"

struct TdcCoreStatus;
struct HptdcAdjustConfig;

typedef union {
   quint32 all;
   struct {
       unsigned int leading_resolution: 3;  // 2:0
       unsigned int width_select: 4;        // 6:3 //unused
       unsigned int unused: 1;              // 7
       unsigned int mode_rc_compression: 1; // 8
       unsigned int mode_rc: 1;             // 9
       unsigned int DLL_Mode: 2;            // 11:10
       unsigned int enable_leading: 1;      // 12
       unsigned int enable_trailing: 1;     // 13
       unsigned int enable_pair: 1;         // 14
       unsigned int core_clock_source: 2;   // 16:15
       unsigned int dll_clock_source: 3;    // 19:17
       unsigned int unused2: 4;             // 23:20
       unsigned int enable_matching: 1;     // 24     //unused
       unsigned int unused3: 7;             // 31:25
   } bit;
} reg_measurement_ctrl;

typedef union {
    quint32 all;
    struct {
        unsigned int enable_local_header: 1;      // 0
        unsigned int enable_local_trailer: 1;     // 1
        unsigned int unused: 2;                   // 3:2
        unsigned int enable_error_mark: 1;        // 4
        unsigned int unused2: 3;                  // 7:5
        unsigned int readout_fifo_size: 3;        // 10:8 //used
        unsigned int unused3: 1;                  // 11
        unsigned int max_event_size: 4;           // 15:12
        unsigned int reject_readout_fifo_full: 1; // 16
        unsigned int unused4: 3;                  // 19:17
        unsigned int dead_time: 2;                // 21:20
        unsigned int unused5: 10;                 // 31:22
    } bit;
} reg_read_ctrl;

//const QList<QPair<QString, int>> LEAD_RES_VALUES = {
//    {"clock_40",      0b000},
//    {"pll_clock_40",  0b001},
//    {"pll_clock_160", 0b010},
//    {"pll_clock_320", 0b011},
//    {"aux_clock",     0b100}
//};

class TdcCoreModule : public BaseDeviceModule
{
public:
    TdcCoreModule(QtMregDevice * const dev);
    int getChipNum() const { return chipNum; }
    mlink::RegOpVector setStandbyRegOp(bool standBy) const;
    mlink::RegOpVector setChEnMaskRegOp(std::vector<quint32> chMask) const;
    mlink::RegOpVector setMatchWinAndLatRegOp(quint32 mW, quint32 lat) const;
    void setMeasurementCtrl(bool enVHR, bool enLead, bool enTrail);
    mlink::RegOpVector setHptdcAdjustConfigRegOp(const HptdcAdjustConfig &adjustConfig);

    mlink::RegOpVector getStatusRegOp() const;
    TdcCoreStatus getStatus(const mlink::RegOpVector &regRes, int posOffset=0) const;

    bool waitHptdcInit() const;
protected:
    AFI_SDB_ID getSdbId() const final { return AFI_SDB_TDC_Trigger_Control; }
    void post_init() override;

private:
    void restoreDefaultValues();

    int chipNum = 0;
    reg_read_ctrl readCtrlVal;
    reg_measurement_ctrl measurementCtrlVal;
};

#endif // TDCCOREMODULE_H
