#include "TdcCoreStatus.h"

#include <QStringList>

namespace {
const quint16 HPTDC_ERR_MASK = 0x7FF;
const QString HPTDC_ST_STR[16] = {
    "Vernier Error",
    "Coarse Error",
    "Channel Select Error",
    "L1 Buffer Parity Error",
    "Trigger FIFO Parity Error",
    "Trigger Matching State Error",
    "Readout FIFO Parity Error",
    "Readout State Error",
    "Setup Parity Error",
    "Control Parity Error",
    "JTAG Instruction Error",
    "TDC Have Readout Token",
    "Readout FIFO Full",
    "Readout FIFO Empty",
    "Trigger FIFO Full",
    "Trigger FIFO Empty"
};
} //namespace


bool HptdcStatus::isOk() const
{
    return (regValues[0] & HPTDC_ERR_MASK) && isDllLock();
}

bool HptdcStatus::isDllLock() const
{
    return regValues[1] & (1<<12);
}

bool HptdcStatus::hasError() const
{

    return !isDllLock() || (regValues[0] & 0x7F);
}

QString HptdcStatus::getSummary() const
{
    QStringList sList;
    for(int i=0; i<16; ++i)
        if(regValues[0] & (1<<i) & HPTDC_ERR_MASK)
            sList << HPTDC_ST_STR[i];

    //    quint8 readOutFifoOccup = regValues[1] & 0xFF;
    //    quint8 trigFifoOccup = (regValues[1]>>12) & 0xFF;

    //    quint8 occupL1Buffer[4];
    //    occupL1Buffer[0] = regValues[2] & 0xFF;
    //    occupL1Buffer[1] = (regValues[2]>>12) & 0xFF;
    //    occupL1Buffer[2] = regValues[3] & 0xFF;
    //    occupL1Buffer[3] = (regValues[4]>>12) & 0xFF;
    if(!isDllLock())
        sList << QString("Dll %1lock").arg(isDllLock()?"":"un");

    return sList.join(' ');
}
