#include "TrigCtrlStatModule.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"


using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

// Trigger control & statistic
enum {
    REG_TRIG_CSR                = 0x00,
    REG_TRIG_SRC                = 0x01,
    REG32_TRIG_TIMER_PERIOD     = 0x02, // 32bit RW
    REG64_TRIG_EVENT_NUM_LOAD   = 0x04, // 64bit RW
    REG64_TRIG_CUR_EVENT_NUM    = 0x08, // 64bit RO
    REG_TRIG_QUEUE              = 0x0C, // RO
    REG_TRIG_BUSY_STATUS        = 0x0D, // RO
    REG_TRIG_SELFTRIG_DELAY     = 0x0F, // RW 24ns step, 1us max
    REG64_SELF_TRIG_MASK        = 0x10,

//    REG64_TRIG_CNT_ALL          = 0x10, // 64bit RO //todo: findIt
//    REG64_TRIG_CNT_CANDID       = 0x14, // 64bit RO //todo: findIt
//    REG64_TRIG_CNT_ACCEPTED     = 0x18, // 64bit RO //todo: findIt
//    REG64_RUN_TIME_NS           = 0x1C, // 64bit RO
};

enum {
    REG_TRIG_CSR_BIT_CNT_RESET = 1,
    REG_TRIG_CSR_BIT_RUN_CNT_RESET = 2,
    REG_TRIG_CSR_BIT_CNT_LOCK = 4,
};

enum {
    REG_TRIG_SRC_BIT_TIMER = 1,
    REG_TRIG_SRC_BIT_LEMO = 2,
    REG_TRIG_SRC_BIT_TTC = 4,
};


TrigCtrlStatModule::TrigCtrlStatModule(QtMregDevice * const dev)
    :BaseDeviceModule(dev)
{
    if(dev->getDeviceId() == DEVICE_ID_PHOS_FEC)
        setBaseAddr(0x100, 0xFF);

}

mlink::RegOpVector TrigCtrlStatModule::setCtrlRegOp(quint16 regVal) const
{
    return RegOpVector();
//    return RegWrite(baseAddr + REG_TRIG_CSR, REG_DEVICE_CTRL_BIT_RUN); // set RUN
}

mlink::RegOpVector TrigCtrlStatModule::setTrigSrcRegOp(quint16 regVal) const
{
    return RegOpVector{RegWrite(baseAddr() + REG_TRIG_SRC, regVal)};
}

mlink::RegOpVector TrigCtrlStatModule::setTimerPeriodRegOp(quint32 regVal) const
{
    return RegOpVector{RegWrite32(baseAddr() + REG32_TRIG_TIMER_PERIOD, regVal)};
}

mlink::RegOpVector TrigCtrlStatModule::setSelfTrigDelayRegOp(quint16 regVal) const
{
    return RegOpVector{RegWrite(baseAddr() + REG_TRIG_SELFTRIG_DELAY, regVal)};
}

mlink::RegOpVector TrigCtrlStatModule::setSelfTrigChMaskRegOp(quint64 regVal) const
{
    return RegOpVector{RegWrite64(baseAddr() + REG64_SELF_TRIG_MASK, regVal)};
}

mlink::RegOpVector TrigCtrlStatModule::resetEvNumAndStatsRegOp(quint64 evNumLoad) const
{
    RegOpVector regOp;
    regOp << RegWrite64(baseAddr() + REG64_TRIG_EVENT_NUM_LOAD, evNumLoad);
    regOp << RegWrite(baseAddr() + REG_TRIG_CSR, REG_TRIG_CSR_BIT_CNT_RESET); // reset stats
    regOp << RegWrite(baseAddr() + REG_TRIG_CSR, 0);
    return regOp;
}

mlink::RegOpVector TrigCtrlStatModule::getStatusRegOp() const
{
    RegOpVector v;
    v << RegWrite(baseAddr() + REG_TRIG_CSR, REG_TRIG_CSR_BIT_CNT_LOCK);
    v << RegRead(baseAddr() + REG_TRIG_QUEUE);
    v << RegRead(baseAddr() + REG_TRIG_BUSY_STATUS);
//    v << RegRead64(baseAddr + REG64_TRIG_CNT_ALL);
//    v << RegRead64(baseAddr + REG64_TRIG_CNT_CANDID);
//    v << RegRead64(baseAddr + REG64_TRIG_CNT_ACCEPTED);
    v << RegRead64(baseAddr() + REG64_TRIG_CUR_EVENT_NUM);
//    v << RegRead64(baseAddr + REG64_RUN_TIME_NS);
    v << RegWrite(baseAddr() + REG_TRIG_CSR, 0);
    return v;
}

TrigStatus TrigCtrlStatModule::getStatus(const mlink::RegOpVector &regRes, int posOffset) const
{
    checkRegResValid(&TrigCtrlStatModule::getStatusRegOp, regRes, posOffset);

    TrigStatus st;
    int p = posOffset+1;
    st.trigQueue = regRes[p].data&0xf; p += 1;
    st.busyStatus = regRes[p].data; p += 1;
//    st.allTrigCandid = QtMregDevice::get64(regRes, p); p += 4;
//    st.runTrigCandid = QtMregDevice::get64(regRes, p); p += 4;
//    st.acceptedTrig = QtMregDevice::get64(regRes, p); p += 4;
    st.curEvNum = QtMregDevice::get64(regRes, p); p += 4;
//    st.runTimeNs = QtMregDevice::get64(regRes, p); p += 4;;
    return st;
}
