#include "TtlIoModule.h"

#include "mregdev/qtmregdevice.h"

#include "TtlIoConfig.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

// Deserializers control
enum {
    REG_STATUS      = 0x00,
    REG_CFG_BASE    = 0x01,
};

void TtlIoModule::post_init()
{
    ttlCount = dev->regRead(baseAddr() + REG_STATUS);
}

TtlIoConfig TtlIoModule::readConfig() const
{
    RegOpVector regOp;
    const quint16 &cfgBase = baseAddr() + REG_CFG_BASE;
    for(int i=0; i<ttlCount; ++i)
        regOp << RegRead(cfgBase+i);

    const auto &regRes = dev->regOpExec(regOp);

    TtlIoConfig cfg;
    cfg.ttl_regs.resize(ttlCount);
    for(int i=0; i<ttlCount; ++i)
        cfg.ttl_regs[i].all = regRes[i].data;
    return cfg;
}

mlink::RegOpVector TtlIoModule::setConfig(const TtlIoConfig &cfg) const
{
    RegOpVector regOp;
    const quint16 &cfgBase = baseAddr() + REG_CFG_BASE;
    for(int i=0; i<cfg.ttl_regs.size(); ++i) {
        if(i>ttlCount)
            break;
        regOp << RegWrite(cfgBase+i, cfg.ttl_regs[i].all);
    }
    return regOp;
}
