#ifndef WRSTATUS_H
#define WRSTATUS_H

#include <QtCore>

typedef union {
    quint16 all;
    struct {
        quint32 sfpOk: 1;      // [0]
        quint32 linkUp: 1;     // [1]
        quint32 timeSync: 1;   // [2]
        quint32 unused: 13;    // [15:3]
    } bit;
} reg_wr_status_code;

typedef union {
    quint64 all;
    struct {
        quint32 flags:2;
        quint32 nSec:30;
        quint32 sec;
    } bit;
} reg_wr_time_tai;

struct WrStatus
{
    bool timeNull() const { return !timeCode.bit.nSec && !timeCode.bit.sec; }
    bool timeValid() const {return (timeCode.bit.flags == 0x2) && !timeNull(); }

    quint64 mSecsSinceEpoch() const {
        quint64 ms_sec = (quint64)seconds() * 1e3;
        quint64 ms_ns = (quint64)nanoseconds() / 1e6;
        return  ms_sec + ms_ns;
    }
    quint32 seconds() const { return timeCode.bit.sec; }
    quint32 nanoseconds() const { return timeCode.bit.nSec; }
    /// Check time is in sync with local time, ntpd should be running.
    int wrTimeCheck() const
    {
        auto wrTimeOffsetMs = mSecsSinceEpoch() - acquiredAtMs;
        if (acquireErrorMs < 5) {
            int wrTimeErrorMs = std::abs((int)(wrTimeOffsetMs + 500) % 1000 - 500); // milliseconds part
            // no check for seconds (offset changes every time leap second is added)
            if (wrTimeErrorMs > 50)
                return 2;
            if (wrTimeErrorMs > 10)
                return 1;
        }
        return 0;
    }
    WrStatus & operator-=(const WrStatus& st) {
        uptime     -= st.uptime;
        linkError  -= st.linkError;
        syncFail   -= st.syncFail;
        codeError  -= st.codeError;
        otherError -= st.otherError;
        return *this;
    }
    bool hasWrEpStatus = false;
    qint64 acquiredAtMs = 0;
    qint32 acquireErrorMs = 0;

    reg_wr_status_code statusCode = {0};    // Reg 0x60
    quint32 uptime = 0;                     // Reg 0x62
    reg_wr_time_tai timeCode = {0};         // Reg 0x64
    quint32 linkError = 0;                  // Reg 0x68
    quint32 syncFail = 0;                   // Reg 0x6A
    quint32 codeError = 0;                  // Reg 0x6C
    quint32 otherError = 0;                 // Reg 0x6E
};

#endif // WRSTATUS_H
