#include "WrStatusModule.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "WrStatus.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

// MCU SPI Slave
enum {
    REG_WR_EP_STATUS_CODE            = 0x00,
//    REG_WR_EP_STATUS_RESERVED        = 0x01,
    REG32_WR_EP_STATUS_UPTIME        = 0x02, // 32-bit
    REG64_WR_EP_STATUS_TIMECODE      = 0x04, // 64-bit
    REG32_WR_EP_STATUS_LINK_LOSS_CNT = 0x08, // 32-bit
    REG32_WR_EP_STATUS_SYNC_LOSS_CNT = 0x0A, // 32-bit
    REG32_WR_EP_STATUS_CODE_ERR_CNT  = 0x0C, // 32-bit
    REG32_WR_EP_STATUS_OTHER_ERR_CNT = 0x0E, // 32-bit
};

WrStatusModule::WrStatusModule(QtMregDevice * const dev)
    :BaseDeviceModule(dev)
{
}

void WrStatusModule::post_init()
{
    if(!isAvailable() && dev->hasSdbDevice(AFI_SDB_WR_ENDPOINT_STATE))
        addrRange = dev->getAddrRange(AFI_SDB_WR_ENDPOINT_STATE);
}

mlink::RegOpVector WrStatusModule::getStatusRegOp() const
{
    RegOpVector regOp;
    regOp << mlink::RegRead  (baseAddr() + REG_WR_EP_STATUS_CODE);
//    op << mlink::RegRead  (baseAddr() + REG_WR_EP_STATUS_RESERVED);
    regOp << mlink::RegRead32(baseAddr() + REG32_WR_EP_STATUS_UPTIME);
    regOp << mlink::RegRead64(baseAddr() + REG64_WR_EP_STATUS_TIMECODE);
    regOp << mlink::RegRead32(baseAddr() + REG32_WR_EP_STATUS_LINK_LOSS_CNT);
    regOp << mlink::RegRead32(baseAddr() + REG32_WR_EP_STATUS_SYNC_LOSS_CNT);
    regOp << mlink::RegRead32(baseAddr() + REG32_WR_EP_STATUS_CODE_ERR_CNT);
    regOp << mlink::RegRead32(baseAddr() + REG32_WR_EP_STATUS_OTHER_ERR_CNT);

    return regOp;
}

WrStatus WrStatusModule::readStatus() const
{
    const auto &regOp = getStatusRegOp();
    const auto &regRes = dev->regOpExec(regOp);
    if(regRes.size() != regOp.size())
        return WrStatus();

    const auto &wrSt = getStatus(regRes, 0);
    return wrSt;
}

WrStatus WrStatusModule::getStatus(const mlink::RegOpVector &result, int posOffset) const
{
    checkRegResValid(&WrStatusModule::getStatusRegOp, result, posOffset);

    int pos = posOffset;
    WrStatus wrStatus;
    wrStatus.hasWrEpStatus  = true;
    wrStatus.statusCode.all = result.at(pos).data; pos += 1;
    wrStatus.uptime         = QtMregDevice::get32(result, pos); pos += 2;
    wrStatus.timeCode.all   = QtMregDevice::get64(result, pos); pos += 4;
    wrStatus.linkError      = QtMregDevice::get32(result, pos); pos += 2;
    wrStatus.syncFail       = QtMregDevice::get32(result, pos); pos += 2;
    wrStatus.codeError      = QtMregDevice::get32(result, pos); pos += 2;
    wrStatus.otherError     = QtMregDevice::get32(result, pos); pos += 2;
    return wrStatus;
}
