#include "AdcBaselineCalibration.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "device-modules/AdcCore/AdcCoreModule.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;

const int CALIBRATE_LIMIT = 10e3;


AdcBaselineCalibration::AdcBaselineCalibration(AdcCoreModule *_adcCor, QtMregDevice * const _dev)
    :dev(_dev)
    ,adcCore(_adcCor)
{
}

void AdcBaselineCalibration::calibrateBaseline(QMap<int, double> *meanPtr, QMap<int, double> *rmsPtr)
{
    if(!meanPtr || !rmsPtr)
        return;
    auto &mean = *meanPtr;
    auto &rms = *rmsPtr;

    QMap<int, QVector<qint16>> adcVals;

    const auto &adcNCh = adcCore->getStatus().adcNCh;

    const RegOpVector &regOp = adcCore->getChannelDataRegOp();
    for(int ch=0; ch < adcNCh; ++ch) {
        adcVals[ch].reserve(CALIBRATE_LIMIT);
        mean[ch] = 0;
        rms[ch] = 0;
    }
    if(!dev->isConnected())
        return;

    int loop;
    for(loop = 0; loop<CALIBRATE_LIMIT; ++loop) {
        RegOpVector regRes = dev->regOpExec(regOp);
        if(regRes.size() != regOp.size()) {
            qWarning() << "error in getting current adc value while calibration";
            break;
        }
        for(int ch=0; ch < adcNCh; ++ch) {
            int data = regRes[ch].data;
            data -= 0x8000;
            adcVals[ch].push_back(data);
            mean[ch] += data;
        }
    }
    QStringList calReult;
    for(int ch=0; ch < adcNCh; ++ch) {
        double &chMean = mean[ch];
        if(loop)
            chMean /= loop;

        double &chRms=rms[ch];
        for (qint16 val : adcVals[ch]) {
            chRms += (chMean-val)*(chMean-val);
        }
        if(loop)
            chRms /= loop;
        chRms = sqrt(chRms);

        calReult << QString::number(chMean, 'f', 2);
    }
}
