#ifndef ADCSERDESCALIBRATION_H
#define ADCSERDESCALIBRATION_H

class AdcSpiBaseModule;
class AdcCoreModule;
class DeserializersControlModule;
class QtMregDevice;

#include <QVector>
#include <QMap>
#include <QObject>

#include "AdcClkStatus.h"

class AdcSerDesCalibration : public QObject
{
    Q_OBJECT
public:
    explicit AdcSerDesCalibration(AdcCoreModule *_adcCore,
                      AdcSpiBaseModule *_spiAdc,
                      DeserializersControlModule * _desCrtl,
                      QtMregDevice *dev = nullptr);
    virtual ~AdcSerDesCalibration() override {}
    void setGroupTwoDes(bool group = true) { groupTwoDes = group; } // option for TQDC

    AdcClkStatus getStatus() const { return delayStatus; }

    void calibrate(bool hardMode, const QString &resDir);

signals:
    void progressUpdated(int percentage);

private:
    void init();
    void generateSpiPatternSet();
    QMap<int, bool> patternTestByChannel();
    QMap<int, bool> TestAdcCustomPatternByChannel(const quint16 &pattern);
    inline int getNumberOfChannels() const { return desCnt*desCh; }
    double readTemp() const;
    QString getFirmwareVersionStr() const;
    void saveResultsToFile(const QString &resDir) const;

    AdcCoreModule * const adcCore;
    AdcSpiBaseModule * const adcSpi;
    DeserializersControlModule * const desCtrl;

    QVector<quint16> spiPatternSet;
    quint8 desCnt = 0;
    quint8 desCh=0;
    quint8 adcBits = 0;
    int clkDelay=0;
    int dataDelay=0;

    bool hardPatternTestEnabled = false;
    bool groupTwoDes = false;

    QMap<int, QMap<int, int> > passTestCh;
    QMap<int, QMap<int, int> > failTestCh;
    QMap<int, int> chPassTestCh;

    AdcClkStatus delayStatus;
};

#endif // ADCSERDESCALIBRATION_H
