#include "DacCalibration.h"

#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"

#include "DacHelper.h"
#include "device-modules/HitCore/HitCoreModule.h"
#include "device-modules/Ad5328/Ad5328Module.h"

using mlink::RegOpVector;

DacCalibration::DacCalibration(QtMregDevice * const dev, HitCoreModule * const hitCore, Ad5328Module * const thresholdModule)
    :dev(dev)
    ,hitCore(hitCore)
    ,thresholdMod(thresholdModule)
{
}

QMap<int, int> DacCalibration::calibrateDac()
{
    RegOpVector cmpStRegOp;
    for(int i=0; i<num_samples; ++i)
        cmpStRegOp << hitCore->getCmpStateRegOp();


    QMap<int, int> chThr;
    QMap<quint16, QVector<bool> > dacToChState;

    constexpr const int DAC_MAX_VALUE = DacConverter<Ad5328Module>::getDacMaxVal();
    quint16 dacVal = DAC_MAX_VALUE/2;
    minFound = maxFound = false;

    for(dacVal = DAC_MAX_VALUE/2;dacVal<DAC_MAX_VALUE && !maxFound; ++dacVal) {
        for(int ch=0; ch<nCh; ++ch)
            chThr[ch] = dacVal;
        dev->regOpExec(thresholdMod->setChDacDataRegOp(chThr));
        dacToChState[dacVal] = getChStates(cmpStRegOp);
    }
    if(!maxFound) {
        qCritical() << "DAC Calibration of" << dev->getIdent() << " failed  (max wasn't detected)";
//        return chThr;
    }

    for(dacVal = DAC_MAX_VALUE/2;dacVal>=0 && !minFound; --dacVal) {
        for(int ch=0; ch<nCh; ++ch)
            chThr[ch] = dacVal;
        dev->regOpExec(thresholdMod->setChDacDataRegOp(chThr));
        dacToChState[dacVal] = getChStates(cmpStRegOp);
        if(dacVal==0)
            break;
    }
    if(!minFound) {
        qCritical() << "DAC Calibration of" << dev->getIdent() << " failed (min wasn't detected)";
//        return chThr;
    }

    chThr.clear();
    for(int ch=0; ch<nCh; ++ch) {
        for(auto dacIt=dacToChState.cbegin(); dacIt!=dacToChState.cend(); ++dacIt) {
            if(!dacIt.value()[ch]) {
                if(dacIt.key() == 0) {
                    qCritical() << QString("DAC Calibration of %1 failed (min wasn't detected for ch:%2)")
                                   .arg(dev->getIdent()).arg(ch);
                }

                chThr[ch] = dacIt.key();
                break;
            }
        }
//        Q_ASSERT(chThr.contains(ch));
        if(!chThr.contains(ch)) {
            qCritical() << QString("DAC Calibration of %1 failed (max wasn't detected for ch:%2)")
                           .arg(dev->getIdent()).arg(ch);
            chThr[ch] = DAC_MAX_VALUE;
        }
    }
    Q_ASSERT(chThr.size() == nCh);
    return  chThr;
}

QVector<bool> DacCalibration::getChStates(const RegOpVector &cmpStRegOp)
{
    QVector<bool> chState(nCh, false);
    QVector<int> chCnt(nCh, 0);

    if(!dev->isConnected())
        return chState;

    RegOpVector regRes = dev->regOpExec(cmpStRegOp);
    for(int i=0; i<num_samples; ++i) {
        const quint64 &dacSt = QtMregDevice::get64(regRes, i*4);
        for(int ch=0; ch<nCh; ++ch){
            if(dacSt & (1<<ch))
                ++chCnt[ch];
        }
    }

    bool isMin = true, isMax = true;
    for(int ch=0; ch<nCh; ++ch) {
        const bool &st = chState[ch] = chCnt[ch]>(num_samples/2);
        isMin &= st;
        isMax &= !st;
    }
    minFound |= isMin;
    maxFound |= isMax;

    return chState;
}
