#ifndef DACHELPER_H
#define DACHELPER_H

#include <QtCore>

template<class T>
class DacConverter
{
public:
    static quint16 dacValue(int mV, int dacShift)
    {
        const double &vRef = getGainedVRef();
        int dacData = 1.*mV/vRef * getDacMaxVal()+dacShift;
        if(dacData < 0)
            return 0;
        else if(dacData > getDacMaxVal())
            return getDacMaxVal();
        else
            return dacData;
    }
    static quint16 dacValue(int mV)
    {
        return dacValue(mV, getDacDefaultZero());
    }

    static double dac_mV(quint16 dacData)
    {
        Q_ASSERT(dacData<=getDacMaxVal());
        return getGainedVRef()*((int)dacData-getDacDefaultZero())/getDacMaxVal();
    }

    static int getDacDefaultZero() { return 0; }
    static int getOutRange() { return getGainedVRef(); }
    static int getOutMax() { return getGainedVRef() + getOutShift(); }
    static int getOutMin() { return getOutShift(); }

    static bool getGain() { return false; }
    constexpr static inline quint16 getDacMaxVal() {
        constexpr const quint16 DAC_MAX_VALUE = (1<<12) - 1;
        return DAC_MAX_VALUE;
    }

protected:
    static inline double getVRef() { return 0; }

private:
    static int getOutShift() { return dac_mV(0); }
    static inline double getGainedVRef() {return getGain() ? getVRef()*2 : (getVRef());}
};

#endif // DACHELPER_H
