#include "FeLinkStatusDialog.h"
#include "ui_FeLinkStatusDialog.h"

#include <QTabWidget>
#include <QTableView>
#include <QHeaderView>

namespace {
constexpr const char* PROP_DEVICE_INDEX = "PROP_DEVICE_INDEX";
}

FeLinkStatusDialog::FeLinkStatusDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::FeLinkStatusDialog)
{
    ui->setupUi(this);
}

FeLinkStatusDialog::~FeLinkStatusDialog()
{
    delete ui;
}


void FeLinkStatusDialog::setDeviceList(const DeviceIndexEnabledMap &devices_)
{
    const DeviceIndexSet &newDeviceSet = devices_.getEnabled();
    if(readerDeviceSet == newDeviceSet)
        return;

    const auto &toDel = readerDeviceSet-newDeviceSet;
    readerDeviceSet = newDeviceSet;

    for(auto i : toDel)
        feLinkStatusMap.remove(i);
}

void FeLinkStatusDialog::setMasterList(const DeviceIndexSet &devices_)
{
    if(masterDevices == devices_)
        return;

    const auto &toDel = masterDevices-devices_;
    const auto &toAdd = devices_-masterDevices;
    masterDevices = devices_;

    removeMaster(toDel);

    for(auto index : toAdd) {
        addMaster(index);
    }
}

void FeLinkStatusDialog::setFeLinkStatus(const DeviceIndex &readerIndex, const QMap<int, FeLinkStatus> &st)
{
    if(!readerDeviceSet.contains(readerIndex))
        return;
    feLinkStatusMap[readerIndex] = st;
    updateMasterList();
}

void FeLinkStatusDialog::removeMaster(const DeviceIndexSet &setToDel)
{
    for(int i=0; i<ui->tabWidget->count();) {
        auto widget = ui->tabWidget->widget(i);
        const auto &devIndex = widget->property(PROP_DEVICE_INDEX).value<DeviceIndex>();
        if(setToDel.contains(devIndex)) {
            ui->tabWidget->removeTab(i);
            feLinkStatusMap.remove(devIndex);
            auto view = tableViewMap.take(devIndex);
            if(view)
                delete view;
            auto model = modelMap.take(devIndex);
            if(model)
                delete model;
        } else
            ++i;
    }
}

void FeLinkStatusDialog::addMaster(const DeviceIndex &index)
{
    auto view = new QTableView(ui->tabWidget);
    view->setProperty(PROP_DEVICE_INDEX, QVariant::fromValue(index));
    ui->tabWidget->addTab(view, index.getIdent());
    tableViewMap[index] = view;
    auto model = modelMap[index] = new FeLinkStatusModel(this);
    view->setModel(model);
    view->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

DeviceIndexSet FeLinkStatusDialog::getMasterSet(QMap<DeviceIndex, QMap<int, FeLinkStatus> > feLinkStatusMap)
{
    DeviceIndexSet masters;
    for(auto stMap : feLinkStatusMap) {
        for(auto st : stMap) {
            masters << st.getMasterIndex();
        }
    }
    return masters;
}

void FeLinkStatusDialog::updateMasterList()
{
    const auto &newMasterSet = getMasterSet(feLinkStatusMap);
    setMasterList(newMasterSet);

    QMap<DeviceIndex, QList<FeLinkStatus> > statusMap;
    for(auto v : feLinkStatusMap)
        for(auto st : v)
            statusMap[st.getMasterIndex()] << st;

    for(auto it=statusMap.begin(); it!=statusMap.end(); ++it) {
        auto modelIt = modelMap.find(it.key());
        if(modelIt != modelMap.end())
            (*modelIt)->setStatusList(it.value());
    }
}
