﻿#ifndef HISTMNGRWIDGET_H
#define HISTMNGRWIDGET_H

#include <QElapsedTimer>
#include <QMap>
#include <QMetaType>
#include <QWidget>
#include <QHash>

#include "HistMngrCfg.h"
#include "HistogramItem.h"
#include "DeviceCalibration.h"
#include "Histogramm.h"
#include "HistWorker.h"
#include "InlCheckResults.h"

namespace Ui {
class HistMngrWidget;
}

struct CompleteEvent;
class DeviceIndex;
struct DecodedData;
class QwtPlotCurve;
struct TdcChHit;
struct TqdcEvent;
class QTime;
class QTimer;
class QSemaphore;

struct InlStat
{
    QElapsedTimer time;
    QMap<int, double> count;
};

class HistMngrWidget : public QWidget
{
    Q_OBJECT
public:
    HistMngrWidget(QWidget *parent = nullptr);
    ~HistMngrWidget();
    static QVector<double> makeDNLCorrectionTable(const QVector<double> code_density);
    void updateInlCorr(QMap<DeviceIndex, DeviceCalibration> allCal);
    void updateInlCorr(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);
    QHash<HistKey, Histogramm> getHists() const { return histogramms; }

signals:
    void inlCheckUpdated(InlCheckResults res);
    void histConfigChanged(const HistMngrCfg config);
    void clearHist(const HistKey &hist_key) const;

private slots:
    void refreshCurrentPlot(bool force=false);

    void on_comboBoxHistGroup_currentIndexChanged(int i);
    void on_checkBoxHistGroupEn_toggled(bool checked);
    void on_comboBoxCurDev_currentIndexChanged(int i);
    void on_spinBoxCurCh_valueChanged(int i);
    void on_groupBoxStopCh_toggled(bool val);
    void on_comboBoxStopDev_currentIndexChanged(int i);
    void on_spinBoxStopCh_valueChanged(int ch);
    void on_spinBoxMwMin_valueChanged(int min);
    void on_spinBoxMwMax_valueChanged(int max);
    void on_radioButtonInl_toggled(bool checked);
    void on_radioButtonAdcAmplMin_toggled(bool checked);
    void on_pushButtonClear_clicked();
    void on_pushButtonClearAll_clicked();
    void on_groupBoxGlobalEnable_toggled(bool checked);
    void on_groupBoxFit_toggled(bool checked);
    void on_checkBoxLogY_toggled(bool checked);
    void on_checkBoxZeroBin_clicked(bool checked);

public slots:
    void hist_updated(QHash<HistKey, Histogramm> histogramms_, bool forceRefresh=false);
    void inlCheckRequest();
    void devListUpdated(QMap<DeviceIndex, int> dev_ch_offset, QHash<HistKey, Histogramm> histogramms_);
    void setHistConfig(HistMngrCfg config_);
    void taiReperUpdated(int taiReperSec);

private:
    void addDevice(const DeviceIndex &index);
    void addBaseQwtItems(const HistGroupCfg &group_config);
    void addDeviceChQwtItems(HistId id, const DeviceIndex &index, int ch);
    void removeExtraQwtItems(QHash<HistKey, QwtPlotItem *> HistMngrWidget::*field);
    void updateFit(const HistGroupCfg &histCfg, const Histogramm &hist) const;
    DeviceIndex getDevIndex(bool getStopDev, int idInComboBox=-1) const;
    DeviceIndex getCurDev(bool getStopDev=false) const;
    int getCurCh() const;
    void onCurGroupChanged(); // update GUI for current hist & return histCfg
    void updateDeviceComboBoxItems(bool stopDev);
    void updateHistGroupComboBoxItems();
    int getTotalChNum(const QMap<DeviceIndex, int> &devChOffsetOld);
    QVector<double> getInl(const TqdcEvent &ev, const TdcChHit &tdcEv) const;
    QVector<double> getInl(const DeviceIndex &index, int ch) const;
    void updateHistToolTip();
    void updateSpinBoxMaxCh(bool stopDev);
    void block_gui_signals(bool block);
    void enable_hist_group(bool enable);

    Ui::HistMngrWidget *ui;
    QList<QWidget*> block_signal_widgets_list;
    int block_signal_counter = 0;
    QElapsedTimer lastRefreshTime;
    QTimer * const refreshTimer;
    InlResults inlFromConfig;
    InlStat inlStat;
    QList<DeviceIndex> knownDevices;
    bool hide_zero_bin;

    HistMngrCfg config;
    QMap<DeviceIndex, int> dev_ch_offset;

    QHash<HistKey, QwtPlotItem *> histQwtItems; // HistogramItem
    QHash<HistKey, QwtPlotItem *> histQwtCurve; // QwtPlotCurve
    QHash<HistKey, Histogramm> histogramms;
    QString tdcCh_tool_tip;
    QString taiTime_tool_tip;
};

#endif // HISTMNGRWIDGET_H
