#ifndef TQDCANALYZER_H
#define TQDCANALYZER_H

#include <QObject>
#include <QByteArray>

#include "base/DeviceIndex.h"

#include "DeviceData.h"

class QSemaphore;
class QTimer;

class TQDCAnalyzer : public QObject
{
    Q_OBJECT
public:
    explicit TQDCAnalyzer(QSemaphore *_analyzerSem,
                          QSemaphore *_histSem,
                          QSemaphore *_writerSem,
                          QSemaphore *_zmqSem,
                          QObject *parent = nullptr);

signals:
    void addLog(QString) const;
    void deviceDataUpdated(const DecodedData &data,
                           bool writeDecodedEnabled) const;
    void deviceStatsUpdated(const AnalyzerStat &stats) const;
    void rawDataUpdated(const QStringList &data);
    void gotStopEvent(quint32 stopEvNum);

public slots:
    void dataAccuired(QByteArray newData);
    void reset();
    void requestRawData(int events);
    void setRawDataEveryEv(bool everyEv);
    void invertUpdated(QMap<DeviceIndex, QVector<bool> > _invertMap);
    void set_can_skip(bool skip);
    void prepareForStop(quint32 evNum);
    void startWrite(QString, bool rawData);
    void stopWrite();

private slots:
    void read();
    void sendStat();

private:
    void sendLog(QString s) const;
    quint32 readDevice(uint upperLimitToDevLength);
    bool findNextEventHeader();
    bool readEvent();
    void flushData();
    void decodeTdcBlock(const quint32* payloadPtr);
    void decodeAdcBlock(const quint32* payloadPtr);
    void decodePureAdcBlock(const quint32* payloadPtr);
    void decodeTtvxsBlock(const quint32* payloadPtr);
    void decodeDevStat(const quint32* payloadPtr);
    void printRawTxt(uint size, QString header=QString(), int shift=0);
    void printRawWord(const void *word, QString suffix=QString());

    QSemaphore * const analyzerSem;// free space in analyzer queue
    QSemaphore * const histSem;// free space in histogram queue
    QSemaphore * const writerSem;// free space in writer queue (raw or decoded)
    QSemaphore * const zmqSem;// free space in 0MQ queue

    QTimer * const statTimer;
    QTimer * const readTimer;
    QByteArray data;
    uint curPos;
    uint curEventStartPos;
    const MpdEventHdr *mpdEventHdr;
    const MpdDeviceHdr *mpdDeviceHdr;
    const MpdMStreamHdr *mstreamHdr;
    const TaiTimeBlock *tsTAI;
    const DataBlockHeader *dataBlockHdr; // seems will be deprecated

    quint32 lastEventNumAtPrevFrame;
    quint32 prevEventNum;

    bool previousProceed;
    bool enabled;

    AnalyzerStat analyzerStats;
    DecodedData decodedData;

    CompleteEvent curCompleteEv;
    TqdcEvent curTqdcEv;
    TtvxsEvent curTtvxsEv;
    DevStat curTdcStat;
    TdcChHit curTdcHit;
    AdcChHit curAdcHit;
    bool signalInverted;
    bool writeDecodedEnabled;
    QStringList rawDataTxt;
    int rawDataEvents; // number events that should be listed (tab rawData)
    bool printBadEvent;
    bool rawDataEveryEv;
    QStringList lastErrStr;
    int clear_err_stack;
    QMap<DeviceIndex, QVector<bool> > invertMap;
    bool can_skip;
    bool lookForStop;
    quint32 stopEvNum;
    QElapsedTimer noHistSpace;
    QElapsedTimer noWriterSpace;
    QElapsedTimer noZmqSpace;
    const bool adcRawDataSigned = true;
};

#endif // TQDCANALYZER_H
