#ifndef CLIENTMGRFSM_H
#define CLIENTMGRFSM_H

#include "fsm/BaseFsm.h"
#include <QMap>
#include <QSet>

struct ClientsFsmInput : public BaseFsmInput
{
    QSet<int> clientSet;
    QMap<int, FsmState> clientStates;
};

class ClientManagerFsm : public BaseFsm
{
    Q_OBJECT
public:
    explicit ClientManagerFsm(QObject *parent = nullptr);

public slots:
    void setClientList(const QSet<int> &newList,
                       const QMap<int, FsmState> &clientStates);
    void setClientState(int id, const FsmState &clState);

protected slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline ClientsFsmInput *getInput() {
        return static_cast<ClientsFsmInput*>(BaseFsm::getInput());
    }
    inline const ClientsFsmInput *getInput() const {
        return static_cast<const ClientsFsmInput*>(BaseFsm::getInput());
    }

private:
    bool allClientsInState(const FsmState &state) const;
    bool isAnyClientInError() const;
};

QDebug operator<<(QDebug dbg, const ClientsFsmInput &state);

#endif // CLIENTMGRFSM_H
