#include "FeLinkDeviceUtil.h"

#include "regio/regop.h"
#include "mregdev/qtmregdevice.h"

#include "FeLinkStatus.h"
#include "FeLinkRegs.h"

using mlink::RegOpVector;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;
using mlink::RegWrite;
//using mlink::RegWrite32;
//using mlink::RegWrite64;


namespace {
quint32 get32(const mlink::RegOpVector &rx, int startPos)
{
    return QtMregDevice::get32(rx, startPos);
}
quint64 get64(const mlink::RegOpVector &rx, int startPos)
{
    return QtMregDevice::get64(rx, startPos);
}
}


mlink::RegOpVector getFeLinkStatusROV(quint16 base, int ch)
{
    RegOpVector regOp;

    base += ch*0x80;

    regOp << RegWrite(base+REG_FELINK_CTRL, REG_FELINK_CTRL_BIT_CNT_LOCK);
    regOp << RegRead(base+REG_FELINK_STATUS);
    regOp << RegRead(base+REG_FELINK_RX_BITSLIDE);
    regOp << RegRead(base+REG_FELINK_REMOTE_VER);
    regOp << RegRead64(base+REG64_FELINK_REMOTE_ID);
    regOp << RegRead(base+REG_FELINK_SYNC_RESEND_TIME);
    regOp << RegRead(base+REG_FELINK_TIME_FRAME_TIMEOUT);
    regOp << RegRead32(base+REG32_FELINK_TAI_NSEC);
    regOp << RegRead64(base+REG64_FELINK_TAI_SEC);
    regOp << RegRead32(base+REG32_FELINK_LINK_STATE_TIME);
    regOp << RegRead32(base+REG32_FELINK_CNT_RX_CODE_ERROR);
    regOp << RegRead32(base+REG32_FELINK_CNT_RX_CRC_ERROR);
    regOp << RegRead32(base+REG32_FELINK_CNT_TIME_FRAMES_SND);
    regOp << RegRead32(base+REG32_FELINK_CNT_TRIG_FRAMES_SND);
    regOp << RegRead32(base+REG32_FELINK_CNT_TIME_FRAMES_RCV);
    regOp << RegRead32(base+REG32_FELINK_CNT_TRIG_FRAMES_RCV);
    regOp << RegRead32(base+REG32_FELINK_CNT_UNKNOWN_FRAMES);
    regOp << RegWrite(base+REG_FELINK_CTRL, 0x0);

    return regOp;
}

FeLinkStatus getFeLinkStatus(const mlink::RegOpVector &regRes, int shift)
{
    FeLinkStatus st;
    int pos = shift+1;
    st.status = FeLinkStatusReg(regRes[pos].data); pos += 1;
    st.RX_Bitslide = regRes[pos].data; pos += 1;
    st.remoteVer = FeLinkProtocolVersion(regRes[pos].data); pos += 1;
    st.remoteIndex = DeviceIndex(static_cast<DeviceId>((get64(regRes, pos)>>48)&0xFF),
                                 get64(regRes, pos)&0xffffFFFFffff); pos += 4;
    st.REG_FELINK_SYNC_RESEND_TIME = regRes[pos].data; pos += 1;
    st.REG_FELINK_TIME_FRAME_TIMEOUT = regRes[pos].data; pos += 1;
    st.tai_valid = get32(regRes, pos) & 0x2;
    st.tai_nsec = get32(regRes, pos)>>2; pos += 2;
    st.tai_sec = get64(regRes, pos); pos += 4;

    st.Current_link_state_time = get32(regRes, pos); pos += 2;

    st.RX_code_error_counter = get32(regRes, pos); pos += 2;
    st.RX_CRC_error_counter = get32(regRes, pos); pos += 2;
    st.Time_frames_sent_counter = get32(regRes, pos); pos += 2;
    st.Trigger_frames_sent_counter = get32(regRes, pos); pos += 2;
    st.Time_frames_received_counter = get32(regRes, pos); pos += 2;
    st.Trigger_frames_received_counter = get32(regRes, pos); pos += 2;
    st.Unknown_frames_received_counter = get32(regRes, pos); pos += 2;

    return st;
}
