#include "FeLinkStatus.h"

#include <QDebug>

QDebug operator<<(QDebug dbg, const FeLinkStatus &c)
{
    dbg
            << "status:" << c.status << "; "
            << "RX_Bitslide:" << c.RX_Bitslide << "; "

            << "remoteVer:" << c.remoteVer << "; "
            << "remoteIndex:" << c. remoteIndex << "; "
            << "REG_FELINK_SYNC_RESEND_TIME:" << c.REG_FELINK_SYNC_RESEND_TIME << "; "
            << "REG_FELINK_TIME_FRAME_TIMEOUT:" << c.REG_FELINK_TIME_FRAME_TIMEOUT << "; "

            << "tai_valid:" << c.tai_valid << "; "
            << "tai_nsec:" << c.tai_nsec << "; "
            << "tai_sec:" << c.tai_sec << "; "

            << "Current_link_state_time:" << c.Current_link_state_time << "; "

            << "RX_code_error_counter:" << c.RX_code_error_counter << "; "
            << "RX_CRC_error_counter:" << c.RX_CRC_error_counter << "; "
            << "Time_frames_sent_counter:" << c.Time_frames_sent_counter << "; "
            << "Trigger_frames_sent_counter:" << c.Trigger_frames_sent_counter << "; "
            << "Time_frames_received_counter:" << c.Time_frames_received_counter << "; "
            << "Trigger_frames_received_counter:" << c.Trigger_frames_received_counter << "; "
            << "Unknown_frames_received_counter:" << c.Unknown_frames_received_counter
               ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkStatusReg &c)
{
    dbg
            << "PHY_level_sync_accure:" << c.data.bit.PHY_level_sync_accure << "; "
            << "rx_sync:" << c.data.bit.rx_sync << "; "
            << "remote_rx_sync:" << c.data.bit.remote_rx_sync << "; "
            << "softpll_locked:" << c.data.bit.softpll_locked << "; "
            << "link_ok:" << c.data.bit.link_ok << "; "
            << "is_master:" << c.data.bit.is_master
               ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkProtocolVersion &c)
{
    dbg
            << "maj:" << c.maj << "; "
            << "min:" << c.min
               ;

    return dbg;
}

bool FeLinkStatusReg::operator==(const FeLinkStatusReg &r) const
{
    return data.all == r.data.all;
}

bool FeLinkStatusReg::allOk() const
{
    return data.bit.PHY_level_sync_accure
            && data.bit.rx_sync
            && data.bit.remote_rx_sync
            && (data.bit.softpll_locked || isMaster())
            && data.bit.link_ok;
}

QString FeLinkStatusReg::getInfoStr() const
{
    QStringList list;
    if(!data.bit.PHY_level_sync_accure)
        list << "PHY_level_sync_accure";
    if(!data.bit.rx_sync)
        list << "rx_sync";
    if(!data.bit.remote_rx_sync)
        list << "remote_rx_sync";
    if(!data.bit.softpll_locked && !isMaster())
        list << "softpll_locked";

    static bool warn_incorrect_link_ok = true;
    if(warn_incorrect_link_ok && data.bit.link_ok != list.isEmpty()) {
        warn_incorrect_link_ok = false;
        qWarning() << __PRETTY_FUNCTION__ << "warn_incorrect_link_ok status:" << *this;
    }

    for(auto &v : list) v.append(" fail");

    if(data.bit.link_ok) {
        list << "link_ok";
    } else {
        if(list.isEmpty())
            list << "link_fail";
    }

    return list.join(",");
}

bool FeLinkProtocolVersion::operator==(const FeLinkProtocolVersion &r) const
{
    return
            this->maj == r.maj &&
            this->min == r.min;
}

QString FeLinkProtocolVersion::getInfoStr() const
{
    return QString("%1.%2").arg(maj).arg(min);
}

bool FeLinkStatus::operator==(const FeLinkStatus &r) const
{
    return
            this->readerIndex == r.readerIndex &&
            this->readerCh == r.readerCh &&
            this->status == r.status &&
            this->RX_Bitslide == r.RX_Bitslide &&

            this->remoteVer == r.remoteVer &&
            this->remoteIndex == r. remoteIndex &&
            this->REG_FELINK_SYNC_RESEND_TIME == r.REG_FELINK_SYNC_RESEND_TIME &&
            this->REG_FELINK_TIME_FRAME_TIMEOUT == r.REG_FELINK_TIME_FRAME_TIMEOUT &&

            this->tai_valid == r.tai_valid &&
            this->tai_nsec == r.tai_nsec &&
            this->tai_sec == r.tai_sec &&

            this->Current_link_state_time == r.Current_link_state_time &&

            this->RX_code_error_counter == r.RX_code_error_counter &&
            this->RX_CRC_error_counter == r.RX_CRC_error_counter &&
            this->Time_frames_sent_counter == r.Time_frames_sent_counter &&
            this->Trigger_frames_sent_counter == r.Trigger_frames_sent_counter &&
            this->Time_frames_received_counter == r.Time_frames_received_counter &&
            this->Trigger_frames_received_counter == r.Trigger_frames_received_counter &&
            this->Unknown_frames_received_counter == r.Unknown_frames_received_counter;
}
