#ifndef FELINKSTATUS_H
#define FELINKSTATUS_H

#include <QMetaType>

#include "base/DeviceIndex.h"

class QDebug;

typedef union {
    quint16 all;
    struct {
        unsigned int PHY_level_sync_accure: 1;  // [0]
        unsigned int rx_sync: 1;                // [1] // remote device PHY level sync
        unsigned int remote_rx_sync: 1;         // [2] // remote device verified our PHY level sync
        unsigned int softpll_locked: 1;         // [3] // only for time receiving side
        unsigned int link_ok: 1;                // [4]
        unsigned int unused1: 10;               // [5-14]
        unsigned int is_master: 1;              // [15]
    } bit;
} reg_fe_link_status;

struct FeLinkStatusReg
{
    FeLinkStatusReg() = default;
    FeLinkStatusReg(quint16 v) :
        data{v}
    {}
    bool operator==(const FeLinkStatusReg &r) const;
    bool operator!=(const FeLinkStatusReg &r) const { return !operator==(r); }
    bool isEmpty() const { return data.bit.PHY_level_sync_accure; }
    bool isMaster() const { return data.bit.is_master; }
    bool allOk() const;
    QString getInfoStr() const;

    reg_fe_link_status data;
};
QDebug operator<<(QDebug dbg, const FeLinkStatusReg &c);

struct FeLinkProtocolVersion
{
    FeLinkProtocolVersion() = default;
    FeLinkProtocolVersion(quint16 v) :
        maj ((v>>4) & 0xF),
        min ( v     & 0xF)
    {}
    bool operator==(const FeLinkProtocolVersion &r) const;
    bool operator!=(const FeLinkProtocolVersion &r) const { return !operator==(r); }

    QString getInfoStr() const;

    quint8 maj=0;
    quint8 min=0;
};
QDebug operator<<(QDebug dbg, const FeLinkProtocolVersion &c);

struct FeLinkStatus
{
    bool isEmpty() const { return status.isEmpty(); }
    bool isMaster() const { return status.isMaster(); }
    DeviceIndex getMasterIndex() const { return isMaster()? readerIndex : remoteIndex; }
    DeviceIndex getSlaveIndex() const { return !isMaster()? readerIndex : remoteIndex; }
    bool operator==(const FeLinkStatus &r) const;
    bool operator!=(const FeLinkStatus &r) const { return !operator==(r); }

    DeviceIndex readerIndex;
    int readerCh;

    FeLinkStatusReg status;
    quint16 RX_Bitslide;

    FeLinkProtocolVersion remoteVer;
    DeviceIndex remoteIndex;
    quint16 REG_FELINK_SYNC_RESEND_TIME; // R/W
    quint16 REG_FELINK_TIME_FRAME_TIMEOUT; // R/W

    bool tai_valid;
    quint32 tai_nsec;
    quint64 tai_sec;

    quint32 Current_link_state_time; // miliseconds

    quint32 RX_code_error_counter;
    quint32 RX_CRC_error_counter;
    quint32 Time_frames_sent_counter;
    quint32 Trigger_frames_sent_counter;
    quint32 Time_frames_received_counter;
    quint32 Trigger_frames_received_counter;
    quint32 Unknown_frames_received_counter;
};
Q_DECLARE_METATYPE(FeLinkStatus);
QDebug operator<<(QDebug dbg, const FeLinkStatus &c);

#endif // FELINKSTATUS_H
