#include "HwipErrorHdlr.h"

#include "regio/regop.h"
#include "qtmregdevice.h"

//Error Counters
enum {
    REG64_HWIP_LINK_DOWN = 0x1000,
    REG64_REGIO_ERR = 0x1004,
    REG64_SFP_ERR = 0x1008,
    REG64_HWIP_ERR = 0x100C,
    REG64_TX_ERR = 0x1010,
    REG64_RX_ERR = 0x1014,
    REG64_TX_DISABLE = 0x1018,
};

using mlink::RegOpVector;
using mlink::RegWrite;
using mlink::RegWrite32;
using mlink::RegWrite64;
using mlink::RegRead;
using mlink::RegRead32;
using mlink::RegRead64;

//HwipError HwipErrorHdlr::read(QtMregDevice &dev)
//{
//    const auto regOp = getRegOp();
//    auto regRes = dev.regOpExec(regOp);
//    if(regRes.size() != regOp.size()) {
//        return HwipError();
//    }
//    HwipError err;
//    readRegOp(regOp, 0, &err);
//    return err;
//}

mlink::RegOpVector HwipErrorHdlr::getRegOp()
{
    RegOpVector regOp;
    regOp << RegRead64(REG64_HWIP_LINK_DOWN);
    regOp << RegRead64(REG64_REGIO_ERR);
    regOp << RegRead64(REG64_SFP_ERR);
    regOp << RegRead64(REG64_HWIP_ERR);
    regOp << RegRead64(REG64_TX_ERR);
    regOp << RegRead64(REG64_RX_ERR);
    regOp << RegRead64(REG64_TX_DISABLE);

    return regOp;
}

void HwipErrorHdlr::readRegOp(const mlink::RegOpVector &regRes, int errCntPos, HwipError *errCnt)
{
    auto get64 = [](const mlink::RegOpVector &a, int b)
    { return QtMregDevice::get64(a,b); };
    errCnt->hwipLinkDown = get64(regRes, errCntPos);
    errCnt->regIoErr=get64(regRes, errCntPos+4);
    errCnt->sfpErr= get64(regRes, errCntPos+8);
    errCnt->hwIpErr=get64(regRes, errCntPos+12);
    errCnt->txErr=get64(regRes, errCntPos+16);
    errCnt->rxErr=get64(regRes, errCntPos+20);
    errCnt->txDis=get64(regRes, errCntPos+24);
}
