#include "SdbReader.h"

#include <QDebug>
#include <vector>

#include "mregdev/mregdevice.h"

#include "sdb_util.h"

enum {SDB_BASE = 0x7C00};

sdb_t SdbReader::read(mlink::MregDevice *dev)
{
    sdb_t sdb;

    sdb_interconnect &ic = sdb.ic;
    std::vector<int> registers;
    for(uint i=0; i<(sizeof(sdb_interconnect)/sizeof(quint16)); ++i)
        registers.push_back(SDB_BASE+i);

    std::vector<quint16> ic_data = dev->regReadBlk(registers);
    memcpy(&ic, &ic_data[0], sizeof(sdb_interconnect));
    sdb_interconnect_fix_endian(&ic);
    if(ic.sdb_magic == SDB_MAGIC) {
        static bool single_warn = true;
        if(ic.sdb_records > MAX_SDB_DEVICE_COUNT && single_warn) {
            qWarning() << QString("sdb_records(%1) is larger than MAX_SDB_DEVICE_COUNT(%2)")
                          .arg(ic.sdb_records).arg(MAX_SDB_DEVICE_COUNT);
            single_warn = false;
        }

        uint device_count = std::min(ic.sdb_records, MAX_SDB_DEVICE_COUNT) - 1;
        const uint sdb_size = sizeof(sdb_interconnect) + device_count * sizeof(struct sdb_device);
        registers.clear();
        for(uint i=(sizeof(sdb_interconnect)/sizeof(quint16)); i<(sdb_size/sizeof(quint16)); ++i)
            registers.push_back(SDB_BASE+i);

        std::vector<quint16> dev_data = dev->regReadBlk(registers);
        for(uint i=0; i<device_count; ++i) {
            sdb_device &dev = sdb.device[i];
            memcpy(&dev, &dev_data[i*(sizeof(sdb_device)/sizeof(quint16))], sizeof(sdb_device));
            sdb_device_fix_endian(&dev);
        }
    }
    return sdb;
}
