#include "SdbTypes.h"

#include <QDebug>

namespace {
QString nameToStr(const uint8_t *name)
{
    char nameStr[20];
    memset(nameStr, 0, 20);
    memcpy(nameStr, name, 19);
    return QString::fromLatin1(nameStr);
}

QDebug operator<<(QDebug dbg, const sdb_product &product)
{
    dbg.noquote();
    dbg << QString("device_id 0x%1;").arg(product.device_id, 0, 16);
    dbg << QString("vendor_id: 0x%1;").arg(product.vendor_id, 0, 16);
    dbg << QString("version: 0x%1;").arg(product.version, 0, 16);
    dbg << QString("date: 0x%1;").arg(product.date, 0, 16);
    dbg << QString("name: %1; ").arg(nameToStr(product.name));
    dbg << QString("record_type: 0x%1; ").arg(product.record_type, 0, 16);
    return dbg;
}
}

QDebug operator<<(QDebug dbg, const sdb_interconnect &ic)
{
    dbg.noquote();
    dbg << QString("sdb_magic: 0x%1;").arg(ic.sdb_magic, 0, 16);
    dbg << QString("sdb_records: %1;").arg(ic.sdb_records);
    dbg << QString("device_id: 0x%1;").arg(ic.sdb_component.product.device_id, 0, 16);
    dbg << QString("name: %1;").arg(nameToStr(ic.sdb_component.product.name));
    return dbg;
}

QDebug operator<<(QDebug dbg, const sdb_device &dev)
{
    dbg.noquote();
    Q_ASSERT(dev.abi_class == 0);
    dbg << QString("ver: %1.%2;").arg(dev.abi_ver_major).arg(dev.abi_ver_minor);
    dbg << QString("bus: %1;").arg(dev.bus_specific==BUS_REGIO?"regio":"memio");

    const auto &component = dev.sdb_component;
//    dbg << "component:";
    dbg << QString("addr(16-bit): 0x%1-0x%2;")
           .arg(component.addr_first>>1, 4, 16, QChar('0'))
           .arg(component.addr_last>>1,  4, 16, QChar('0'));

    dbg << "product:" << component.product;

    return dbg;
}

QDebug operator<<(QDebug dbg, const sdb_t &sdb)
{
    dbg.noquote();
    dbg << "sdb_interconnect:{" << sdb.ic << "};";
    if(sdb.isValid()) {
        for(int i=0; i<sdb.ic.sdb_records-1; ++i) {
            dbg << QString("\n{device_%1:").arg(i)
                << sdb.device[i] << "};";
        }
    } else {
        dbg << "SDB sdb_interconnect is invalid";
    }

    return dbg;
}
