#include "MongoCredentialsFile.h"

#include "base/Globals.h"
#include "mongoqt/MongoHost.h"

#include <QCoreApplication>
#include <QSettings>


MongoCredentialsFile::MongoCredentialsFile()
{}

MongoHost MongoCredentialsFile::get_host(QString environment)
{
    MongoHost cred;
    QSettings *settings = new QSettings(QCoreApplication::organizationName(),
                             Globals::common_settings_app_name,
                             nullptr);

    QString path = environment + "/mongodb/";
    QString hostname = settings->value(path + "hostname").toString();
    QString port = settings->value(path + "port").toString();
    QString username = settings->value(path + "username").toString();
    QString password = settings->value(path + "password").toString();
    QString default_db = settings->value(path + "default_db").toString();

    cred.environment = environment;
    if(!hostname.isEmpty())
        cred.hostname = hostname;
    else
        qCritical() << "Unable to find 'hostname' in section"
                    << environment << "in" << settings->fileName();

    if(!port.isEmpty())
        cred.port = port;
    if(!username.isEmpty())
        cred.username = username;
    if(!password.isEmpty())
        cred.password = password;
    if(!default_db.isEmpty())
        cred.default_db = default_db;

    settings->beginGroup(path);
    settings->beginGroup("options");
    QStringList keys = settings->childKeys();
    for(int i=0; i<keys.size(); ++i){
        cred.options[keys[i]] = settings->value(keys[i]).toString();
    }
    settings->endGroup();
    settings->endGroup();
    return cred;
}

QList<MongoHost> MongoCredentialsFile::get_all_hosts()
{
    QSettings *settings = new QSettings(QCoreApplication::organizationName(),
                             Globals::common_settings_app_name,
                             nullptr);

    QList<MongoHost> creds_list;
    QStringList groups = settings->childGroups();
    for(int i=0; i<groups.size(); ++i){
        creds_list.append(get_host(groups[i]));
    }
    return creds_list;
}
