
#include "mongo_access.h"
#include "MongoDbLogger.h"

#include <cassert>
#include <mongocxx/uri.hpp>
#include <mongocxx/exception/exception.hpp>
#include <QDebug>

mongo_access::mongo_access()
    : _instance(bsoncxx::stdx::make_unique<mongocxx::instance>(
                    bsoncxx::stdx::make_unique<MongoDbLogger>()
                    ))
{}

mongo_access::~mongo_access()
{
    destroyed = true;
}

mongo_access& mongo_access::instance()
{
    try{
        static mongo_access instance;
        assert(!instance.destroyed);
        return instance;
    } catch (const std::exception& xcp){
        qWarning() << xcp.what();
    }
}

bool mongo_access::configure(const std::string &uri)
{
    try{
        _pool = bsoncxx::stdx::make_unique<mongocxx::pool>(mongocxx::uri(uri));
    } catch (const std::exception& xcp){
        qWarning() << xcp.what();
        return false;
    }
    return true;
}

mongocxx::pool::entry mongo_access::get_connection()
{
    try{
        assert(_pool);
        return _pool->acquire();
    } catch (const std::exception& xcp){
        qWarning() << xcp.what();
    }
}

bsoncxx::stdx::optional<mongocxx::pool::entry> mongo_access::try_get_connection()
{
    try{
        assert(_pool);
        return _pool->try_acquire();
    } catch (const std::exception& xcp){
        qWarning() << xcp.what();
    }
}
