#ifndef REMOTECONTROLSERVER_H
#define REMOTECONTROLSERVER_H

#include <QObject>

#include "fsm/FsmState.h"
#include "RemoteControlServerStatus.h"
#include "remote-control-server/types.h"

class ProgramDescription;
class RcRunId;
class RemoteControlTlvServer;
struct TypeLengthHeader;

class RemoteControlServer : public QObject
{
    Q_OBJECT
public:
    explicit RemoteControlServer(QObject *parent, quint16 preferedTcpPort=0, QString _descr="");
    virtual ~RemoteControlServer() override;

public slots:
    void startListen(bool state);
    void sendState(const QString &state);
    void onFsmStateChange(const FsmState &state);
    void onRunError(const RcRunId &runId, QString runError=QString());
    void onRunWarning(const RcRunId &runId, QString warnStr);
    void onBackPressure(const RcRunId &runId, bool backPressure);
    void onVmeFlag(bool hasVme);
    void config_transfer(QByteArray data) const;
    void changeProgramDescription(const ProgramDescription &pd);
    void evNumCheckResponse(int checkIndex, const EvNumSet &evNums) const;

signals:
    void startRequest(const RcRunId &runId, quint64 startEvNum=0);
    void stopRequest(const RcRunId &runId, quint64 stopEvNum=0);
    void timeToStart(QString timeToStart);
    void stateRequest();
    void evNumCheckRequest(int checkIndex);
    void connectionStatusUpdated(const RemoteControlServerStatus &) const;

private slots:
    void connectionAccepted();
    void tlvDataReceived(const TypeLengthHeader &hdr, const QByteArray &data);

private:
    void sendDescription() const;
    void sendFsmState() const;
    void send_command_message(QString mess) const;
    void command_line_parse(QByteArray data);

    QString descr;
    RemoteControlTlvServer * const server;
    FsmState fsmState;
};

#endif // REMOTECONTROLSERVER_H
